ROM00000 DAD 0
ROM20000 DAD 20000             ! base address for the 2nd system ROM
ROM40000 DAD 40000             ! base address for the 3rd system ROM
ROM60000 DAD 60000             ! base adddress for bank-selected ROMs
!ROM60014 DAD 60014:340         ! address of PWO routine for Service ROM
ROM60014 DAD 60014
START1   DAD 64                ! address to resume PWO to normal BASIC

ALPHASIZ EQU 10000             ! LENGTH OF CRT ALPHA RAM

         ABS 60000
! ************************************************************
! * HP-85 SERVICE ROM DISASSEMBLY
! *
! * Most of the ROM is now disassembled, but there are certainly
! * mistakes and misunderstandings. Such is the nature of the
! * task, and were I to go through the code carefully again,
! * I imagine that many things would become clear that were
! * obfuscated during the first pass. But, for now, this is
! * where it stands.
! *           Everett Kaser - May 5, 2020
! *
! ************************************************************
! * GINTEN (177400) ANY WRITE TO THIS ADDRESS ENABLES GLOBAL INTERRUPTS
! * GINTDS (177401) ANY WRITE TO THIS ADDRESS DISABLES GLOBAL INTERRUPTS
! ************************************************************
! * KEYSTS (177402) READ:
! *   BIT0 = 1 DEVICE (KBD) INTERRUPTS ENABLED
! *   BIT1 = 1 KEY IS DOWN
! *   BIT2 =
! *   BIT3 = 1 SHIFT IS DOWN
! *   BIT4 =
! *   BIT5 =
! *   BIT6 =
! *   BIT7 = 1 GLOBAL INTERRUPTS (GINTEN/GINTDS) ENABLED
! * KEYSTS (177402) WRITE:
! *   BIT0 = 1 ENABLE KEY SCANNING
! *   BIT1 = 1 DISABLE KBD INTERRUPTS
! *   BIT2 =
! *   BIT3 =
! *   BIT4 =
! *   BIT5 = 1 BEEPER ON, = 0 BEEPER OFF
! *   BIT6 = 1 STD BEEP (1.2 Khz) ON, 0 = STD BEEP OFF
! *   BIT7 = 1 TOGGLE CAPSLOCK FLIP-FLOP
! * KEYCOD (177403) READ: KEY CODE OF PRESSED KEY
! * KEYCOD (177403) WRITE: WRITING 001 RESTARTS KBD SCANNER
! ************************************************************
! * CRTSAD (177404, WRITE-ONLY) SETS THE DISPLAY START ADDRESS IN CRT RAM
! * CRTBAD (177405, WRITE-ONLY) SETS THE "WRITE ADDRESS" FOR WRITING CHARS/BYTES TO CRT RAM
! * CRTSTS (177406) READ:
! *   BIT0 = 1 WHEN DATA READY (FROM READ REQUEST)
! *   BIT1 = 1 DISPLAY TIME, = 0 RETRACE TIME
! *   BIT2 =
! *   BIT3 =
! *   BIT4 =
! *   BIT5 = ??? (see 67205) ???
! *   BIT6 =
! *   BIT7 = 1 IF BUSY, 0 IF NOT BUSY
! * CRTSTS (177406) WRITE:
! *   BIT0 = 1 TO ISSUE READ REQUEST (CRT PUTS CHAR IN CRTDAT)
! *   BIT1 = 1 TO WIPE-OUT CRT, = 0 TO UNWIPE CRT
! *   BIT2 = 1 TO POWER-DOWN CRT, = 0 TO POWER-UP CRT
! *   BIT3 =
! *   BIT4 =
! *   BIT5 =
! *   BIT6 =
! *   BIT7 = 1 FOR GRAPHICS, = 0 FOR ALPHA
! * CRTDAT (177407) READS/WRITES TO ACCESS CRT RAM WHEN CRTSTS-BIT7==0
! ************************************************************
! * TAPSTS (177410) READ:
! *   0 =1 IF CARTRIDGE PRESENT
! *   1 =1 IF STALL
! *   2 =1 IF ILIM (CURRENT OVER LIMIT)
! *   3 =1 IF WRITE ENABLED, 0=WRITE PROTECT
! *   4 =1 IF HOLE (OR END OF TAPE)
! *   5 =1 IF NOT GAP, =0 IF GAP
! *   6 =1 IF SAW MOTOR TACHOMETER TICK
! *   7 =1 IF READY
! * TAPSTS (177410) WRITE:
! *   0 TRACK NUMBER (0 OR 1)
! *   1 =1 TO POWER UP TAPE, =0 TO POWER DOWN
! *   2 =1 MOTOR ON, =0 MOTOR OFF
! *   3 =1 IF MOTOR FWD (TO END), =0 IF BACK (TO START)
! *   4 =1 HIGH SPEED, =0 LOW SPEED
! *   5 =1 TO WRITE DATA
! *   6 =1 TO WRITE SYNC
! *   7 =1 TO WRITE GAP
! * TAPDAT (177411) READ/WRITE TAPE DATA WHEN TAPSTS-BIT7==1
!
! NOTE: notational differences - track0==track A, track1==track B.
!
! TAPSTS BIT0 (CART) of TAPSTS is "double buffered."
! If you read TAPSTS and BIT0=0, that means that EITHER there
! is NOT a tape cartridge in OR that the cartridge has been
! ejected and then a tape cartridge inserted. To know which
! case is true, you must read TAPSTS a second time. If, on
! the second read, TAPSTS BIT0=0, then there is NOT a cartridge
! in the drive, but if BIT0=1, then it's a NEW tape cartridge
! (an "ejected, inserted" cycle has occured).
!
! But in the case that TAPSTS BIT0=1 on the first read, then
! you know a cartridge IS in the drive, and has NOT been
! ejected and inserted since the previous read of TAPSTS.
!
! TAPSTS bits 1 (STALL), 2 (ILIM), and 6 (TACH) reflect
! whether those situations have occured since the previous
! read of TAPSTS. Reading TAPSTS automatically clears those
! bits.
!
! I BELIEVE that TAPSTS bits 3 (WRITE-PROTECT), 4 (HOLE), and
! 5 (NOT-GAP) and 7 (READY) are "current states", not latched
! or buffered, but I'm not entirely sure.
! 						
! ************************************************************
! CLKSTS (177412) READ:
! *   BIT0 = TIMER 0 ENABLED
! *   BIT1 = TIMER 1 ENABLED
! *   BIT2 = TIMER 2 ENABLED
! *   BIT3 = TIMER 3 ENABLED
! *   BIT4 =
! *   BIT5 =
! *   BIT6 =
! *   BIT7 = TIMER DATA READY (THROUGH CLKDAT)
! CLKSTS (177412) WRITE:
! *   BIT0 = DISABLE TIMER
! *   BIT1 = ENABLE TIMER
! *   BIT2 = STOP TIMER
! *   BIT3 = START TIMER
! *   BIT4 = CLEAR TIMER
! *   BIT5 = CLEAR INTERRUPT SERVICE FLIP-FLOP
! *   BIT6 =  / 6 & 7 TOGETHER PROVIDE THE TIMER# (0-3)
! *   BIT7 = / TO WHICH BITS 0-5 APPLY
! CLKDAT (177413) READ: CURRENT VALUE OF THE REQUESTED TIMER (4 BYTES, 8 BCD DIGITS)
! CLKDAT (177413) WRITE: (4 BYTES, 8 BCD DIGITS) THE NEW VALUE FOR THE SPECIFIED TIMER)
! ************************************************************
! * PRMLEN (177414) = PRINTER MESSAGE LENGTH (0-32)
! * PRCHAR (177415) = INDEX INTO PRINTER'S "FONT ROM" OF CHARACTER TO READ DATA FOR
! * PRSTS (177416) READ:
! *   BIT0 = 1 IF PRINTER CONTROLLER READY (0 IF NOT READY)
! *   BIT1 = 0
! *   BIT2 = 0
! *   BIT3 = 0
! *   BIT4 = 0
! *   BIT5 = 0
! *   BIT6 = 1 IF DATA IS READY (0 IF NOT READY)
! *   BIT7 = 1 IF PAPER IS LOADED (0 IF OUT OF PAPER)
! * PRSTS (177416) WRITE:
! *   BIT0 =  \
! *   BIT1 =   0-7 COLUMN OF CHARACTER DATA TO READ
! *   BIT2 =  /
! *   BIT3 =
! *   BIT4 =
! *   BIT5 = 1 TO READ COLUMN# (BITS 0-2) OF FONT DATA FROM FONT ROM
! *   BIT6 = 1 SEND HEAD HOME IN ALPHA MODE
! *   BIT7 = 1 POWER-UP PRINTER, = 0 POWER-DOWN PRINTER
! * PRDATA (177417) = READ (NEXT BYTE OF DATA FROM "FONT ROM") OR WRITE (CHAR OR COLUMN OF GRAHPICS)
! ************************************************************
! * SVCWRD ORGANIZATION (FUNCTION CALLED TO SERVICE IT)
! *   BIT 0 = KEYBOARD INTERRUPT (CHEDIT)
! *   BIT 1 = I/O INTERRUPT (IOSP)
! *   BIT 2 = TIMER 1 INTERRUPT (ONCLK1)
! *   BIT 3 = TIMER 2 INTERRUPT (ONCLK2)
! *   BIT 4 = TIMER 3 INTERRUPT (ONCLK3)
! *   BIT 5 = "OTHER" (SPAR0)
! ************************************************************
         BYT  340,37           ! ROM# 340 AND 1'S COMPLEMENT
         DEF  RUNTIM           ! RUNTIME TABLE POINTER
         DEF  TOKENS           ! ASCII KEYWORD TABLE POINTER
         DEF  PARSE            ! PARSE TABLE POINTER
         DEF  ERMSGS           ! ERROR MESSAGES TABLE POINTER
         DEF  INIT             ! ROM INITIALIZATION ROUTINE POINTER
         DEF  PWORUN           ! POWER-ON VECTOR ADDRESS
TOKENS   ASP  "SYSTEM"         !
         ASP  "CPU"            !
         ASP  "RAM"            !
         ASP  "ROM"            !
         ASP  "BEEPER"         !
         ASP  "DISPLAY"        !
         ASP  "PRT"            !
         ASP  "ATAPE"          !
         ASP  "TPROT"          !
         ASP  "TIMER"          !
         ASP  "KBD"            !
         ASP  "XRAM"           !
         ASP  "XROM"           !
         ASP  "HEAT"           !
         ASP  "TAPE"           !
         ASP  "TSTAT"          !
         ASP  "TSPEED"         !
         ASP  "THOLE"          !
         ASP  "TWRITE"         !
         ASP  "TREAD"          !
         ASP  "TREC"           !
         ASC  "CYCL"           !
PARSE    BYT  305              !
         BYT  377              ! END OF ASCIIS TABLE
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
         DEF  GENPAR           !
RUNTIM   DEF  GENPAR           ! RUNTIME ROUTINES TABLE (ENTRY 0 ISN'T USED)
RUNLTTRS DEF  SYSTM.           ! A
         DEF  CPU.             ! B
RAMRUN   DEF  RAM.             ! C
         DEF  ROM.             ! D
         DEF  BEEPR.           ! E
         DEF  DISPL.           ! F
         DEF  PRT.             ! G
         DEF  ATAPE.           ! H
TPROTRUN DEF  TPROT.           ! I
         DEF  TIMER.           ! J
         DEF  TSTKBD           ! K
XRAMRUN  DEF  XRAM.            ! L
         DEF  XROM.            ! M
         DEF  HEAT.            ! N
         DEF  TAPE.            ! O
         DEF  TSTAT.           ! P
         DEF  TSPED.           ! Q
         DEF  THOLE.           ! R
         DEF  TWRIT.           ! S
         DEF  TREAD.           ! T
         DEF  TREC.            ! U
         DEF  CYCLE.           ! V
! ************************************************************
GENPAR   LDB  R14,=370         ! GET "ROM TOKEN" TOKEN
         PUBD R14,+R12         ! PUSH IT
         PUBD R41,+R12         ! PUSH ROM#
         PUBD R43,+R12         ! PUSH ROM TOKEN #
         JSB  =ROMJSB          !
         DEF  SCAN             ! SCAN FOR NEXT TOKEN
         BYT  0                !
         GTO ROMRTN            ! AND RETURN TO SYSTEM PARSER
! ************************************************************
! * POWER-ON ENTRY POINT
! ************************************************************
PWORUN   BIN                   !
         CLM  R20              !
         STBD R20,=PRMODE      ! CLEAR PRINTING FLAG
         STBD R20,=SVCWRD      ! CLEAR INTERRUPTS PENDING FLAGS
         STMD R20,=CRTBAD      ! SET CRT BYTE
         STMD R20,=CRTRAM      !   AND START PTRS TO 0
         STMD R20,=CRTBYT      ! AND THE RAM COPY OF CRTBAD
! *** NOW CLEAR THE CRT RAM
         LDM  R20,=0,10        ! # CHARS TO WRITE (2K)
CLRCRT2  LDM  R22,=0,2         ! TIME-OUT COUNTER
PWOWAIT  DCM  R22              ! DEC TIME-OUT
         JZR  PWOTIMOU         ! JIF TIMED-OUT
         LDBD R24,=CRTSTS      ! GET CRT STATUS
         JNG  PWOWAIT          ! JIF BUSY
         LDB  R24,=15          ! CARRIAGE RETURN CHAR
         STBD R24,=CRTDAT      ! WRITE TO CRT MEM
         DCM  R20              ! DEC CHAR COUNTER
         JNZ  CLRCRT2          ! JIF NOT DONE
PWOTIMOU CLM  R#               !
         STMD R#,=CRTBAD       ! AGAIN, SET BYTE
         STMD R#,=CRTSAD       !   AND START PTRS TO 0
         STBD R#,=CRTSTS       ! UNWIPE & POWER-UP CRT IN ALPHA
         STBD R#,=CRTWRS       ! SAVE A COPY IN RAM
         LDB  R#,=200          ! TURN ON SENDING-MSGS-TO-PRINTER
         STBD R#,=PRMODE       !
         JSB  =CPU.            ! DO CPU TEST
         JSB  =RAM.            ! DO RAM TEST
         JSB  =ROM.1           ! DO ROM TEST
         JSB  =CRTIMES         ! CHECK CRT DISP/RETRACE TIMES
         JSB  =CHKFONT         !
         JSB  =TREGSSUB        ! CHECK TAPE CONTROLLER REGISTERS
         JSB  =TSTTLENJ        ! TEST SPEED OF CLOCKS
         JSB  =CHKINTSJ        ! CHECK KBD/GLOBAL INTERRUPTS ENABLE/DISABLE
!
SPITTSTM BYT  142,251          ! LDM R42,=
         DEF  MSGA-V           ! " A-V"
         BYT  30,4             ! LEN 1, LEN 2
         DEF  MSGSVROM         ! "SERVICE ROM: SELECT TEST"
!
         JSB  =OUTLIN+S        ! SPIT "SERVICE ROM: SELECT TEST A-V"
         LDB  R52,=200         ! WARN & REQUIRE KEY IF TIME-OUT
         JSB  =GETTAKEY        ! WAIT FOR THE KEY
         JZR  NOKEYRTN         ! JIF NO KEY
!
RUNKEY   STB  R27,R75          ! COPY KEYCODE
         ANM  R27,=337         ! UPPERCASE THE KEYCODE LETTER
         LDB  R21,=101         ! A FOR STARTERS
         LDM  R22,=RUNLTTRS    ! BASE OF RUNTIME TESTS
KEYLETLP POMD R24,+R22         ! GET THE NEXT ADDRESS
         CMB  R27,R21          ! CUR LETTER MATCH THE KEY?
         JNZ  NXTKEYLT         ! JIF NO
         JSB  X24,ZRO          ! CALL THE TEST ROUTINE
         JMP  SPITTSTM         ! RE-SPIT THE PROMPT
!
NXTKEYLT ICB  R21              ! INC TO NEXT LETTER
         CMB  R21,=127         ! W?
         JNZ  KEYLETLP         ! JIF NO, TRY AGAIN
!
         CMB  R21,R27          ! W THE KEY PRESSED?
         JNZ  NOKEYRTN         ! JIF NO, RETURN TO SYSTEM
GKSLOOP  JSB  =GKSHORT         ! TRY TO GET KEY AGAIN
         JZR  GKSLOOP          ! JIF NO KEY, KEEP WAITING
         JMP  RUNKEY           ! GO RUN THE TEST (HOPEFULLY)
!
NOKEYRTN BYT  144,251          ! LDM R44,=
         DEF  MSGRTNSY         ! "RETURN TO BASIC SYSTEM  "
         BYT  0,30             ! LEN 1, LEN 2
         STM  R#,R32           ! COPY MSG ARGS
         JSB  =OUTLINGO        ! OUTPUT THE MSG
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         RTN                   ! RETURN TO NORMAL SYSTEM BOOT
!
! ************************************************************
! * ROM INITIALIZATION ROUTINE
! ************************************************************
INIT     LDBD R32,=ROMFL       ! REASON FOR CALL
         JNZ  NOTPWO           ! JIF NOT PWO
         LDM  R32,=ROM60000    ! BASE ADDR OF THIS ROM
         JSB  =RSUM8K          ! CHECKSUM IT
         JZR  NOTPWO           ! JIF OK
         LDBD R34,=ERRORS      ! ANY EARLIER ERRORS?
         JNZ  NOTPWO           ! JIF YES, IGNORE OURS
         LDBD R34,=ROM60000    ! GET OUR ROM#
         STBD R34,=ERRROM      ! TELL SYSTEM
!
         JSB  =ERROR           !
         BYT  14               ! "BAD SERVICE ROM"
NOTPWO   RTN                   !
!
OUTBEEP+ STM  R42,R32          ! MOVE MESSAGE DATA
OUTBEEP- LDB  R75,=15          ! TRAILING CR CHAR
OUTBEEP  BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R20,=100         ! STD BEEP *ON*
BEEPOFF  STBD R#,=KEYSTS       !
         LDM  R22,=0,20        ! DELAY COUNT
BEEPLOOP DCM  R#               ! DEC COUNT
         JNZ  BEEPLOOP         ! JIF NOT DELAYED YET
         SBB  R20,=100         ! TURN BEEP OFF (OR DONE)
         JZR  BEEPOFF          ! JIF TURN OFF
!
! Fall through into OUTLINE to display message
!
! ************************************************************
! Spit a messag to the display
! ENTRY:
!   R32-33 = address of 2nd message
!   R34    = length of message
!   R35    = length of 2nd message (or 0)
!   R36-37 = address of message
!   R75    = blank-fill character (frequently CR)
! ************************************************************
OUTLINE  BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         CLB  R20              !
         LDM  R26,R36          ! ADDRESS OF MSG
         LDB  R24,R34          ! LENGTH OF MSG
         ADB  R24,R35          ! TOTAL LEN OF BOTH MESSAGES
MNXTCHAR CMB  R#,R35           ! FINISHED WITH MSG1? (LEN=LEN(MSG2))
         JNZ  MNOT0            ! JIF NO
         LDM  R26,R32          ! MSG1 DONE, NOW USE MSG2 ADDRESS
MNOT0    LDBD R25,R26          ! GET NEXT CHAR
         ICM  R26              !
MCRBLNKS LDMD R22,=CRTBYT      ! GET CRT'S CHARACTER ADDRESS
         ICM  R22              ! MOVE RIGHT 1 CHAR (NYBBLE ADDRESSING=2)
         ICM  R22              !
         ANM  R22,=377,17      ! WRAP AROUND IN CASE OFF END OF CRT RAM
         STMD R22,=CRTBYT      ! SET NEW CRT CHARACTER ADDRESS
         LDM  R22,=0,2         ! TIME-OUT VALUE
WAITNBSY DCM  R22              !
         JZR  L61134           ! JIF TIMED-OUT
         LDBD R21,=CRTSTS      ! WAIT UNTIL CRT NOT BUSY
         JNG  WAITNBSY         !
         STBD R25,=CRTDAT      ! WRITE CHARACTER TO CRT
         DCB  R24              ! DEC LEN OF MSG
         JNZ  MNXTCHAR         ! LOOP TIL DONE
         ICB  R24              ! R24 == 1 (OUTPUT 1 TRAILING CHAR)
         LDB  R25,=15          ! DEFAULT CR CHAR
         TSB  R20              ! HAVE WE OUTPUT THE TRAILING CHAR?
         JNZ  L61006           ! JIF YES, FILL REST OF LINE WITH CRs
         ICB  R20              ! FLAG WE'VE DONE THE TRAILING CHAR
         LDB  R25,R75          ! GET CR/SPACE/ETC TRAILING CHARACTER
         JMP  MCRBLNKS         ! GO DO ONE OF THOSE
!
! *** NOW BLANK FILL THE REST OF THE LINE WITH CR CHARACTERS
!
L61006   LDMD R22,=CRTBYT      ! GET CRT BYTE ADDRESS
         ANM  R22,=77,0        ! AT START OF LINE?
         JNZ  MCRBLNKS         ! JIF NO, OUTPUT ANOTHER CR
         ICB  R20              ! BUMP STATE CONTROL
         JOD  L61031           ! JIF YES (==3), DONE
         LDMD R26,=CRTBYT      ! GET CURRENT BYTE ADDRESS
         JMP  MCRBLNKS         ! DO MORE (BLANK NEXT LINE)
!
L61031   LDM  R22,=0,2         ! TIME-OUT VALUE
L61035   DCM  R22              ! TIMED-OUT?
         JZR  OLTO             ! JIF YES
         LDBD R21,=CRTSTS      ! GET CRT STATUS
         JNG  L61035           ! JIF NOT RETRACE
OLTO     STMD R26,=CRTBAD      ! WRITE BYTE-ADDRESS
         STMD R26,=CRTBYT      ! AND IN RAM
         LDMD R24,=CRTRAM      ! GET CRT START ADDRESS
L61062   LDM  R22,R26          ! GET BYTE ADDRESS
         CMM  R22,R24          ! LESS THAN START ADDRESS?
         JCY  L61074           ! JIF NO
         ADM  R22,=0,20        ! ELSE WRAP IT
L61074   SBM  R#,R#            ! GET 'CURSOR' OFFSET
         CMM  R#,=0,4          ! OFF BOTTOM OF SCREEN?
         JNC  L61134           ! JIF NO
         ADM  R24,=100,0       ! SCROLL UP 1 ROW OF CRT
         ANM  R24,=377,17      ! WRAP AROUND IF NECESSARY
         STMD R24,=CRTSAD      ! SAVE NEW START (HOME) OF PAGE
         STMD R24,=CRTRAM      ! AND IN RAM
         JMP  L61062           ! REPEAT TEST FOR SCROLL NEED
!
! R32-3 = MSG 2 PTR
! R34   = LEN 1
! R35   = LEN 2
! R36-7 = MSG 1 PTR
!
DISPPRT  BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R75,=15          ! FILL WITH CR CHARACTERS
         CLB  R74              ! CLEAR ERRORS FLAG
         JMP  L61137           ! OUTPUT THE MESSAGE
!
L61134   LDB  R74,=340         ! SERVICE ROM#
L61137   LDBD R27,=PRMODE      ! GET PRINT FLAGS
         LLB  R27              ! CLEAR UPPER BIT
         LRB  R27              ! RESTORE THE OTHER BITS
         STBD R74,=ERNUM#      ! SAVE IN ERNUM#
         JNZ  L61156           ! JIF OTHER BITS SET
         ADB  R27,=200         ! ELSE SET UPPER BIT
L61156   STBD R27,=PRMODE      ! SAVE
         LDB  R20,=1           ! CRT DEVICE FOR OUTPUT
         JMP  PRTMAYBE         !
! ************************************************************
! ** POWER DOWN CRT AND PRINTER UNITS
! ************************************************************
DOWNCRT  STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         CLB  R20              ! DON'T POWER DOWN PRINTER
DOWNCP   LDM  R22,=232,1       ! TIME-OUT VALUE
DCPWAIT  DCM  R22              ! DEC TIME-OUT COUNT
         JZR  DCPGO            ! JIF TIMED-OUT
         LDBD R21,=CRTSTS      ! GET CRT STATUS
         LRB  R21              ! GET RETRACE BIT
         JOD  DCPWAIT          ! JIF DISPLAY TIME
DCPGO    LDB  R21,=6           ! POWER-DOWN CRT, WIPE-OUT CRT
         LDBD R25,=CRTWRS      ! GET PREVIOUS WRITE BITS
         ORB  R25,R21          ! OR IN PWDOWN, WPO
         STBD R25,=CRTSTS      ! TELL THE CRT
         STBD R25,=CRTWRS      ! SAVE IN RAM, SINCE WE CAN'T READ BACK
         TSB  R20              ! PRINTING?
         JNZ  PDPRINT          ! JIF YES
         RTN                   ! DONE
!
PDPRINT  LDB  R#,=100          ! HOME HEAD IN ALPHA MODE AND POWER-DOWN
         STBD R#,=PRSTS        !
         JMP  PRTMAYBE         !
!
! *** R27 HAS PRMODE AT ENTRY
! *** BIT0=1 IF CRT IS POWERED UP
! *** BIT1=1 IF WRITING TO PRINTER
! ***
! *** I THINK...
!
PRINTIT  TSB  R27              ! WRITING TO PRINTER, TOO?
         JOD  L61265           ! JIF YES
         ICB  R27              ! SET BIT0
         STBD R27,=PRMODE      ! SAVE PRMODE
         CLB  R20              ! CLEAR PRINT/CRT FLAGS
         STBD R20,=PRMLEN      ! SET PRINT LEN TO 0
         JMP  PRTMAYBE         ! OUTPUT TO CRT
!
L61265   CLB  R20              !
         LDM  R26,R36          ! ADDRESS OF STRING 1
         LDB  R24,R34          ! LEN 1
         ADB  R24,R35          ! TOTAL LEN (1+2)
         ICB  R24              ! +1 (TRAILING CHAR)
         STBD R24,=PRMLEN      ! TELL PRT CONTROLLER
         JZR  JUSTCRT          ! JIF NOTHING TO PRINT
         DCB  R24              ! REMOVE THE +1
MORSTR1? CMB  R#,R35           ! DONE WITH STRING 1?
         JNZ  NOSTR1           ! JIF NO
         LDM  R26,R32          ! ADDRESS OF STRING 2
NOSTR1   LDBD R25,R26          ! GET NEXT CHAR
PIT      STBD R#,=PRDATA       ! SEND IT TO PRINTER
         ICM  R26              ! BUMP ADDRESS POINTER
         DCB  R24              ! DEC LEN COUNTER
         JNZ  MORSTR1?         ! JIF NON-ZERO
         ICB  R24              ! +1 FOR TRAILING CHAR
         ICB  R20              ! "DONE TRAILING CHAR" FLAG
         JEV  JUSTCRT          ! JIF ALREADY DID IT, DONE
         LDB  R25,R75          ! COPY TRAILING CHAR
         JMP  PIT              ! OUTPUT IT
!
JUSTCRT  LDB  R20,=2           ! FLAG PRINTING DONE
PRTMAYBE LDM  R22,=377,377     ! TIME-OUT COUNT
PRTBSY   DCM  R22              ! DEC TIME-OUT COUNT
         JZR  UPCRT            ! JIF TIMED-OUT, JUST WRITE TO CRT
         LDBD R21,=PRSTS       ! GET PRINTER STATUS
         JEV  PRTBSY           ! JIF BUSY (NOT READY)
         JPS  UPCRT            ! JIF NO PAPER LOADED, DON'T PRINT, JUST DO CRT
         TSB  R20              ! CRT POWERED UP?
         JOD  DOWNCP           ! JIF YES, POWER DOWN CRT
         JLN  PRINTIT          ! JIF ALREADY HOMED PRINT-HEAD
         JZR  L61265           ! JIF NOT PRINTING OR DONE PRINTING
!
! *** OTHERWISE POWER UP CRT AT VERY START OF A RETRACE
!
UPCRT    LDBD R25,=CRTWRS      ! GET CRT STATUS
         ANM  R25,=373         ! CLEAR BIT2 (POWER UP CRT)
         STBD R25,=CRTSTS      ! WRITE IT BACK
         ANM  R25,=371         ! CLEAR BIT1 (UNWIPE CRT)
         LDB  R24,=3           ! RETRACE COUNT (WAIT 3 RETRACES)
UPCRTRTC CLB  R26              ! SET "WAIT FOR NOT RETRACE"
UPCRTHI  LDM  R22,=45,1        ! TIME-OUT VALUE
UPCRTLO  DCM  R22              ! DECREMENT TIME-OUT
         JZR  UNWIPE           ! JIF TIMED-OUT
         LDBD R21,=CRTSTS      ! GET CRT STATUS
         ANM  R21,=2           ! IN RETRACE?
         LRB  R21              ! SHIFT IT OVER TO LSBIT
         CMB  R21,R26          ! STILL IN SAME IS/NOT RETRACE STATE?
         JZR  UPCRTLO          ! JIF YES
         ICB  R26              ! SWITCH TO "WAIT FOR RETRACE" STATE
         JOD  UPCRTHI          ! JIF WAITING WHILE RETRACE HIGH
         DCB  R24              ! ELSE DEC RETRACE COUNT
         JNZ  UPCRTRTC         ! JIF NOT DONE, COUNT ANOTHER RETRACE
UNWIPE   STBD R25,=CRTSTS      ! UNWIPE CRT
         STBD R25,=CRTWRS      ! SAVE THAT STATE IN RAM
DOHOOKS  LDBD R20,=ROMFL       ! MOST RECENT ROM INIT REASON
         JPS  L61520           ! JIF DON'T TAKE HOOKS
!
! *** TAKE KYIDLE, PUT RTNs IN IRQ20,SPAR0, SPAR1
!
TAKHOOKS STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         STM  R44,R20          ! SAVE R44
!
! *** LDM R44,= JSB=KYIDLENT RTN
!
         BYT  251              ! LDM R44,= THE FOLLOWING 2 LINES OF CODE
         JSB  =KYIDLENT        !
         RTN                   !
!
         STMD R#,=KYIDLE       ! TAKE KEYBOARD HOOK
         STBD R47,=IRQ20       ! STUFF RTN IN OTHER HOOKS
         STBD R47,=SPAR0       !
         STBD R47,=SPAR1       !
         LDM  R44,R#           ! RESTORE R44
L61520   STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         RTN                   ! DONE
! ************************************************************
! SYSTEM TEST - automatically runs:
!   a) CPU test
!   b) RAM test
!   c) ROM test
!   d) BEEPER test
!   e) CRT test
!   f) PRINTER test
!   g) ABBREVIATED TAPE test
!   h) TAPE WRITE-PROTECT test
!   i) TIMER test
!   j) KEYBOARD test
! ************************************************************
         BYT  241              ! ATTRIBUTES
SYSTM.   BYT  142,251          ! LDM R42,=
         DEF  MSGSPTST         ! " TEST BEGINS"
         BYT  6,14             ! LEN 1, LEN 2
         DEF  MSGSYSTM         ! "SYSTEM"
!
         JSB  =OUTLIN+S        ! SPIT THE STRING
         JSB  =CPU.            ! CHECK THE CPU
         STB  R74,R73          ! SAVE ERROR STATUS
         LDM  R76,=RAMRUN      ! PTR TO TABLE OF RUNTIMES (START WITH RAM.)
SYSLOOP  POMD R20,+R76         ! GET THE NEXT TEST ADDRESS
         JSB  X20,ZRO          ! CALL IT
         ORB  R73,R74          ! OR RESULT INTO ACCUM RESULTS
         CMM  R76,=TPROTRUN    ! JUST FINISHED ATAPE.?
         JNZ  CKSYSLST         ! JIF NO
         TSB  R74              ! ERROR?
         JZR  CKSYSLST         ! JIF YES
         CMM  R32,=MSGOUT      ! "CART OUT" MESSAGE?
         JZR  ISCARTOU         ! JIF YES
         ICB  R73              ! ELSE FORCE ERROR
ISCARTOU POMD R20,+R76         ! THROW AWAY NEXT TEST
CKSYSLST CMM  R76,=XRAMRUN     ! AT END OF SYSTEM TESTS?
         JNZ  SYSLOOP          ! JIF NO
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGTEND          ! " TEST ENDS"
         BYT  6,12             ! LEN 1, LEN 2
         DEF  MSGSYSTM         ! "SYSTEM"
!
SPITMESS JSB  =OUTLIN+S        ! SPIT MSG
         LDB  R74,R73          ! GET RESULTS
         LRB  R74              ! THROW AWAY LSBIT
         LLB  R74              !
         STBD R74,=ERNUM#      ! SAVE AS ERNUM#
         RTN                   ! DONE
! ************************************************************
! CYCLE TEST - runs SYSTEM test repeatedly until key press.
! ************************************************************
         BYT  241              ! ATTRIBUTES
CYCLE.   JSB  =SYSTM.          ! DO THE SYSTEM TEST
         TSB  R73              ! ERRORS?
         JOD  CYCEXIT          ! JIF YES, BAIL OUT
         JSB  =GKSHORT         ! WAIT FOR A KEY
         STBD R53,=KEYSTS      ! ENABLE KEY SCANNING (R53=1)
         TSB  R52              ! KEY?
         JZR  CYCLE.           ! JIF NO, CYCLE
CYCEXIT  STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         RTN                   ! DONE
! ************************************************************
! HEAT TEST - turns CRT on full so the entire screen is white,
! and simultaneously runs a repeating series of IC checks.
! This was designed as a "production test" to "burn in" the
! units and detect "infant mortality."
! ************************************************************
         BYT  241              ! ATTRIBUTES
HEAT.    LDM  R36,=MSGHEAT     ! "HEAT TEST BEGINS"
         LDM  R34,=20,0        ! LEN 1, LEN 2
         JSB  =DISPPRT         ! OUTPUT THE MESSAGE
         JSB  =STRTTIMR        ! TEST TIMERS & START TIMER #3
         TSB  R74              ! ERRORS?
         JNZ  HEATBAIL         ! JIF YES
HEATMORE JSB  =CPU.            ! DO CPU TEST
         STB  R74,R73          ! COPY ERROR
         JNZ  HEATBAIL         ! JIF ERROR
         LDM  R76,=HEATTABL    ! ADDRESS OF LIST OF HEAT TESTS
HEATLOOP POMD R20,+R76         ! GET NEXT TEST
         JSB  X20,ZRO          ! DO THAT TEST
         TSB  R74              ! ERROR?
         JNZ  HEATBAIL         ! JIF YES
         STBD R31,=GINTEN      ! ENABLE INTERRUPTS
         LDBD R31,=SVCWRD      ! GET PENDING INTERRUPTS
         ANM  R31,=21          ! KEEP ONLY TIMER #3 AND KBD
         JLZ  HEATCK           ! JIF TIMER #3 HAS *NOT* GONE OFF
         JSB  =CLRTINTS        ! CLEAR TIMER INTERRUPTS
         JSB  =INCROM#         ! INC THE TSTBUF "000" COUNTER
         LDBD R32,=PRMODE      ! GET PRINT MODE FLAGS
         LRB  R32              !
         JOD  CLRCRT           ! JIF YES
         LDM  R32,=MSGHLOOP    ! " HR"
         LDM  R34,=3,3         !
         JSB  =DISPPRT         ! UPDATE THE SCREEN MSG
HEATCK   TSB  R31              ! KEY PRESSED?
         JOD  CLRCRT           ! JIF YES
         CMM  R76,=HEATEND     ! FINISHED LAST TEST IN LIST?
         JNZ  HEATLOOP         ! JIF NO, KEEP GOING
         LDB  R76,=202         ! GRAPHICS MODE, WIPE-OUT, POWERED-UP
         STBD R76,=CRTSTS      ! TELL CRT
         JMP  HEATMORE         ! DO IT ALL AGAIN
!
HEATBAIL STB  R#,R73           ! SAVE ERROR INFO
         LDB  R#,=206          ! GRAPHICS, WIPE-OUT, POWER-DOWN CRT
         STBD R#,=CRTSTS       ! TELL IT
         STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         LDBD R#,=PRMODE       ! GET PRINT MODE FLAGS
         LRB  R#               !
         JOD  CLRCRT           ! JIF PRINTING
HEATKEY  LDBD R#,=SVCWRD       ! GET PENDING INTERRUPTS (WAIT FOR KEY)
         JEV  HEATKEY          ! JIF NO KEY PRESSED
CLRCRT   JSB  =HOMCLEAR        ! CLEAR THE CRT
         JSB  =OFFTIMRS        ! CLEAR, STOP ALL TIMERS
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGTEND          ! " TEST ENDS"
         BYT  4,12             ! LEN 1, LEN 2
         DEF  MSGHEAT          ! "HEAT"
!
         JSB  =SPITMESS        ! SPIT THE MESSAGE
         JSB  =CLRKBD          ! CLEAR PENDING KBD, ENABLE INTS
         RTN                   ! DONE
!
STRTTIMR JSB  =TIMER.          ! RUN THE TIMER TEST
         LDB  R27,=25          ! CLEAR, STOP, DISABLE ALL TIMERS
         JSB  =ALLCLKS         ! DO IT
         LDM  R64,=0,0,60C,3   !
         STMD R64,=CLKDAT      ! INIT TIMER #3 COUNT
         LDM  R64,=312,60,60,60 ! START & ENABLE TIMER #3; "000"
         STBD R64,=CLKSTS      ! START TIMER #3
         STMD R65,=TSTBUF      ! STORE "000" COUNTER IN TSTBUF
         RTN                   ! DONE
! ************************************************************
! CURRENT ROM# (IN DECIMAL) IS STORED IN 'TSTBUF' AS 3 ASCII CHARACTERS
! THIS ROUTINE ADDS 1 TO IT (HANDLING 'CARRY' AS NEEDED).
!
INCROM#  LDM  R36,=TSTBUF      ! ADDRESS OF ASCII ROM ###
         LDMD R65,R36          ! GET IT
         LDB  R0,=67           ! POINT AT R67
NOT64    ICB  R*               ! ++ IT
         CMB  R*,=72           ! ':'? (PAST '9'?)
         JNZ  NOT72            ! JIF NO
         LDB  R*,=60           ! RESET IT TO '0'
         DCB  R0               ! POINT AT PREV REG (R66 FIRST TIME THRU)
         CMB  R0,=64           ! REACHED R64? TOO FAR?
         JNZ  NOT64            ! JIF NO
NOT72    STMD R65,R36          ! STORE IT BACK TO RAM
         RTN                   !
!
HEATTABL DEF  TSTCRAM          ! CHECK CRT RAM
         DEF  CHKFONT          ! CHECK PRINTER FONT
         DEF  TREGSSUB         ! TEST TAPE CONTROLLER REGISTER
         DEF  TSTTLENJ         ! TEST TIMER-READY LEN (CLOCK SPEEDS)
         DEF  RAM.             ! TEST RAM
         DEF  ROM.1            ! CHECKSUM ROMS
! ************************************************************
! CPU TEST - Since the CPU must be mostly functional in order
! even GET here, only a perfunctory test is performed, not an
! exhaustive test.
! POSSIBLE RESULTS:
!   OK
!   CPU BAD!
!   CPU BAD! N OR RAM BAD! (CHARACTER)
! THE (CHARACTER) INDICATES THAT A FAILURE HAS BEEN FOUND DURING
! AN ATTEMPT BY THE CPU TO STORE DATA INTO RAM. WHEN THIS
! MESSAGE APPEARS, EITHER THE CPU, RAM CONTROLLER, OR A RAM IC
! MAY BE BAD. THE CHARACTER AT THE END OF THE MESSAGE IS A CODE
! THAT INDICATES WHICH RAM IC MAY BE BAD.
! ************************************************************
HEATEND  BYT  241              ! ATTRIBUTES
CPU.     BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R75,=101         ! 'A' ERROR LETTER
!
! NOT SURE WHAT THEY HOPE TO ACCOMPLISH WITH THESE
! ARPs AND DRPs, AS THEY'RE NEVER TESTED FOR SUCCESS.
! JUST TO MAKE SURE THEY DON'T CAUSE THE CPU TO TANK, MAYBE?
!
         DRP  R0               ! CHANGE DRP/ARP TO ALL 0s
         ARP  R0               !
         DRP  R25              ! CHANGE DRP/ARP TO 00010101
         ARP  R25              !
         DRP  R52              ! CHANGE DRP/ARP TO 00101010
         ARP  R52              !
!
! TEST IMMEDIATE AND REG-TO-REG LOADS, STORES AND COMPARES
!
         LDM  R60,=252,125,125,252,125,252,125,252 ! ALTERNATING 1s AND 0s
         JMP  JMPOK            !
!
         GTO CPUERR            !
!
JMPOK    LDM  R0,R66           ! COPY REG-TO-REG ALTERNATING 1s AND 0s TO R0-1
         CMM  R0,=125,252      ! MULTI-BYTE HAVE THE RIGHT VALUE?
         JNZ  CPUER+1J         ! JIF NO
         JZR  CMPOK1           ! JIF YES
         JMP  CPUER+1J         ! JNZ/JZR SHOULD HAVE WORKED, SO FAIL
!
CMPOK1   CMB  R0,=252          ! SINGLE BYTE HAVE THE RIGHT VALUE?
         JZR  CPUER+1J         ! JIF NO
         JNZ  CMPOK2           ! JIF YES
         JMP  CPUER+1J         ! JNZ/JZR SHOULD HAVE WORKED, FAIL!
!
CMPOK2   LDM  R0,=252,125      ! LOAD IMMEDIATE ALTERNATING 1s AND 0s
         CMM  R0,R65           ! 2-BYTE CMP REG-TO-REG
         JZR  CMPOK3           ! JIF OK
CPUER+1J GTO CPUERR+1          ! CPU 'B'
!
! TEST OR, XOR, AND
!
CMPOK3   LDM  R44,R64          ! 44-47= 125 252 125 252
         STM  R44,R54          ! 54-57= 125 252 125 252
         STM  R44,R50          ! 50-53= 125 252 125 252
         XRB  R44,R60          ! 44-47= 377 252 125 252
         ORM  R45,R61          ! 44-47= 377 377 125 252
         XRM  R65,R61          ! 65-67=     377 000 000
         ORB  R64,R60          ! 64-67= 377 377 000 000
         CMM  R64,=377,377,0,0 ! RIGHT?
         JNZ  CPUER+2J         ! JIF NO, ERROR
         ANM  R54,R60          ! 54-57= 000 000 125 252
         STM  R54,R40          ! 40-43= 000 000 125 252
         CMM  R40,=0,0,125,252,377,377,125,252 ! RIGHT?
         JNZ  CPUER+2J         ! JIF NO, ERROR
         ANM  R46,=132,137     ! 46-47= 120 012
         CMM  R46,=120,12      ! RIGHT?
         JZR  CMPOK4           ! JIF YES
CPUER+2J GTO CPUERR+2          !
!
! TEST BIN MODE ARITHMETIC AND SHIFTING
!
CMPOK4   LDM  R55,R64          ! 55-57= 377 377 000
         ADM  R60,R50          ! SEE ADDITION BELOW
! 60-67= 252 125 125 252 377 377 000 000
! 50-57= 125 252 125 252 000 377 377 000
!    CY=                   1   1   1   1
! 60-67= 377 377 252 124 000 377 000 001
         SBM  R56,R54          ! 56-57= 377 001
         ICM  R63              ! 63-67= 125 000 377 000 001
         DCM  R62              ! 62-67= 251 125 000 377 000 001
         TCB  R55              ! 55   = 001
         NCB  R54              ! 54   = 377
         LLB  R53              ! 53   = 124
         ELB  R53              ! 53   = 251
         ELB  R53              ! 53   = 122
         LRB  R53              ! 53   = 051
         ERB  R53              ! 53   = 024
         ERB  R53              ! 53   = 212
         CMM  R53,=212,377,1,377,1 ! RIGHT?
         JNZ  CPUER+3J         ! JIF NO, ERROR
         CMM  R60,=377,377,251,125,0,377,0,1 ! RIGHT?
         JZR  CMPOK5           ! JIF YES
CPUER+3J GTO CPUERR+3          !
!
! NOW TEST BCD MODE ARITHMETIC AND SHIFTING
!
CMPOK5   BCD                   ! NOW CHECK BCD MODE
         LDM  R41,R40          ! 40-47 = 000 000 000 000 000 000 000 000
         LDM  R70,=356,314,252,42,104,146,231,0 !
!
! NOTE: THE ABOVE CONTAIN SOME INVALID BCD VALUES,
! SO THE RESULTS OF ADDING 0 TO THEM IS UNEXPECTED.
!
         ADM  R70,R40          ! 70-77= 146 104 042 042 104 146 231 000
         SBM  R76,=0,231       ! 76-77= 231 001
         NCB  R75              ! 75   = 063
         ICB  R74              ! 74   = 105
         TCB  R73              ! 73   = 170
         DCB  R72              ! 72   = 041
         CMM  R70,=146,104,41,170,105,63,231,1 ! RIGHT?
         JNZ  CPUERE           ! JIF NO, ERROR
         LDB  R70,=245         ! 70   = 245
         LLB  R70              ! 70   = 120 E=12
         ELB  R70              ! 70   = 012 E=05
         ELB  R70              ! 70   = 245 E=00
         LRB  R70              ! 70   = 012 E=05
         ERB  R70              ! 70   = 120 E=12
         ERB  R70              ! 70   = 245 E=00
         CMB  R70,=245         ! RIGHT?
         JZR  CMPOK6           ! JIF YES
CPUERE   LDB  R75,=105         ! 'E' CPU ERROR
         GTO CPUERR            !
!
! TEST VARIOUS INDEXING MODES FOR FETCHING ADDRESSES FROM TABLE
!
CMPOK6   BIN                   ! BACK TO GOOD OL' BIN MODE
         LDM  R36,=374,377     ! = -4
         STM  R36,R34          ! COPY
         LDMD R34,X34,IXTABLE2 ! 34-35= IXTABLE3 (READ FROM IXTABLE0)
         LDMI R36,X36,IXTABLE2 ! 36-37= IXTABLE1 (READ FROM IXTABLE3 VIA IXTABLE0)
         LDMD R32,=IXTABLE3    ! 32-33= IXTABLE1 (READ FROM IXTABLE3)
         LDMD R32,R32          ! 32-33= IXTABLE2 (READ FROM IXTABLE1)
         LDMI R30,=IXTABLE2    ! 30-31= IXTABLE1 (READ FROM IXTABLE3 VIA IXTABLE2)
         LDMI R30,R30          ! 30-31= IXTABLE3 (READ FROM IXTABLE2 VIA IXTABLE1)
         LDM  R40,R30          ! IXTABLE3 IXTABLE2 IXTABLE3 IXTABLE1
         CMMD R40,=IXTABLE0    ! MATCH CONTENTS OF TABLE?
         JZR  CMPOK7           ! JIF YES, OK
         GTO CPUERR+5          ! CPU ERROR
!
! TEST R* (R1 INDIRECT THROUGH R0) REGISTER OPERATIONS
! AND TEST WRITING ALTERNATING 1s AND 0s TO THE REGISTERS
! AT THE SAME TIME (125s AND 252s).
!
CMPOK7   LDB  R0,=20           ! POINT R0 AT R20
         LDM  R*,=125,77       ! R20-21 = 125 077
         LDB  R0,=107          ! POINT R0 AT "R107"
MBREGLOP SBB  R0,=10           ! POINT R0 AT R77,R67,R57,R47,R37,R27
         CMB  R0,=17           ! R0==17?
         JZR  MBREGEND         ! JIF YES, END LOOP
         STM  R20,R*           ! "STM R20,R77" (R77=125, R0=77) (MULTI-BYTE REGISTER "WRAP-AROUND")
         CLM  R20              ! R20-21 = 0, SET Z FLAG
         JNZ  CPUER+6J         ! JIF Z NOT SET, ERROR
         LDM  R20,R*           ! "LDM R20,R77" R20-1 = 125 077 (MULTI-BYTE REG WRAP-AROUND)
         CMM  R20,=125,77      ! RIGHT?
         JZR  MBREGLOP         ! JIF YES, LOOP TO NEXT "CROSS-MULTI-BYTE-REGISTER BOUNDARY"
CPUER+6J GTO CPUERR+6          !
!
MBREGEND LDB  R0,=320          ! R0=320 (INDIRECT TO R20)
INDIRLOP LDB  R*,=125          ! R20=125
         CMB  R*,=125          ! DID IT WORK?
         JNZ  CPUERRH          ! JIF NO, CPU ERROR H
         ICB  R0               ! ++ INDIRECT POINTER
         JNZ  INDIRLOP         ! TEST THROUGH R77, LOOP
!
         CMB  R77,=125         ! LAST ONE RIGHT?
         JNZ  CPUERRH          ! JIF NO, ERROR H
         LDM  R70,R10          ! SAVE R10-17
         STM  R60,R10          ! 125s TO R10-17
         CMM  R60,R10          ! DID THEY WORK?
         JNZ  RSTOR10          ! JIF NO
         STM  R70,R10          ! RESTORE R10-17
INDIRLP2 DCB  R0               ! DEC INDIR TO R77->R20
         LDB  R*,=252          ! PUT A 252 IN THE INDIR REG
         CMB  R*,=252          ! DID IT WORK?
         JNZ  CPUERRH          ! JIF NO, CPU ERROR H
         CMB  R0,=320          ! LAST REG?
         JNZ  INDIRLP2         ! JIF NO, LOOP
!
         CMB  R20,=252         ! LAST ONE RIGHT?
         JNZ  CPUERRH          ! JIF NO, ERROR H
         LDM  R70,R10          ! SAVE R10-17
         STM  R60,R10          ! WRITE 252s TO R10-17
         CMM  R60,R10          ! DID THEY TAKE?
         JNZ  RSTOR10          ! JIF NO, RESTORE R10-17 AND ERROR H
         STM  R70,R10          ! RESTORE R10-17
         JMP  CPUTSTE          ! CONTINUE ON...
!
RSTOR10  STM  R70,R10          ! RESTORE R10-17
CPUERRH  LDB  R75,=110         ! 'H' CPU LETTER
         GTO CPUERR            !
!
! TEST E REGISTER (EVEN THOUGH WE DEPENDED UPON IT DURING
! THE BCD MODE ARITHMETIC AND SHIFTING TESTS ABOVE...)
!
CPUTSTE  LDB  R75,=111         ! 'I' ERROR LETTER
         BCD                   ! BACK TO BCD MODE
         LDM  R45,=0,177,23    ! 45-47= 000 177 023
         CLE                   ! E=0
ICILOP   ICE                   ! E=1,2,4,10,0 (FOR EACH LOOP)
         ELM  R45              ! (SEE BELOW FOR EACH LOOP RESULT)
! 45-47= 001 360 067 E=01
!        022 000 177 E=03
!        044 001 360 E=07
!        110 022 000 E=17
!        200 044 001 E=00
!
         JEN  ICILOP           ! JIF NOT DONE (AND TEST JEN)
!
         CMM  R45,=200,44,1    ! RIGHT?
         JNZ  CPUERI           ! JIF NO, ERROR I
         LDM  R45,=301,216,0   ! STARTING VALUE FOR DCE TEST
!
! MULTI-BYTE RIGHT SHIFTING GOES TO THE NEXT *LOWEST*
! MULTI-BYTE BOUNDARY, UNLIKE MOST OTHER OPERATIONS.
!
         STM  R45,R40          ! PUT THEM IN R40-42
         CLE                   ! E=0
DCELOP   DCE                   ! E=17,0,13,15,7 (FOR EACH LOOP)
         ERM  R42              ! (SEE BELOW FOR EACH LOOP RESULT)
! 40-42= 354 010 360 E=01
!      = 216 000 017 E=14
!      = 010 360 260 E=16
!      = 000 017 333 E=10
!      = 360 260 175 E=00
         JEZ  EZR              ! JIF DONE (AND TEST JEZ)
         JMP  DCELOP           ! LOOP TIL DONE
!
EZR      LDM  R45,R40          ! GET RESULTS
         CMM  R45,=360,260,175 ! RIGHT?
         JZR  CMPOK8           ! JIF YES
CPUERI   GTO CPUERR            ! ELSE ERROR I
!
! TIME TO CHECK THE CY/NC OPERATION
!
CMPOK8   BIN                   ! BACK TO BELOVED BIN MODE
         CLB  R22              ! CLEAR SHOULD SET NC
         JNC  NCOK             ! JIF TRUE, OK
         JCY  CPUERJ           ! IF CY SET, ERROR!
NCOK     DCB  R#               ! DEC TO 377 & NC
         JCY  CPUERJ           ! JIF CY SET, ERROR
         ICB  R22              ! INC BACK TO 000 AND CY
         JNC  CPUERJ           ! JIF CY NOT SET, ERROR
         ICB  R22              ! INC TO 1 AND NC
         JCY  CPUERJ           ! JIF CY SET, ERROR
         DCB  R22              ! DEC BACK TO 0 AND CY SET
         JNC  CPUERJ           ! JIF CY NOT SET, ERROR
         ELB  R22              ! SHIFT CY (1) INTO 22 AND 0 INTO CY
         JNC  CYOK             ! JIF CY NOT SET, OK
         JMP  CPUERJ           ! ELSE ERROR
!
CYOK     LRB  R#               ! SHIFT 1 INTO CY
         JCY  CYOK2            ! JIF CY SET, OK
CPUERJ   GTO CPUERR+1          ! CPU ERROR J
!
CYOK2    LDM  R24,=177,177     ! 177 = LARGEST POSITIVE BYTE
         JPS  PSOK             ! JIF PS SET, OK
         JNG  CPUERK           ! JIF PS CLEAR (NG), FAIL
PSOK     ICB  R#               ! ++177 = 200 AND OVERFLOW
         JNO  CPUERK           ! JIF OVF NOT SET, ERROR K
         JPS  POSOVF           ! POSITVE WITH OVF SET, EVEN THO MSBIT IS SET
         JMP  CPUERK           !
!
POSOVF   ADB  R25,R24          ! 177+200=377, MSBIT, NC, NOVF
         JNG  NGOK             ! JIF MSBIT SET WITH NO OVFG, OK
         JMP  CPUERK           ! ERROR K
!
NGOK     LLB  R24              ! R24=0, CY AND OVF SET, MSBIT CLEAR
         JPS  CPUERK           ! JIF POS (MSBIT CLEAR, BUT OVF SET, SO *NOT* POSITIVE)
         LRB  R25              ! 377->177 + MSB=0 CY=1 OVF=0
         JNG  CPUERK           ! JIF NEG, ERROR K
         JNO  OVFOK            ! JIF NO OVF, OK
CPUERK   GTO CPUERR+2          !
!
! AND TEST ZR/NZ,RZ,RN,LZ,LN AND ICB
!
OVFOK    LDB  R23,=100         ! FIRST TEST STARTING AT 100
ITSLOOP  JZR  ITS0             !
         JNZ  ITSNOT0          !
ITS0     ICB  R#               !
ITSNOT0  JOD  ITSODD           !
         JEV  ITSEVN           !
         JMP  CPUERL           !
!
ITSODD   ICB  R#               !
ITSEVN   JRZ  ITSRZ            !
         JRN  ITSRN            !
         JMP  CPUERL           !
!
ITSRN    ICB  R#               !
ITSRZ    JLN  ITSLN            !
         JLZ  ITSLZ            !
         ICB  R#               !
ITSLN    CMB  R23,=100         !
         JNZ  CPUERL           !
         CLB  R23              ! NOW TEST STARTING AT 0
         JMP  ITSLOOP          !
!
ITSLZ    CMB  R23,=3           ! RIGHT?
         JZR  CMPOK9           ! JIF YES
CPUERL   GTO CPUERR+3          ! CPU ERROR L
!
CMPOK9   CLB  R30              !
         LDM  R56,=CPUVAL      !
         LDMD R40,=CPUVAL      !
         JZR  CPUERM           !
         JCY  CPUERM           !
         ADMD R40,=CPUVAL      !
         JCY  CPUERM           !
         JRZ  CPUERM           !
         JLZ  CPUERM           !
         JOD  CPUERM           !
         JNO  CPUERM           !
         JNG  CPUERM           !
         LRM  R47              !
         JNG  CPUERM           !
         ADMD R40,R56          !
         SBMD R40,=CPUVAL      !
         SBMD R40,R56          !
         JNZ  CPUERM           !
         JNC  CPUERM           !
         JRN  CPUERM           !
         JLN  CPUERM           !
         JNG  CPUERM           !
         NCB  R40              !
CMMDLOP  CMMD R40,=CPUVAL      !
         JZR  CPUERM           !
         JCY  CPUERM           !
         JRZ  CPUERM           !
         JLZ  CPUERM           !
         JPS  CPUERM           !
         DCB  R30              !
         JNZ  CMMDLOP          !
         NCM  R41              !
         JCY  CPUERM           !
         JEV  CPUERM           !
         JPS  CPUERM           !
         ANMD R40,R#           !
         JCY  CPUERM           !
         JNG  CPUERM           !
         ANMD R40,=CPUVAL      !
CMMDLOP2 CMMD R40,R56          !
         JNZ  CPUERM           !
         JNC  CPUERM           !
         JRN  CPUERM           !
         JLN  CPUERM           !
         JPS  KEEPTRKN         !
CPUERM   GTO CPUERR+4          ! CPU ERROR M
!
KEEPTRKN DCB  R30              !
         JNZ  CMMDLOP2         !
!
! TEST STMD AND STMI
!
         LDM  R34,=TSTBF1      !
         LDM  R36,=TSTBF2      !
         STMD R36,R34          !
         STMI R34,R34          !
         LDM  R32,=TSTBF3      !
         STMI R32,=TSTBF2      !
         CLM  R30              !
         ICM  R30              !
         STBD R30,X30,TSTBF0   !
         LLM  R30              !
         TCM  R30              !
         STBD R30,=TSTBF0      !
         STMI R36,X30,TSTBF2   !
         TCB  R31              !
         LDM  R50,R30          !
         STMD R50,R6           !
         LDMD R60,R6           !
         XRM  R60,R30          !
         JNZ  JJNZ             !
         CMMD R50,=TSTBF0      !
         JZR  JJZR             !
JJNZ     ORM  R61,R60          !
         LDB  R75,R67          !
!
         BYT  142,251          ! LDM R42=
         DEF  MSGRAMBA         ! "RAM BAD! "
         BYT  16,11            ! LEN 1, LEN 2
         DEF  MSGCPUBA         ! "CPU BAD! N OR "
!
         STM  R75,R32          ! ****WHAT????******
         GTO CPUERR-           !
!
! TEST PUMD/I +/-
!
JJZR     LDM  R26,=TSTBF1      !
         LDM  R24,=TSTBF2      !
         PUMD R24,+R26         !
         PUBI R24,-R26         !
         POBI R31,+R26         !
         POMD R32,-R26         !
         PUMD R24,-R26         !
         PUMI R24,+R26         !
         POMI R34,-R26         !
         POMD R36,+R26         !
         CMM  R26,=TSTBF1      !
         JNZ  CPUERO           !
         LDM  R42,R32          !
         CMB  R42,R31          !
         JNZ  CPUERO           !
!
         BYT  311              ! CMM R42,=
         DEF  TSTBF2           !
         DEF  TSTBF2           !
         DEF  TSTBF2           !
!
         JNZ  CPUERO           !
         CMMD R42,=TSTBF0      !
         JZR  CMPOK10          !
CPUERO   GTO CPUERR+6          ! CPU ERROR O
!
! TEST SAD/PAD
!
CMPOK10  BCD                   !
         LDB  R75,=120         ! 'P' ERROR LETTER
         LDM  R20,R6           ! SAVE R6 STACK PTR
         LDM  R44,=200,1,251,200 !
         TSM  R44              !
         JRN  CPUERP+          !
         JLZ  CPUERP+          !
         ARP  R45              !
         TSB  R46              !
         SAD                   !
         BIN                   !
         ADB  R47,R44          !
         SAD                   !
         LDB  R47,=15          !
         BCD                   !
         ARP  R30              !
         DRP  R30              !
         PAD                   !
         JNZ  CPUERP+          !
         JNC  CPUERP+          !
         JRN  CPUERP+          !
         JLN  CPUERP+          !
         JOD  CPUERP+          !
         JPS  CPUERP+          !
         JNO  CPUERP+          !
         ADB  R#,R#            !
         PAD                   !
         JZR  CPUERP+          !
         JCY  CPUERP+          !
         JRZ  CPUERP+          !
         JLZ  CPUERP+          !
         JEV  CPUERP+          !
         JPS  CPUERP+          !
         JNO  NOVFOK           !
         JMP  CPUERP+          !
!
NOVFOK   ADB  R#,R#            !
         BIN                   !
         CMM  R46,=60,215      ! RIGHT?
         JNZ  CPUERP+          ! JIF NO, ERROR
         CMM  R20,R6           ! STACK OK?
         JZR  CMPOK11          ! JIF YES
CPUERP+  STM  R20,R6           ! RESTORE R6 STACK PTR
         GTO CPUERR            ! CPU ERROR P
!
! TEST JSB/JSB X/RTN
!
CMPOK11  JSB  =JSBTST          ! TEST JSB
RTNADDR  CMM  R#,=RTNXADDR     ! MAKE IT THROUGH JSB X, TOO?
         JNZ  CPUERR+1         ! JIF NO, ERROR Q
         CMM  R20,R#           ! STACK OKAY?
         JNZ  CPUERQ           ! JIF NO, ERROR Q
         JMP  CPUOK            ! ELSE DONE WITH CPU TEST
!
JSBTST   POMD R66,-R6          ! GET RTN ADDRESS
         CMM  R66,=RTNADDR     ! RIGHT?
         JNZ  CPUERQ           ! JIF NO, ERROR Q
         PUMD R66,+R6          ! PUSH IT BACK
         LDM  R66,=5,0         ! INDEXED JSB
         JSB  X66,CPUERQ       ! SHOULD GO TO JSBXTST
RTNXADDR RTN                   ! AND RTN TO RTNADDR
!
CPUERQ   STM  R20,R6           ! RESTORE STACK PTR
         JMP  CPUERR+1         ! CPU ERROR Q
!
JSBXTST  CMM  R#,R#            ! STACK OK?
         JNZ  CPUERQ           ! JIF NO, ERROR Q
         POMD R#,-R6           ! POP RETURN ADDRESS
         CMM  R66,=RTNXADDR    ! RIGHT?
         JNZ  CPUERQ           ! JIF NO, ERROR Q
         PUMD R66,+R6          ! PUSH RTN ADDR BACK
         RTN                   ! DONE
!
CPUERR+6 ICB  R75              ! MODIFY ERROR LETTER
CPUERR+5 ICB  R75              !
CPUERR+4 ICB  R75              !
CPUERR+3 ICB  R75              !
CPUERR+2 ICB  R75              !
CPUERR+1 ICB  R75              !
CPUERR   LDM  R36,=MSGCPUBA    ! "CPU BAD! "
         LDM  R34,=11,0        ! LEN 1, LEN 2
CPUERR-  JSB  =OUTBEEP         ! SPIT MSG AND BEEP
         RTN                   !
!
CPUOK    LDM  R36,=MSGCPUOK    ! "CPU OK"
         LDM  R34,=6,0         ! LEN 1, LEN 2
PROMPT   LDBD R21,=PRMODE      ! PRINTING?
         JPS  OUTLIN+          ! JIF NO, JUST DISPLAY MSG
         JSB  =DOHOOKS         ! MAKE SURE WE HAVE HOOKS
         JMP  PLINDON          !
!
OUTLIN+  LDB  R75,=15          ! FILL WITH CR CHAR
OUTLINGO JSB  =OUTLINE         ! DISPLAY THE MESSAGE
PLINDON  CLB  R74              !
         STBD R74,=ERNUM#      ! CLEAR ANY ERNUM#
         RTN                   !
!
! THE FOLLOWING TABLE IS USED TO TEST CPU'S INDEXED ADDRESSING MODE
!
IXTABLE0 DEF  IXTABLE3         !
IXTABLE1 DEF  IXTABLE2         !
IXTABLE2 DEF  IXTABLE3         !
IXTABLE3 DEF  IXTABLE1         !
! ************************************************************
! XRAM TEST
! ************************************************************
         BYT  241              !
XRAM.    LDM  R56,=XRAMOKMS    ! ADDRESS OF TEST VALUES AND "XRAM OK"
         JMP  RAM-             !
! ************************************************************
! RAM TEST - checks the RAM controller IC and the RAM ICs by
! filling the RAM ICs with alternate 1s and 0s and reading
! back the result to see if the contents have been changed.
! If an error is found in one bit or only a few bits at a
! particular address, the message will indicate that the
! corresponding RAM IC is bad. If an error is found in all 8
! bits at a particular address, the message will indicate that
! the RAM controller is bad.
! ************************************************************
         BYT  241              !
RAM.     LDM  R56,=RAMOKMSG    ! ADDRESS OF TEST VALUES AND "RAM OK"
RAM-     BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R75,=101         ! 'A' ERROR LETTER
         LDM  R30,=0,3         ! R30=NO ERRORS YET, R31=RETRY COUNTER
         POMD R40,+R56         ! LOAD TEST VALUES & "RAM OK" ARGS
         STM  R42,R32          ! SET FOR MESSAGE OUTPUT
!
! *** The next two lines load:
!   R22-3 = 100003 (START OF RAM + 3)
!   R24-5 = 100000 (START OF RAM)
!   R26-7 = 140000 [END OF TEST RANGE] (END OF FIRST 16K OF RAM)
!
         POMD R42,+R56         ! GET R22-R26 VALUES
         STM  R42,R22          ! SET THEM
!
         LDM  R42,R40          ! RIPPLE-LOAD R40-47 WITH 125,252,125,252,125,252,125,252
RAMLOOP1 JSB  =!               ! SAVE AND WRITE TEST BYTES
         JSB  =RDRAMRES        ! READ TEST BYTES AND RESTORE
         JZR  RAMOK            ! JIF NO PROBLEM
         CMM  R24,=0,300       ! AT 140000 (END OF 16K)?
         JNZ  RAMOK            ! JIF NO
         CMB  R30,=377         ! ALL BITS BAD (NO 16K RAM CART)?
         JNZ  RAMRETRY         ! JIF NO, TRY AGAIN
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGORXRA         ! " OR XRAM BAD! "
         BYT  7,15             ! LEN 1, LEN 2
         DEF  MSGNOXRA         ! "NO XRAM"
!
         GTO OUTBEEP+          ! SPIT "NO XRAM OR XRAM BAD! " MESSAGE AND BEEP
!
RAMRETRY DCB  R31              ! DEC RETRY COUNTER
         JNZ  RAMLOOP1         ! JIF NOT OUT OF RETRIES, LOOP
!
RAMOK    JSB  =!               ! SAVE RAM AND WRITE TEST VALUES
         LDMD R66,R22          ! SAVE TWO BYTES FROM ABOVE THE SAVWRRAM TEST AREA
         STMD R46,R22          ! WRITE TEST BYTES THERE (THESE ARE BIT-FLIPPED FROM SAVWRRAM BYTES)
         LDM  R20,=200,4       ! DELAY COUNTER FOR MAKING SURE
RAMWAIT  DCM  R#               !   DYNAMIC RAM RETAINS ITS CONTENTS
         JNZ  RAMWAIT          !   OVER TIME (GETS REFRESHED OK)
!
         POBD R#,+R#           ! GET THE TWO BYTES, ONE AT A TIME,
         POBD R21,+R#          !   THAT WERE WRITTEN
         CMM  R26,R#           ! REACHED END OF TEST RANGE?
         JCY  NOPASTEN         ! JIF NO, HAVEN'T GONE OFF END OF 16K
         LDB  R21,=252         ! FORCE CORRECT VALUE FOR "PAST END OF RAM" BYTE
NOPASTEN XRM  R20,R46          ! XOR WRITTEN VALUES INTO READ VALUES
         ORB  R20,R21          ! OR TWO BYTE RESULTS TOGETHER
         JZR  RAMDYNOK         ! JIF ALL OKAY
!
         ORB  R30,R20          ! OR BAD RESULTS INTO RUNNING TALLY
         LDB  R75,=102         ! 'B' ERROR LETTER
RAMDYNOK PUMD R66,-R22         ! RESTORE TWO TEST BYTES
         JSB  =RDRAMRES        ! READ AND RESTORE THE THREE BYTES
         LLM  R22              ! 2*PTR
         ICM  R22              ! +1
         ORM  R22,R24          ! OR-IN START-PTR
         CMM  R22,R26          ! PAST END?
         JNC  RAMOK            ! JIF NO, KEEP TESTING
!
RAM82ND  LDM  R20,R24          ! COPY R24 TEST PTR
RAM8     LDMD R60,R20          ! SAVE 8 BYTES THERE
         PUMD R40,+R20         ! PUSH ALTERNATING 1s AND 0s TO ALL 8 BYTES
         POMD R50,-R20         ! READ THEM BACK
         PUMD R60,+R20         ! RESTORE THE ORIGINAL VALUES
         XRM  R50,R40          ! THOSE EIGHT BYTES WRITE/READ OKAY?
         JZR  EIGHTOK          ! JIF YES, OK
         ORM  R51,R50          ! RIPPLE-OR ALL OF R50-57 TOGETHER
         ORB  R30,R57          ! THEN OR IT INTO THE "ERROR BITS"
         LDB  R75,=103         ! 'C' ERROR LETTER
EIGHTOK  CMM  R20,R26          ! DOES THAT 8-BYTE TEST GO PAST END OF RAM?
         JNC  RAM8             ! JIF NO, LOOP AND DO MORE
         NCM  R40              ! FLIP THE ALTERNATING 1s AND 0s TEST BYTE PATTERNS
         JEV  RAM82ND          ! JIF SECOND TIME, DO IT AGAIN
!
         LDBD R40,=ROMFL       ! GET REASON FOR CALL (=340 AT PWO STARTUP, OTHERWISE 0-13)
         JNG  INPWOCAL         ! JIF IN POWER-ON STARTUP CALL
TST30ERR TSB  R30              ! ANY ERRORS?
         JNZ  ERRSKP           ! JIF YES
         JSB  =PROMPT          ! ELSE SPIT "RAM OK"
         RTN                   ! DONE
!
! THE FOLLOWING SAVES 5 BYTES OF TSTBUF, BUT THEN
! BLOWS AWAY THE LAST 2 OF THOSE WITH PRMODE AND CRTBYT,
! SO REALLY ONLY THE FIRST 3 BYTES OF TSTBUF GET PRESERVED.
! THIS SAVES ALL RAM LOCATIONS THE SVCROM CARES ABOUT,
! THEN TESTS ALL OF RAM BY WRITING 2 BYTES OF ADDRESS OF
! THE LOWER BYTE BEING TESTED TO 2 BYTES OF RAM, THEN READS
! THEM ALL BACK, MAKING SURE THE ADDRESSES READ BACK STILL
! MATCH THE ADDRESS AT WHICH THEY WERE WRITTEN.
!
INPWOCAL LDMD R43,=TSTBUF      ! R43-47 = FIRST 5 BYTES OF TSTBUF
         STM  R43,R40          ! R40-44 =     "     "       "
         LDBD R43,=PRMODE      ! R43 = PRMODE
         LDMD R44,=CRTBYT      ! R44-47 = CRTBYT & CRTRAM
         LDBD R70,=CRTWRS      ! R70 = CRTWRS
         LDBD R71,=ROMFL       ! R71 = ROMFL
         LDBD R72,=SVCWRD      ! R72 = SVCWRD
         POMD R50,-R6          ! POP 8 OFF THE RTN STACK
         POMD R60,-R6          ! POP 8 MORE OFF RTN STACK
!
         LDM  R20,R24          ! COPY RAM PTR
ADRCHECK STMD R#,R20           ! STORE THE ADDRESS IN RAM *AT* THE ADDRESS
         ICM  R#               ! ++ PTR
         ICM  R#               !   BY 2
         CMM  R#,R26           ! PAST END OF TEST RAM?
         JNC  ADRCHECK         ! JIF NO, LOOP TIL DONE
!
ADRREAD  POMD R22,-R20         ! GET PREV BYTES WRITTEN
         XRM  R22,R20          ! XOR ADDRESS INTO ADDRESS (WE HOPE!)
         JZR  ADRMATCH         ! JIF MATCHES, OK
!
         ORB  R22,R23          ! OR ADDRESS BYTES TOGETHER (COMBINE BITS)
         ORB  R30,R22          ! OR THE BITS INTO ERROR BITS
         LDB  R75,=104         ! 'D' ERROR LETTER
ADRMATCH CMM  R20,R24          ! REACHED BEGINNING OF TEST REGION?
         JNZ  ADRREAD          ! JIF NO, LOOP UNTIL WE DO
!
         STBD R43,=PRMODE      ! RESTORE PRMODE
         STMD R44,=CRTBYT      ! RESTORE CRTBYT & CRTRAM
         STMD R40,=TSTBUF      ! RESTORE TSTBUF (ONLY FIRST 3 BYTES ARE GOOD)
         STBD R70,=CRTWRS      ! RESTORE CRTWRS
         STBD R71,=ROMFL       ! RESTORE ROMFL
         STBD R72,=SVCWRD      ! RESTORE SVCWRD
         PUMD R60,+R6          ! PUSH 8 BACK ONTO RTN STACK
         PUMD R50,+R6          ! PUSH 8 MORE BACK ON RTN STACK
         JMP  TST30ERR         !
!
ERRSKP   CMB  R30,=377         ! ALL BAD?
         JNZ  BADRAM           ! JIF NO, SHOW SPECIFIC CHIPS
         LDM  R32,=MSGCTRLR    ! " CONTROL BAD! "
         LDB  R35,=16          ! LEN 2
         JSB  =OUTBEEP         ! SPIT MSG & BEEP
         RTN                   !
!
BADRAM   LDM  R32,=MSGBAD!     ! " BAD! "
         LDM  R35,=6           ! LEN 2
         ICB  R34              ! "RAM " -> "XRAM n" (FOR n=0 TO 7)
         ICB  R34              !
RAMFLOOP POMD R42,+R36         ! ADVANCE TO NEXT "XRAM n" STRING
         LRB  R30              ! SHIFT FAILURE BITS OVER
         JNC  RAMNOBEP         ! JIF NO FAILURE IN THIS BIT POSITION
         JSB  =OUTBEEP         ! ELSE SPIT MSG AND BEEP
RAMNOBEP TSB  R30              ! FOUND ALL BAD RAMS YET?
         JNZ  RAMFLOOP         ! JIF NO, KEEP GOING
         RTN                   ! DONE
!
! ************************************************************
! EXAMPLE (WHY FIRST TIME WORKS WHEN R24=100000)
!  L-1  XXX  77777
!  L+0  AAA 100000
!  L+1  BBB 100001
!  L+2  CCC 100002
!  L+3  DDD 100003
!
! SAVWRRRAM POBD R55,-R24 ! PRE-DECS R24 TO L-1, GETS XXX INTO R55
!           LDMD R55,R24  ! SETS R55=XXX, R56=AAA, R57=BBB
!           STMD R45,R24  ! SETS L-1 (ROM) TO R45 (252), L+0 TO R46 (125), L+1 TO R47 (252)
!           PUBD R45,+R24 ! SETS (AGAIN) L-1 (ROM) TO R45, INCS R24 BACK TO L+0
!
! THEN, A WHILE LATER, RDRAMRES READS AND TESTS THE RESULTS BY:
!
! RDRAMRES  POBD R20,+R24 ! GETS L+0 (125) TO R20, R24=L+1
!           POBD R21,+R24 ! GETS L+1 (252) TO R21, R24=L+2
!           PUMD R55,-R24 ! PRE-DECS R24 BY 3 TO L-1
!                         !   AND SETS L-1=XXX (DOESN'T AFFECT ROM, THE FIRST TIME THROUGH)
!                         !   AND SETS L+0=AAA (R56) AND L+1=BBB (R57)
!           POBD R55,+R24 ! READS L-1, INCS R24 TO L+0
!
! SO, IT APPEARS THAT THE RAM TEST, CALLING THESE TWO OVER AND OVER,
! NEVER ADVANCES R24, BUT KEEPS TESTING THE FIRST TWO BYTES OF RAM
! OVER AND OVER, WITH A DELAY BETWEEN THE SAVE/WRITE AND THE READ/RESTORE
! TO TEST WHETHER THE DYNAMIC RAM REFRESH IS WORKING.
!        POBD R55,-R24         !
         LDMD R55,R24          ! SAVE THOSE 3 BYTES
         STMD R45,R24          ! WRITE TEST BYTES THERE (ALTERNATING 1s AND 0s)
         PUBD R45,+R24         ! WRITE BYTE AND INCREMENT PTR
         RTN                   !
!
RDRAMRES POBD R20,+R24         ! GET TWO TEST BYTES, INC'ING PTR
         POBD R21,+R24         !
         PUMD R55,-R24         ! RESTORE ORIGINAL BYTES (SAVED ABOVE)
         POBD R55,+R24         ! RE-READ BYTE AND MOVE PTR BACK TO STARTING POINT
         XRM  R20,R46          ! XOR WHAT WAS WRITTEN INTO WHAT WAS READ
         ORB  R20,R21          ! COMBINE THE TWO BYTES (SHOULD BE 0)
         ORB  R30,R20          ! OR THE RESULT IN THE "RUNNING TALLY"
         RTN                   !
!
! ***
!
RAMOKMSG BYT  125,252          ! RAM TEST VALUES (01010101, 10101010)
         DEF  MSGSPOK          ! " OK"
         BYT  3,3              ! LEN 1, LEN 2
         DEF  MSGRAM           ! "RAM"
!
         BYT  3,200            ! R22 100003
         BYT  0,200            ! R24 100000 (START OF RAM)
         BYT  0,300            ! R26 140000 (END OF 16K RAM)
!
XRAMOKMS BYT  125,252          ! RAM TEST VALUES (01010101, 10101010)
         DEF  MSGSPOK          ! " OK"
         BYT  4,3              ! LEN 1, LEN 2
         DEF  MSGXRAM          ! "XRAM"
!
         BYT  3,300            ! R22 140003
         BYT  0,300            ! R24 140000 (START OF XRAM)
         BYT  377,376          ! R26 177377 (END OF 32K RAM)
!
! ************************************************************
! ROM TEST ENTRY FROM PWO AND HEAT TEST
! ************************************************************
ROM.1    LDB  R70,=1           ! DON'T TEST SECONDARY CHECKSUM
         JMP  ROM-             !
! ************************************************************
! ROM TEST - checks the ROM ICs by performing a signature
! analysis on their contents. This means that the contents are
! treated as data; the test manipulates this data and checks
! whether the correct results are obtained.
! ************************************************************
         BYT  241              ! ATTRIBUTES
ROM.     CLB  R70              ! (KEYWORD, CYCLE, SYSTEM) TEST SECONDARY CHECKSUM
ROM-     BIN                   !
         CLB  R60              !
         LDBD R71,=ROM60000    ! GET OUR ROM#
         LDM  R66,=ROMATBL     ! PTR TO TABLE OF ROM ADDRESSES
         LDM  R36,=MSGROM      ! "ROM 0"
ROMTLP   LDM  R34,=5,6         ! LEN 1, LEN 2
         JSB  =CHKSOME         ! TEST A ROM
         POMD R53,+R36         ! SKIP THAT ROM MSG
         CMM  R36,=MSGROM3     ! "ROM 3"?
         JNZ  ROMTLP           ! JIF NO, KEEP GOING
         LDM  R30,=GINTEN      ! ENABLE INTERRUPTS
         JMP  XROM+            !
! ************************************************************
! TESTS ALL PLUG-IN ROMS
! ************************************************************
         BYT  241              !
XROM.    CLB  R70              !
         LDM  R30,=1,376       ! START ROM#, # OF ROMS TO TEST
         CLB  R60              !
XROM+    BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDBD R71,=ROM60000    ! GET OUR ROM#
         CLB  R72              ! INIT TO NO XROMS FOUND
! ************************************************************
! The following loop copies code (at RAMCODE) to RAM (RECBUF).
! That code in RAM will be used to checksum bank-selected ROMs,
! which can't be read directly while THIS ROM is mapped in.
! ************************************************************
         LDM  R20,=RECBUF      ! ADDRESS OF TARGET IN RAM
         LDM  R22,=RAMCODE     ! ADDRESS OF CODE TO COPY THERE
COPYCODE POBD R24,+R22         ! GET NEXT BYTE OF CODE
         STBD R24,R20          ! STORE IT TO RAM
         POBD R25,+R20         ! GET THE BYTE BACK FROM RAM
         CMB  R25,R24          ! COPIED OK?
         JNZ  RAMFAILD         ! JIF NO, RAM FAILED!
         CMM  R22,=ROMATBL     ! REACHED END OF CODE TO COPY?
         JNC  COPYCODE         ! JIF NO, KEEP GOING
!
         TSB  R30              ! JUST CALL IT?
         JNZ  ASCROM#          ! JIF NO
         LDM  R20,=RECBUF      ! ADDRESS OF CODE IN RAM
         JSB  X20,ZRO          ! CALL IT RIGHT NOW
         JMP  ROMFIN           ! FINISH UP
!
ASCROM#  LDM  R65,=60,60,60    ! ASCII "000"
         STMD R65,=TSTBUF      ! STORE IN RAM
NXTROM   JSB  =INCROM#         ! INCREMENT ASCII ROM#
! *** NOW SET UP FOR MESSAGE TO OUTPUT
         LDMD R53,=MSGXROM     ! "XROM "
         LDM  R36,=TSTBF0      ! ADDRESS OF TMP BUFFER
         STM  R36,R34          ! COPY IT FOR POSSIBLE OUTST
         PUMD R53,+R34         ! PUSH "XROM " TO RAM BUFFER
         PUMD R65,+R34         ! PUSH ASCII ROM# TO RAM BUFFER
         LDM  R34,=10,6        ! LEN 1, LEN 2
! *** NOW GO TEST THE ROM
         LDM  R20,=RECBUF      ! ADDRESS OF ROM TEST IN RAM
         JSB  X20,ZRO          ! CALL ROM TEST IN RAM
! *** NOW MOVE ON TO THE NEXT ROM
         ICB  R30              ! ++ ROM#
         DCB  R31              ! -- REMAINING ROM COUNT
         JNZ  NXTROM           ! JIF NOT DONE
         TSB  R72              ! ANY XROMS FOUND?
         JNZ  ROMFIN           ! JIF YES
!
! *** NO XROMS FOUND, SAY SO
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGXROM          ! "XROM"
         BYT  3,4              ! LEN 1, LEN 2
         DEF  MSGNOXRA         ! "NO "
!
         JMP  GO_OUTBP         !
!
ROMFIN   STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         TSB  R60              ! ROM 000?
         JZR  ROM000           ! JIF YES
         STB  R60,R74          ! COPY ROM# AS ERROR FLAG
         STBD R60,=ERNUM#      ! AND SAVE IN ERNUM#
ROM000   RTN                   !
!
! *** Tried to copy code to RAM but it didn't take
!
RAMFAILD BYT  142,251          ! LDM R42,=
         DEF  MSGXROMA         ! "XROM TEST ABORTED!"
         BYT  11,22            ! LEN 1, LEN 2
         DEF  MSGRAMBA         ! "RAM BAD! "
!
         TSB  R30              ! ROM 000?
         JNZ  GO_OUTBP         ! JIF NO
         ICM  R42              ! CHANGE "XROM" TO "ROM"
GO_OUTBP JSB  =OUTBEEP+        ! SPIT "RAM BAD! [X]ROM TEST ABORTED!" AND BEEP
         RTN                   !
!
! ************************************************************
! This is the code which gets copied to RECBUF, so that THIS
! (Service) ROM can be de-selected and another bank-selected
! ROM selected, checksummed, and the THIS (Service) ROM
! re-selected, so control can then return to this ROM.
!
! NOTE: Some of this code is ALSO run directly from the ROM
! and I believe R70=0 means running in RAM, R70!=0 means
! running in ROM. I *think*.
! ************************************************************
RAMCODE  LDB  R75,=101         ! 'A' ERROR
         LDM  R20,=ROM60000    ! BASE OF PLUG-IN ROM
         STBD R30,=RSELEC      ! SELECT THE DESIRED ROM
         CMMD R30,R20          ! IS IT THERE?
         JZR  ROMTHERE         ! JIF YES
         TSB  R30              ! ROM 000?
         JZR  BADROM           ! JIF YES
         STBD R71,=RSELEC      ! RE-SELECT THIS ROM
         RTN                   !
!
CHKSOME  LDB  R75,=101         ! 'A' ERROR LETTER
         POMD R20,+R66         ! GET NEXT ROM BASE ADDR FROM TABLE
ROMTHERE LDM  R32,R20          ! COPY BASE ADDRESS
         ICM  R32              ! BUMP UP 1 (FOR LATER TEST BELOW AT ROMREAD)
         LDM  R22,R20          ! COPY BASE ADDRESS AGAIN
         STBD R22,=GINTDS      ! DISABLE INTERRUPTS
         ICB  R72              ! INCREMENT COUNT OF ROMS FOUND (FOR SOME CALLERS)
         CLM  R50              ! PRECLEAR CHECKSUMS
         LDM  R26,=377,17      ! # OF WORDS IN ROM - 1
CHKADDLP POMD R46,+R22         ! GET NEXT WORD OF ROM
         ADM  R54,R46          ! ADD TO CHECKSUM (R46->R51 ADDED, R50=R51=0)
         DCM  R26              ! DECREMENT WORD COUNT
         JNZ  CHKADDLP         ! JIF NOT DONE, LOOP
! *** R22 IS POINTING AT LAST 2 BYTES (CHECKSUM) OF ROM
         ADM  R56,R54          ! ADD R54-55 TO R56-57
         NCM  R56              ! NINE'S (ONE'S) COMPLEMENT
         CMMD R56,R22          ! MATCH ROM CHECKSUM?
         JZR  CHKOK1           ! JIF YES
         ICB  R75              ! ++ERROR LETTER
ROM+2LTR ICB  R75              ! ++ERROR LETTER
ROM+1LTR ICB  R75              ! ++ERROR LETTER
BADROM   STBD R71,=RSELEC      ! RE-SELECT THIS ROM
         STB  R71,R60          ! SAVE IT
         LDM  R32,=MSGBAD!     ! " BAD! "
         JSB  =OUTBEEP         ! SPIT MESSAGE AND BEEP
         RTN                   !
!
CHKOK1   TSB  R70              ! DO EXTENDED CHECKSUM TEST (2ND TO LAST WORD OF ROM)?
         JNZ  RTN2ROM          ! JIF NO, DONE
!
! *** THE LAST 2 BYTES OF THE ROM ARE THE 'NORMAL CHECKSUM.
! *** THE 2ND TO LAST 2 BYTES ARE AN ALTERNATE CHECKSUM.
! *** THE FOLLOWING CODE GENERATES THAT ALTERNATE CHECKSUM
! *** AND VERIFIES IT AGAINST THE ROM'S VALUE, *IF* R70 (ABOVE) = 0.
! *** R70 *APPEARS* TO 1 FOR ONLY THE PWO TEST AND THE HEAT TEST.
! *** IT *APPEARS* TO BE 0 FOR ALL OTHER TESTS (ROM AND XROM).
!
         CLM  R46              !
         LDM  R22,=376,17      ! NUMBER-OF-WORDS-IN-ROM -2
WORDLOOP POMD R24,+R20         ! GET NEXT WORD FROM ROM INTO R24-25
         LDB  R40,=20          ! 16 BITS COUNTER
BITLOOP  STM  R46,R26          ! COPY "RUNNING CHECKSUM"
         STM  R27,R45          ! COPY UPPER BYTE TO R45
         BCD                   ! OPERATE ON NYBBLES
         CLE                   ! CLEAR E SO 0 COMES INTO R26 LOW 4 BITS
         ELB  R27              ! SHIFT THE UPPER BYTE OVER 4
!
! *** (the bits shifted into E are discarded, we're just
! *** shifting R27 over 4 bits and shifting 0s into the
! *** lower 4 bit positions.)
!
         BIN                   ! BACK TO BIT OPERATIONS
         ERM  R25              ! GET LSbit FROM R24-25
         ELB  R26              ! WRAP IT INTO LSbit OF R26
         XRB  R45,R26          ! XOR BOTH 26
         XRB  R45,R27          !   AND 27 INTO 45
         JEV  ITSEVEN          ! JIF EVEN (LSBIT==0)
         ADB  R45,=200         ! ELSE ADD A MSBIT=1 INTO 45
ITSEVEN  LLM  R45              ! << R45-47 BY 1
! *** NOTE: the MSbit of R45 gets shifted into R46-47
! *** which will then get used on the next pass to BITLOOP.
         DCB  R40              ! DEC BIT COUNTER
         JNZ  BITLOOP          ! JIF TIL ALL BITS
!
         DCM  R22              ! DEC WORD COUNTER
         JNZ  WORDLOOP         ! LOOP TIL DONE
!
         CMMD R46,R20          ! CHECKSUM MATCH?
         JNZ  ROM+1LTR         ! JIF NO
!
! *** NOW READ THE ROM IN TWO DIFFERENT WAYS AND MAKE SURE
! *** THE BYTES MATCH UP BOTH WAYS.
!
         POMD R43,+R20         ! SKIP R20 PAST 4 CHKSUM BYTES AND 1 BYTE BEYOND THAT
ROMREAD  LDMD R46,R32          ! GET NEXT TWO BYTES OF ROM (AT ODD ADDRESSES)
         POBD R46,+R32         ! BUMP R32 TO 2ND (EVEN ADDRESS) BYTE
         CMBD R47,R32          ! IS NEXT (EVEN ADDRESS) BYTE STILL THE SAME AS WHAT WE READ?
         JNZ  ROM+2LTR         ! JIF NO, ERROR
         POBD R47,+R32         ! BUMP R32 TO 3RD (ODD ADDRESS) BYTE
         CMM  R20,R32          ! R20 AND R32 POINTING TO SAME PLACE?
         JNZ  ROMREAD          ! JIF NO, KEEP CHECKING ROM
!
RTN2ROM  STBD R71,=RSELEC      ! RE-SELECT SERVICE ROM (IN CASE RUNNING IN RAM)
         LDM  R32,=MSGSPOK     ! POINT AT "OK" MESSAGE
         JSB  =PROMPT          ! SPIT IT OUT
         RTN                   ! DONE
!
ROMATBL  DEF  ZRO              ! 1ST SYSTEM ROM BASE ADDRESS
         DEF  ROM20000         ! 2ND    "
         DEF  ROM40000         ! 3RD    "
         DEF  ROM60000         ! 4TH    "
! ************************************************************
! PRINTER TEST - Checks the printer controller and also causes
! the computer to give you a test printout. By inspecting the
! printout, you can determine whether the printer mechanism
! and the rest of the printer circuitry are operating properly.
! POSSIBLE RESULTS:
!   PRINTER TEST ENDS
!   PRINTER OUT OF PAPER
!   PRINTER CONTROL BAD!
! ************************************************************
         BYT  241              ! ATTRIBUTES
PRT.     JSB  =PRT!BUSY        ! WAIT FOR THE PRINTER TO BE NOT-BUSY
         JSB  =TSTPFONT        ! TEST THE PRINTER'S FONT ROM
         LDB  R30,=100         !
         JSB  =CHKPAPER        ! CHECK PAPER'S LOADED AND PWR-DOWN CRT
         LLB  R30              !
         JSB  =CHKPAPER        ! CHECK PAPER'S LOADED AND PWR-DOWN CRT
         JSB  =PRTNULL         ! KILL PRINTER'S MESSAGE LEN
!
! *** FIRST TIME IN, R56 = POINTER TO PRWEDGET (FROM TSTPFONT)
! *** 2ND TIME, R56 = POINTER TO PRWEDGEB
!
WEDGEBOT POMD R40,+R56         ! R56 SET IN TSTPFONT INITIALLY
         JSB  =CHKPAPER        ! CHECK PAPER'S LOADED AND CRT DOWN
         LDB  R31,=30          ! # OF TIMES TO PRINT A WEDGE (24 DEC)
PRLOOP1  JSB  =PRT!BUSY        ! WAIT FOR PRINTER
         LDB  R#,=10           ! SET MESSAGE LEN (10 (8 DEC) BYTES)
         STBD R#,=PRMLEN       !
         STMD R40,=PRDATA      ! WRITE THE 8 BYTES TO THE PRINTER (A WEDGE)
         DCB  R31              ! DEC WEDGE COUNT
         JNZ  PRLOOP1          ! LOOP TIL 24 OF THEM PRINTED
!
         JSB  =PRTBSYER        ! ERROR IF NOT BUSY
         JSB  =PRT!BUSY        ! WAIT FOR NOT BUSY
         LDB  R30,=300         ! PRINTER POWERED-UP, HOME HEAD IN ALPHA
         STBD R30,=PRSTS       ! TELL PRINTER
!
         LLB  R30              ! R30=200
         CMM  R56,=BCOUNTS     ! DONE WITH WEDGE?
         JNZ  WEDGEBOT         ! JIF NO, PRINT BOTTOM HALF OF WEDGES
         CLM  R30              !
         JSB  =CHKPAPER        ! PAPER IN AND CRT DOWN
         JSB  =PRTNULL         ! SET MSG LEN=0
PNEXTLIN JSB  =CHKPAPER        ! PAPER IN AND CRT DOWN
         LDB  R32,=40          ! MSG LEN = 40 (32 DEC)
         STBD R32,=PRMLEN      ! TELL PRINTER CONTROLLER
NOTEOL   STBD R31,=PRDATA      ! PRINT CHARS 0-127, 32 CHARS PER LINE, 4 LINES
         ICB  R31              ! NEXT CHAR
         CMB  R31,=200         ! DONE?
         JZR  PCHRSDON         ! JIF YES
         DCB  R32              ! DEC LINE LEN
         JNZ  NOTEOL           ! JIF NOT EOL
         JSB  =PRTBSYER        ! ERROR IF PRINTER NOT BUSY
         JMP  PNEXTLIN         ! PRINT NEXT LINE
!
PCHRSDON JSB  =CHKPAPER        ! PAPER IN, CRT DOWN
         JSB  =PRTNULL         ! MSG LEN = 0
         LDB  R40,=302         ! UNDERLINED 'B' CHARACTER
         LDM  R41,R40          ! 8 OF THEM IN R40-47
! *** R56 NOW POINTS AT 'BCOUNTS'
PDOMORE  POBD R32,+R56         ! GET NUMBER TO OF B'S TO PRINT
         JSB  =CHKPAPER        ! CHECK PAPER AND CRT DOWN
         STBD R32,=PRMLEN      ! TELL PRINTER MSG LEN
MOREBS   STMD R40,=PRDATA      ! WRITE 8 B'S
         SBB  R32,=10          ! SUB 8 FROM MSG LEN
         JNZ  MOREBS           ! LOOP TIL DONE
         JSB  =PRTBSYER        ! PRINTER SHOULD BE BUSY
         CMM  R56,=ENDPRINT    ! END OF LIST OF LENGTHS OF B'S?
         JNZ  PDOMORE          ! JIF NO, DO MORE
         JSB  =CHKPAPER        ! CHECK PAPER AND CRT DOWN
         JSB  =PRTNULL         ! MSG LEN = 0
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGTEND          ! " TEST ENDS"
         BYT  7,12             ! LEN 1, LEN 1
         DEF  MSGPRTR          ! "PRINTER"
!
OUTLIN+S STM  R#,R32           ! COPY ARGS (PROBABY R42) TO R32-37
         GTO OUTLIN+           ! SPIT MESSAGE
!
! ************************************************************
! WAIT FOR THE PRINTER CONTROLLER TO NOT BE BUSY
! ************************************************************
PRT!BUSY BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R75,=101         ! 'A'
         CLM  R20              ! TIME-OUT COUNTER
PRTBSY73 ICM  R20              ! INC TIME-OUT
         JCY  PRTER            ! JIF TIMED-OUT, ERROR
         LDBD R22,=PRSTS       ! GET PRINTER STATUS
         JEV  PRTBSY73         ! JIF BUSY
         RTN                   ! DONE
!
! ************************************************************
CHKPAPER CLM  R20              ! TIME-OUT COUNTER
PRTBSY10 ICM  R20              ! INC TIME-OUT
         JCY  PRTER            ! JIF TIMED-OUT, ERROR
         LDBD R34,=PRSTS       ! GET PRINTER STATUS
         JEV  PRTBSY10         ! JIF BUSY
         JNG  PRDWNCRT         ! JIF PAPER LOADED
!
         LDM  R34,=10,14       ! LEN 1, LEN 2
         LDM  R32,=MSGOUT      ! "OUT OF PAPER"
         LDB  R75,=41          ! TRAILING '!' CHAR
         JMP  PRTER-1          ! SPIT THE MESSAGE
!
PRDWNCRT JSB  =DOWNCRT         ! POWER-DOWN CRT
         STBD R30,=PRSTS       ! GET PRINTER STATUS
         RTN                   ! DONE
!
! ************************************************************
PRTNULL  CLB  R36              ! 0 MESSAGE LEN
         STBD R36,=PRMLEN      ! SET PRINTER MESSAGE LEN
PRTBSYER LDBD R36,=PRSTS       ! GET PRINTER STATUS
         JOD  PRTER+3          ! JIF READY, ERROR
         RTN                   !
!
! ************************************************************
! READ A COLUMN OF CHARACTER FONT DATA FROM PRINTER CONTROLLER
! R55 = CONTROL BYTE TO WRITE TO PRTSTS
! ************************************************************
GFONTCOL STBD R55,=PRSTS       ! WRITE CONTROL BYTE
         LDBD R44,=PRSTS       ! GET PTR CONTROLLER STATUS
         JEV  PRTER+4          ! JIF NOT READY
         LLB  R44              ! GET "DATA READY" BIT
         LLB  R44              !
         JNC  PRTER+5          ! JIF NOT "DATA READY"
         LDBD R44,=PRCHAR      ! GET COLUMN OF CHARA FONT DATA
         RTN                   ! DONE
! ************************************************************
! ************************************************************
PRTER+5  ICB  R75              !
PRTER+4  ICB  R75              !
         POMD R20,-R6          ! THROW AWAY 1 RTN
PRTER+3  ICB  R75              !
PRTER+2  ICB  R75              !
PRTER+1  ICB  R75              !
PRTER    LDM  R34,=25,0        ! LEN 1, LEN 2
PRTER-1  POMD R36,-R6          ! THROW AWAY 1 RTN
         CLB  R36              ! POWER-DOWN
         STBD R36,=PRSTS       !    PRINTER
         LDM  R36,=MSGPRTR     ! "PRINTER CONTROL BAD! "
         JSB  =OUTBEEP         ! OUT MSG AND BEEP
         RTN                   !
!
! ************************************************************
! VERIFY THE PRINTER CONTROLLER CHARACTER ROM FONT DATA
! ************************************************************
TSTPFONT DRP  R54              !
         BYT  251              ! LDM R54,=
         BYT  200,46           ! CHAR 200, READ FONT COLUMN 6 CMD
         DEF  FONTDATA         !
!
         STBD R54,=PRCHAR      ! REQUEST CHAR 200 FROM PRINTER FONT
NXTCOLPR JSB  =GFONTCOL        ! GET THE COLUMN OF FONT DATA
!
! *** I BELIEVE THIS IS A BUG IN THE SERVICE ROM!
! *** GFONTCOL READS THE FONT DATA INTO R44 AND RETURNS,
! *** LEAVING THE DRP AT 44. THE FOLLOWING CODE THEN
! *** POPS THE TEST VALUE INTO R# WHICH IS R44, AND
! *** THEN TESTS R# (R44) AGAINST R44, SO NATURALLY
! *** IT *ALWAYS* PASSES! BUT, IT PROBABLY DOESN'T
! *** MATTER, AS ALL THE FONT DATA IS CHECKSUMMED
! *** FURTHER DOWN BELOW.
!
         POBD R#,+R56          ! GET TEST DATA (BUG!!!)
         CMB  R#,R44           ! COMPARE (BUG!!!)
         JNZ  PRTER+1          ! JIF DON'T MATCH
         DCB  R55              ! DEC CHAR FONT COLUMN
         CMB  R55,=40          ! LAST ('FIRST') COLUMN?
         JCY  NXTCOLPR         ! JIF NO, LOOP
!
         DCB  R54              ! DECREMENT CHAR#
         CLM  R44              ! PRE-CLEAR CHECKSUM
NXTCHRLP STBD R54,=PRCHAR      ! SET CHAR IN PTR CONTROLLER
         LDB  R55,=40          ! GET COLUMN 0 OF FONT
PRTCHRLP JSB  =GFONTCOL        ! READ IT
         ADM  R46,R44          ! ADD IT INTO THE CHECKSUM
         ICB  R55              ! ++ COLUMN#
         CMB  R55,=47          ! LAST COLUMN?
         JNZ  PRTCHRLP         ! JIF NO, LOOP
!
         DCB  R54              ! DEC CHAR#
         JCY  NXTCHRLP         ! JIF NOT DONE
         CMM  R46,=340,314     ! CHECKSUM OF ALL FONT DATA MATCH?
         JNZ  PRTER+2          ! JIF NO, ERROR
         RTN                   ! DONE
!
! ************************************************************
! WAIT FOR PRINTER NOT BUSY, THEN CHECK THE PRINTER FONT
! ************************************************************
CHKFONT  JSB  =PRT!BUSY        ! WAIT FOR PRINTER NOT BUSY
         JSB  =TSTPFONT        ! VERIFY THE FONT IS GOOD
         RTN                   ! DONE
!
! ************************************************************
! PRINTER CONTROLLER FONT TEST DATA
! ************************************************************
FONTDATA BYT  1                ! .......X
         BYT  1                ! .......X
         BYT  377              ! XXXXXXXX UNDERLINED CHAR(200/128)
         BYT  175              ! .XXXXX.X COLUMN AT A TIME
         BYT  71               ! ..XXX..X SO 'ROTATED'
         BYT  21               ! ...X...X
         BYT  1                ! .......X
!
PRWEDGET BYT  377              ! XXXXXXXX
         BYT  177              ! .XXXXXXX
         BYT  77               ! ..XXXXXX
         BYT  37               ! ...XXXXX
         BYT  17               ! ....XXXX
         BYT  7                ! .....XXX
         BYT  3                ! ......XX
         BYT  1                ! .......X
!
PRWEDGEB BYT  377              ! XXXXXXXX
         BYT  376              ! XXXXXXX.
         BYT  374              ! XXXXXX..
         BYT  370              ! XXXXX...
         BYT  360              ! XXXX....
         BYT  340              ! XXX.....
         BYT  300              ! XX......
         BYT  200              ! X.......
!
BCOUNTS  BYT  10               ! 8
         BYT  20               ! 16
         BYT  40               ! 32
         BYT  40               ! 32
         BYT  20               ! 16
         BYT  10               ! 8
! ************************************************************
! DISPLAY (CRT) TEST - The CRT test checks the CRT controller
! IC and the CRT RAM ICs, and also causes the CRT to display
! various test patters. By checking whether each pattern is
! displayed correctly, you can determine whether the rest of
! the circuitry is operating properly. The test patters are
! also used for making adjustments to the CRT assembly.
!
! NOTE: While the patterns of steps e, f, and g are in the
! display, you can cause the pattern to remain there indefinitely
! by pressing any key. Pressing any key again will cause the
! next pattern to appear.
!
! a) Vertical bars test pattern (remains about 13 seconds).
! b) Shifted vertical bars test pattern (~13 seconds).
! c) "Binary counting" test pattern (~16 seconds).
! d) Blank screen & beep (~3 seconds).
! e) All white & beep (~3 seconds).
! f) "Pincushion" pattern (hp target) (~3 seconds).
! g) 'B'-filled screen (~3 seconds).
! h) 8 lines with 2 copies of character set.
!
! POSSIBLE RESULTS:
!   CRT TEST ENDS
!   CRT CONTROL BAD!
!   CRT RAM n BAD!
! ************************************************************
ENDPRINT BYT  241              ! ATTRIBUTES
DISPL.   JSB  =WTRACED+        ! WAIT START DISPLAY TIME, CLEAR & HOME CURSOR
         JSB  =TSTTRACE        ! CHECK DISPLAY/RETRACE TIMING
         JSB  =TSTBUSYC        ! TEST CONTROLLER NOT-BUSY/BUSY
         CLM  R46              !
         LDB  R30,=245         ! 10100101 VERTICAL BAR PATTERN
BARLOOP  JSB  =FILLGRAF        ! FILL GRAPHICS WITH IT
         JSB  =TSTCRTDA        ! VERIFY ALL BYTES READ BACK OK
         NCB  R30              ! INVERT THE PATTERN: 01011010
         JEV  BARLOOP          ! JIF THIS IS THE SECOND TIME
!
         ICB  R0               ! R0 = 31
         JSB  =FILLGRA-        ! FILL WITH BINARY PATTERN
         JSB  =TSTCRTDA        ! VERIFY IT'S OK
         TSB  R47              ! ANY ERRORS?
         JZR  DSPNOER1         ! JIF NO
         GTO DHOMBAD           ! GOTO ERROR MSG
!
DSPNOER1 JSB  =STDBEEP         ! BEEP
         JSB  =WAITDISP        ! WAIT FOR DISP TRACE
         LDB  R26,=204         ! GRAPHICS, POWER-DOWN
         JSB  =SETMODE         ! TELL CRT CONTROLLER
         LDB  R30,=377         ! ALL WHITE
         JSB  =FILLGRAF        ! FILL GRAPHICS ALL-WHITE
         JSB  =STDBEEP         ! BEEP
         JSB  =WAITDISP        ! WAIT
         JSB  =GRAFMODE        ! SET GRAPHICS MODE
         JSB  =KEYWAIT         ! GIVE KEYS (OR PEACE) A CHANCE
!
         LDB  R66,=202         ! GRAHPICS, POWER-UP, WIPE-OUT
         STBD R66,=CRTSTS      ! TELL CRT CONTROLLER
         LDM  R66,=L67643      ! ADDRESS OF DATA
DFILLLOP POMD R26,+R66         ! GET # OF BYTES TO WRITE
         POBD R30,+R66         ! GET BYTE TO WRITE
         JSB  =FILLMORE        ! FILL THEM
         CMM  R66,=PINBOX1     ! DONE WITH LIST TO FILL?
         JNZ  DFILLLOP         ! JIF NO
!
         JSB  =GRAFMODE        ! GRAPHICS MODE
         JSB  =DRAWBOXS        ! DRAW BOXES OF TARGET
LOGOLP2  POMD R40,+R66         ! GET DATA FOR WRITING
LOGOLP1  JSB  =WR3CRT          ! WRITE 3 BYTES
         ADM  R46,R43          ! MOVE TO NEXT LINE
         DCB  R45              ! DEC LINE COUNT
         JNZ  LOGOLP1          ! LOOP TIL DONE
!
         CMM  R66,=LOGO_LOC    ! DONE W/LOGO?
         JNZ  LOGOLP2          ! JIF NO
!
HPLOP2   LDM  R24,=HPLOGO      ! ADDRESS OF CENTER PART OF LOGO
         POMD R45,+R66         ! GET # LINES AND ADDRESS
HPLOP1   POMD R55,+R24         ! GET 3 BYTES OF DATA
         STM  R55,R40          ! COPY TO R40 FOR WR3CRT
         JSB  =WR3CRT          ! WRITE THEM
         ADM  R46,=100,0       ! NEXT LINE ADDRESS
         DCB  R45              ! DEC LINE COUNT
         JNZ  HPLOP1           ! LOOP TIL DONE
         CMM  R66,=HPLOGO      ! DONE WITH LOGO CENTERS?
         JNZ  HPLOP2           ! JIF NO, DO THE OTHER ONE
!
         JSB  =KEYWAIT         ! WAIT FOR A KEY
         JSB  =WAITRETR        ! WAIT FOR RETRACE TIME
         LDM  R30,=0,14        !
         JSB  =GOMODE          ! SWITCH TO ALPHA
         JSB  =KEYWAIT         ! WAIT FOR A KEY
         JSB  =ALLCHARS        ! DISPLAY ALL 256 CHARS (128 + 128 UNDERLINED)
DDLAY2   LDM  R20,=0,100       ! DELAY COUNT
DDLAYA   DCM  R#               ! DEC COUNT
         JNZ  DDLAYA           ! LOOP TIL DELAYED ALL DAY
         ADM  R30,R#           ! SCROLL UP
         STMD R30,=CRTSAD      ! BY CHANGING START ADDRESS
         CMM  R30,=100,16      ! 100,16=7100=3648DEC (7 LINES FROM END OF ALPHA MEMORY)
         JNZ  DDLAY2           ! JIF NOT
         STMD R30,=CRTRAM      ! SAVE START ADDRESS IN SYSTEM RAM
!
! *** AND THUS ENDS THE DISPLAY TEST
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGTEND          ! " TEST ENDS"
         BYT  3,12             ! LEN 1, LEN 2
         DEF  MSGCRT           ! "CRT"
         JSB  =OUTLIN+S        ! SPIT MESSAGE
         RTN                   ! DONE
!
! ************************************************************
! DRAW BOXES ON CRT
! ENTRY:
!   R66 = POINTER TO "LINE DATA"
! ************************************************************
DRAWBOXS POMD R55,+R66         ! GET # SETS, OFFSET BETWEEN BYTES
VERLOP2  POMD R44,+R66         ! GET BYTE TO WRITE, # TIMES TO WRITE, STARTING ADDRESS
VERLOP1  JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
         STMD R46,=CRTBAD      ! SET BYTE ADDRESS
         JSB  =CRT!BUSY        ! WAIT FOR NOT BUSY
         STBD R44,=CRTDAT      ! WRITE DATA BYTE
         ADM  R46,R56          ! ADVANCE TO NEXT LOCATION
         DCB  R45              ! DEC "LINE LENGTH"
         JNZ  VERLOP1          !
!
         DCB  R55              ! # OF LINES IN THAT ORIENTATION
         JNZ  VERLOP2          ! JIF NOT DONE
!
         CMM  R66,=PINBOX2     ! ANOTHER SET?
         JNZ  DRAWBOXS         ! JIF YES
!
HORLOP2  POMD R44,+R66         ! GET NEXT HOR LINE DATA
         JSB  =CRT!BUSY        ! WAIT FOR NOT BUSY
         STMD R46,=CRTBAD      ! SET BYTE ADDRESS
HORLOP1  JSB  =CRT!BUSY        ! WAIT FOR NOT BUSY
         STBD R44,=CRTDAT      ! WRITE DATA BYTE
         DCB  R45              ! DEC LEN COUNT
         JNZ  HORLOP1          ! LOOP TIL DONE
         CMM  R66,=HPLOGOBK    ! DONE WITH LAST LINE?
         JNZ  HORLOP2          ! JIF NO, DO NEXT ONE
         RTN                   ! DONE
!
! ************************************************************
! WRITE ALL CHARS TO ALPHA DISPLAY
! ************************************************************
ALLCHARS JSB  =CRT!BUSY        ! WAIT FOR NOT BUSY
         LDM  R24,=100,0       ! 2ND LINE OF DISPLAY ADDRESS
         STMD R24,=CRTBAD      ! TELL CRT CONTROLLER
         LDM  R26,=0,2         ! 0 (CHAR), 2
!
! NOTE: R26 IS DOING DOUBLE-DUTY.
! R26 CONTAINS THE CHARACTER TO WRITE, WHICH STARTS AND ENDS AT 0.
! R27 IS THE UPPER BYTE OF THE CRT ADDRESS TO WRITE AT.
! SO WE START WRITING AT 100 (2ND LINE OF DISPLAY), THEN AFTER R26
! CYCLES FROM 0 TO 0, WE ADD 0,2=1000=512DEC=8 LINES TO MOVE DOWN 8 LINES.
!
CHARLOP  JSB  =CRT!BUSY        ! WAIT TIL NOT BUSY
         STBD R26,=CRTDAT      ! STORE CHAR TO ALHPA
         ICB  R26              ! INC CHAR
         JNZ  CHARLOP          ! JIF NOT WRITTEN ALL 256
         ADM  R26,R24          ! MOVE DOWN 8 LINES
         STMD R26,=CRTBYT      ! STORE THAT IN RAM FOR SYSTEM.
         RTN                   ! DONE
!
! ************************************************************
! WAITS FOR A KEY OR UNTIL TIME EXPIRES. IF A KEY IS PRESSED,
! THEN WAITS FOREVER UNTIL ANOTHER KEY IS PRESSED.
! ************************************************************
KEYWAIT  JSB  =GKSHORT         ! WAIT FOR A KEY OR TIME-OUT
         JZR  FKEY             ! JIF NO KEY
MKEY     JSB  =GKSHORT         ! WAIT FOR ANOTHER KEY OR TIME-OUT
         JZR  MKEY             ! JIF NO KEY, KEEP WAITING
FKEY     STBD R53,=KEYSTS      ! MIGHT DISABLE KEY INTERRUPTS
         RTN                   !
!
HOMCLEAR JSB  =HOMCURS         ! START AT THE START
! *** R26 = # CR CHARACTERS TO WRITE
CRTCLX   LDM  R20,=0,2         ! TIME-OUT COUNT
CRTBSY77 DCM  R20              ! DEC TIME-OUT
         JZR  HCNOTO           ! JIF TIMED-OUT
         LDBD R22,=CRTSTS      ! GET CRT STATUS
         JNG  CRTBSY77         ! JIF BUSY
         LDB  R22,=15          ! CR CHARACTER
         STBD R22,=CRTDAT      ! WRITE TO CRT
         DCM  R26              ! DEC CR COUNT
         JNZ  CRTCLX           ! JIF NOT DONE
HCNOTO   CLM  R26              ! ALPHA MODE
GOMODE   JSB  =SETMODE         ! SET ALPHA MODE
         STMD R#,=CRTSAD       ! HOME PAGE
         STMD R#,=CRTRAM       ! AND IN RAM
HOMCURS  CLM  R26              ! START OF CRT RAM
         STMD R26,=CRTBAD      ! HOME BYTE ADDRESS
         STMD R26,=CRTBYT      ! AND IN RAM
         LDM  R26,=0,10        ! LEN OF CRT RAM IN BYTES
         RTN                   ! DONE
!
GRAFMODE LDB  R26,=200         ! GRAPHICS MODE
SETMODE  STBD R#,=CRTSTS       ! SET CRT MODE
         STBD R#,=CRTWRS       ! SAVE IN RAM
         RTN                   ! DONE
!
! ************************************************************
! TEST THAT CRT DISPLAY- AND RETRACE-TIMES ARE APPROPRIATE
! ************************************************************
CRTIMES  LDB  R61,=1           ! FLAG DON'T HOME & CLEAR ON ERRORS
         JSB  =WTRACED         ! WAIT FOR START OF DISPLAY-TIME TRACE
         JSB  =TSTTRACE        ! MEASURE LENGTHS OF DISPLAY AND RETRACE TIMES
         RTN                   ! DONE
!
! ************************************************************
! TEST CRT RAM TO SEE IF IT'S WORKING
! ************************************************************
TSTCRAM  JSB  =WTRACED+        ! WAIT FOR VERY BEGINNING OF RETRACE
         JSB  =TSTTRACE        ! CHECK CRT DISP/RETRACE TIMING
         JSB  =GRAFMODE        ! SET CRT TO GRAPHICS MODE
         CLM  R46              !
         LDB  R30,=377         ! ALL 1s FOR CRT
         JSB  =FILLGRAF        ! FILL GRAPHICS
         JSB  =TSTCRTDA        ! TEST THAT DATA IN CRT RAM IS STILL GOOD
         LDB  R30,=245         ! 10100101
TSTCRT_B JSB  =HOMCURS         ! SET BYTE PTR TO START OF ALPHA
         JSB  =FILLMORE        ! WRITE A BUNCH THERE
         JSB  =HOMCURS         ! SET BYTE PTR TO START OF ALPHA
         JSB  =TSTCRTD-        ! CHECK THAT IT'S STILL THERE
         NCB  R30              ! 01011010
         JEV  TSTCRT_B         ! JIF UNTIL WE'VE DONE IT TWICE
         JSB  =HOMCURS         ! SET BYTE PTR TO START OF ALPHA
         TSB  R47              ! ANY ERRORS?
         JZR  RTN67124         ! JIF NO
!
DHOMBAD  JSB  =HOMCLEAR        ! ELSE HOME AND CLEAR CRT
!
         DRP  R52              !
         BYT  251              ! LDM R52,=
         DEF  MSGBAD!          ! " BAD! "
         BYT  11,6             ! LEN 1, LEN 2
         DEF  MSGCRT           ! "CRT RAM 0"
!
         STM  R52,R32          ! COPY MESSAGE ARGS
         BCD                   !
         CLE                   !
         ERB  R47              ! GET ERROR BITS
         ELB  R46              !
         BIN                   !
         ORB  R47,R46          ! OR-TOGETHER (COMBINE BECAUSE ONLY 4 RAMS - NYBBLES)
         CMB  R47,=17          ! ALL RAMS BAD?
         JZR  CRTERJM          ! JIF YES
NXTCBIT  LRB  R#               ! SHIFT OVER ERROR BITS
         JNC  NXTCRAM          ! JIF THAT BIT NOT BAD
         JSB  =OUTBEEP         ! OUTPUT THE MESSAGE & BEEP
NXTCRAM  POMD R50,+R36         ! SKIP A "CRT RAM x" MESSAGE
         POBD R50,+R36         !
         TSB  R47              ! STILL AN ERROR BIT SET?
         JNZ  NXTCBIT          ! JIF YES
RTN67124 RTN                   ! ELSE DONE
!
! ************************************************************
! WE'RE AT THE VERY START OF A CRT DISPLAY-TIME.
! CHECK THAT DISPLAY- AND RETRACE-TIMES ARE APPROPRIATE
! ************************************************************
TSTTRACE LDM  R45,=2,56,360    !
DONTWANT DCB  R47              ! DEC TIME-OUT COUNT
         JNC  CRTER+4          ! JIF TIMED-OUT
         LDBD R23,=CRTSTS      ! GET CRT STATUS
         ANM  R23,=2           ! DISPLAY/RETRACE TIME
         CMB  R23,R45          ! MATCH THE ONE WE DON'T WANT? (IN R45)
         JZR  DONTWANT         ! JIF YES, KEEP WAITING
         CMB  R46,R47          ! IS LEN OF DISPLAY TIME TOO SHORT?
         JNC  CRTER+5          ! JIF YES
!
         LDM  R46,=17,106      ! RELOAD
         SBB  R45,=2           ! CHANGE TO MEASURING RETRACE TIME (OR -2 FOR END TEST)
         JZR  DONTWANT         ! JIF MEASURING RETRACE TIME
         RTN                   ! ELSE DONE
!
CRTERJM  JMP  CRTER            !
!
! ************************************************************
! WAIT UNTIL CRT HAS *JUST* STARTED A DISPLAY-TIME TRACE
! ************************************************************
WTRACED+ CLB  R61              ! FLAG TO CLEAR & HOME CURSOR ON ERRORS
WTRACED  BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R75,=101         ! 'A' ERROR CODE
         LDBD R23,=CRTSTS      ! GET CRT STATUS
         ANM  R23,=140         ! BUSY & ????
         CMB  R23,=140         ! IS IT?
         JNZ  CRTER+2          ! JIF NO
         JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
         JSB  =WAITRTRC        ! WAIT FOR A RETRACE TIME
! ************************************************************
! WAIT FOR CRT TO BE IN 'DISPLAY' TIME (NOT RETRACE)
! ************************************************************
WAITDISP LDM  R20,=232,1       ! TIME-OUT
RTRCLP   DCM  R20              ! DEC TIME-OUT
         JZR  CRTER+3          ! JIF TIMED-OUT
         LDBD R23,=CRTSTS      ! GET CRT STATUS
         LRB  R23              ! GET DISPLAY/RETRACE BIT
         JEV  RTRCLP           ! JIF RETRACE
         RTN                   ! DONE
!
WAITRETR JSB  =WAITRTRC        ! GIVES US AN EXTRA RTN
         RTN                   !   THAT GETS THROWN AWAY ON ERROR
!
! ************************************************************
! WAIT FOR CRT RETRACE
! ************************************************************
WAITRTRC LDM  R20,=232,1       ! TIME-OUT COUNT
RTRCLOP  DCM  R20              ! DEC TIME-OUT
         JZR  CRTER+8          ! JIF TIME-OUT, ERROR
         LDBD R22,=CRTSTS      ! GET CRT STATUS
         LRB  R22              ! GET RETRACE BIT
         JOD  RTRCLOP          ! JIF DISPLAY TIME
         RTN                   !
!
! ************************************************************
! CRT ERRORS
! ************************************************************
CRTER+8  ICB  R75              !
CRTER+7  ICB  R75              !
         POMD R20,-R6          ! THROW AWAY 1 RTN
CRTER+6  ICB  R75              !
CRTER+5  ICB  R75              !
CRTER+4  ICB  R75              !
CRTER+3  ICB  R75              !
CRTER+2  ICB  R75              !
CRTER+1  ICB  R75              !
         POMD R20,-R6          ! THROW AWAY 1 RTN
CRTER    DRP  R42              !
         BYT  251              ! LDM R42,=
         DEF  MSGCTRLR         ! " CONTROL BAD! "
         BYT  3,16             ! LEN 1, LEN 2
         DEF  MSGCRT           ! "CRT"
!
         STM  R42,R32          ! COPY STRING ARGS
         TSB  R61              ! CLEAR & HOME CURSOR?
         JNZ  NOHOMECL         ! JIF NO
         JSB  =HOMCLEAR        ! HOME CLEAR SCREEN
NOHOMECL JSB  =OUTBEEP         ! OUTPUT "CRT CONTROL BAD! x" WHERE x==R75
         RTN                   !
!
! ************************************************************
! WAIT FOR CRT NOT BUSY (OR TIME-OUT)
! ************************************************************
CRT!BUSY LDM  R20,=0,2         ! TIME-OUT VALUE
CRTBSYLP DCM  R20              ! DEC TIME-OUT COUNTER
         JZR  CRTER+7          ! JIF TIMED-OUT
         LDBD R22,=CRTSTS      ! GET CRT STATUS
         JNG  CRTBSYLP         ! JIF BUSY
         RTN                   ! RETURN WHEN NOT BUSY
!
! ************************************************************
! THIS APPEARS TO BE DOING LITTLE BUT TESTING THAT THE
! CRT CONTROLLER BECOMES BUSY WHEN WRITING DATA TO THE CRT
! RAM, BOTH IN ALPHA AND IN GRAPHICS MODES, LEAVING THE
! DISPLAY IN GRAPHICS MODE.
! ************************************************************
TSTBUSYC LDM  R22,=2,2         !
         STBD R22,=CRTSTS      ! WIPE-OUT CRT
!
! THIS LOOP WRITES TO CRTDAT (WHILE CRT IS WIPED-OUT)
! UNTIL EITHER THE CRT CONTROLLER BECOMES BUSY, OR
! UNTIL TIME-OUT (600 TIMES).
!
LOOP366  LDM  R20,=200,1       ! TIME-OUT COUNTER
LOOP372  DCM  R20              ! DEC TIME-OUT
         JZR  CRTER+6          ! JIF TIMED-OUT
         STBD R24,=CRTDAT      ! WRITE TO CRT (GARBAGE APPARENTLY)
         LDBD R24,=CRTSTS      ! GET CRT STATUS
         JPS  LOOP372          ! JIF NOT BUSY
!
         XRB  R24,R22          ! TOGGLE "DISP TIME" BIT
         LRB  R24              ! GET DISP/RETRACE TIME BIT
         JEV  LOOP372          ! JIF RETRACE, KEEP GOING
!
         NCB  R22              ! INVERT DISP/RETRACE BIT
         JOD  LOOP366          ! JIF ONLY BEEN THROUGH ONCE SO FAR
!
         LDM  R24,=0,20        ! START OF GRAPHICS MEM
         STMD R24,=CRTSAD      ! SET START OF CRT DISPLAY
         JSB  =GRAFMODE        ! AND GRAPHICS MODE
         DCB  R23              ! DONE GRAPHICS YET?
         JOD  LOOP366          ! JIF NO
         RTN                   !
!
! ************************************************************
! TEST THAT ALL CRT GRAPHICS RAM BYTES CONTAIN THE BYTE IN R30
! OR R31 (DEPENDING UPON R0/R*.
! ************************************************************
TSTCRTDA LDM  R26,=0,40        ! SIZE OF GRAPHICS MEMORY
TSTCRTD- LDM  R24,=0,40        ! DELAY COUNTER
TSTCRTDL DCM  R#               ! DELAY
         JNZ  TSTCRTDL         !   UNTIL 0
         LDBD R#,=CRTWRS       ! GET CRT STATUS
         ICB  R#               ! SET "READ REQUEST" BIT
MOREDATA JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
         JSB  =FIXADDR         ! FIX ADDRESS IF NEEDED
         JSB  =WAITRTRC        ! WAIT FOR RETRACE
         STBD R24,=CRTSTS      ! WRITE THE READ-REQUEST TO CRT
         LDM  R20,=0,2         ! TIME-OUT COUNT
WAITDATA DCM  R20              ! DEC TIME-OUT
         JZR  CRTER+1          ! JIF TIMED-OUT
         LDBD R22,=CRTSTS      ! GET CRT STATUS
         JEV  WAITDATA         ! JIF DATA NOT READY
         LDBD R22,=CRTDAT      ! GET THE CHARACTER
         XRB  R22,R*           ! TEST IF BYTE READ MATCHES WHAT WAS WRITTEN
         ORB  R47,R22          ! OR RESULT INTO ERROR ACCUMULATOR
         DCM  R26              ! DECREMENT DATA COUNTER
         JNZ  MOREDATA         ! JIF NO, READ MORE
         RTN                   ! DONE
!
! ************************************************************
! FILL CRT GRAPHICS MEMORY WITH R30
! ************************************************************
FILLGRAF LDB  R0,=30           ! POINT TO BYTE-TO-WRITE REGISTER
FILLGRA- LDM  R24,=0,20        ! START ADDRESS OF GRAPHICS MEMORY
         LDM  R26,=0,40        ! 12288 GRAPHICS MODE NYBBLE ADDRESSES
         JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
         STMD R24,=CRTBAD      ! SET BYTE PTR TO START OF GRAPHICS MEM
         STMD R24,=CRTSAD      ! AND START OF DISPLAY AT GRAPHICS MEM
FILLMORE JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
!
! NOTE: If R0==30, then FIXADDR does nothing.
! If R0==31, then FIXADDR creates a varying binary
! value in R31 to be written at each CRT graphics location.
!
         JSB  =FIXADDR         ! FIX ADDRESS, MAYBE, AND (IF R31) CREATE BINARY FILL BYTE
         STBD R*,=CRTDAT       ! STORE OUT DATA (R30 OR R31)
         DCM  R26              ! DEC GRAPHICS COUNTER
         JNZ  FILLMORE         ! LOOP UNTIL DONE
         RTN                   !
!
! ************************************************************
! ENTRY:
!   R46-7 = CRT BYTE ADDRESS
!   R40-2 = 3 BYTES (24 BITS) TO WRITE TO CRT AT BYTE ADDRESS
! ************************************************************
WR3CRT   JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
         STMD R46,=CRTBAD      ! SET BYTE ADDRESS
         LDB  R0,=40           ! POINT TO BYTE-TO-WRITE REGISTER
WRITMORE JSB  =CRT!BUSY        ! WAIT FOR CRT NOT BUSY
         STBD R*,=CRTDAT       ! WRITE THE CHARACTER
         ICB  R0               ! INC THE WRITE-REGISTER
         CMB  R0,=43           ! R43?
         JNZ  WRITMORE         ! JIF NO, DO R40-42
         RTN                   !
!
!
! ************************************************************
! NOTE: If R0==30, then FIXADDR does nothing.
! If R0==31, then FIXADDR creates a varying binary
! value in R31 to be written at each CRT graphics location.
! ************************************************************
FIXADDR  TSB  R0               ! R30 OR R31
         JEV  RTN67642         ! JIF R30, USE NORMAL ADDRESS
!
! ELSE WE'RE FILLING THE CRT WITH A BINARY TEST PATTERN BUILT
! FROM THE CRT ADDRESS WE'RE WRITING TO.
!
         LDB  R31,R27          ! GET UPPER BYTE OF ADDR
         ADB  R31,R26          ! ADD LOWER BYTE OF ADDR
!
         LDM  R32,R26          ! COPY ADDR
         DCM  R32              ! ADDR -1
         LLM  R32              ! 2*(ADDR-1)
         STMD R32,=CRTBAD      ! SET BYTE ADDRESS
RTN67642 RTN                   !
!
! ************************************************************
! ************************************************************
L67643   BYT  0,30,0           ! 2-BYTE # BYTES TO WRITE (14000=6144) AND 1 BYTE OF DATA
!
         BYT  0,6,15           ! 2-BYTE # BYTES TO WRITE (3000=1536) AND 1 BYTE OF DATA
!
         BYT  0,2,102          ! 2-BYTE # BYTES TO WRITE (1000=512) AND 1 BYTE OF DATA
!
! *** LEFT SIDE OF 'TARGET' OUTER BOX
PINBOX1  BYT  7                ! # OF SETS OF LINES
         BYT  100              ! 100,0=100=64 DEC (ONE LINE OF GRAPHICS WIDTH)
         BYT  0                !
!
         BYT  200              ! GRAPHICS DATA TO WRITE
         BYT  300              ! 300=192 DEC = HEIGHT OF GRAPHICS
         BYT  0                ! 0,20=10000=4096 DEC (WHERE TO START ON CRT) PIXEL (0,0)
         BYT  20               !
! *** LEFT SIDE OF 'TARGET' MIDDLE BOX
         BYT  200              ! DATA TO WRITE
         BYT  240              ! 240=160 DEC = HEIGHT OF LINE
         BYT  6                ! 6,24=12006=5126 DEC (-4096=1030) =BYTE COL,ROW (6,16) = PIXEL (24,16)
         BYT  24               !
! *** LEFT SIDE OF 'TARGET' INNER BOX
         BYT  200              ! DATA TO WRITE
         BYT  220              ! 220=144 DEC = HEIGHT OF LINE
         BYT  12               ! 12,26 = 13012 = ... PIXEL (40,24)
         BYT  26               !
! *** RIGHT SIDE OF 'TARGET' INNER BOX
         BYT  200              ! DATA TO WRITE
         BYT  221              ! 221=145 DEC = HEIGHT OF LINE
         BYT  66               ! 66,26=13066= ... PIXEL (216,24)
         BYT  26               !
! *** RIGHT SIDE OF 'TARGET' MIDDLE BOX
         BYT  200              ! DATA TO WRITE
         BYT  241              ! 241=161 DEC = HEIGHT OF LINE
         BYT  72               ! 72,24=12072= ... PIXEL (232,16)
         BYT  24               !
! *** RIGHT SIDE OF 'TARGET' OUTER BOX
         BYT  1                ! DATA TO WRITE
         BYT  300              ! 300=192 DEC = HEIGHT OF LINE
         BYT  76               ! 76,20=10076= ... PIXEL (255,0)
         BYT  20               !
! *** CENTER CROSSHAIR VERTICAL
         BYT  200              ! DATA TO WRITE
         BYT  300              ! 300=192 DEC = HEIGHT OF LINE
         BYT  40               ! 40,20=10040= ... PIXEL (128,0)
         BYT  20               !
! *** CROSS TICKS ON VERTICAL CROSS-HAIR
         BYT  1                ! # OF SETS OF LINES
         BYT  0                ! 0,2=1000=512 DEC = OFFSET TO MOVE BETWEEN DRAWS (8 VERTICAL PIXELS)
         BYT  2                !
!
         BYT  76               ! BYTE TO WRITE (00111110) 5-PIXEL 'TICK'
         BYT  30               ! # OF TIMES TO WRITE IT (24 DEC TICKS)
         BYT  37               ! 37,20=10037= ... PIXEL (124,0)
         BYT  20               !
! *** TOP SIDE OF 'TARGET' OUTER BOX
PINBOX2  BYT  377              ! BYTE TO WRITE (11111111)
         BYT  40               ! # OF TIMES TO WRITE IT (32 DEC, = 256 PIXELS)
         BYT  0                ! 0,20=10000= ... PIXEL (0,0)
         BYT  20               !
! *** TOP SIDE OF 'TARGET' MIDDLE BOX
         BYT  377              ! BYTE TO WRITE
         BYT  32               ! # OF TIMES TO WRITE (208 DEC PIXELS)
         BYT  6                ! 6,24=12006= ... PIXEL (24,16)
         BYT  24               !
! *** TOP SIDE OF 'TARGET' INNER BOX
         BYT  377              ! BYTE TO WRITE
         BYT  26               ! # OF TIMES TO WRITE (22 DEC = 176 PIXELS)
         BYT  12               ! 12,26=13012= ... PIXEL (40,24)
         BYT  26               !
! *** HORIZONTAL "CROSS-HAIR" OF TARGET
         BYT  377              ! BYTE TO WRITE
         BYT  40               ! # OF TIMES TO WRITE (32 DEC = 256)
         BYT  0                ! 0,50=24000 ... PIXEL (0,96)
         BYT  50               !
! *** BOTTOM SIDE OF 'TARGET' INNER BOX
         BYT  377              ! BYTE TO WRITE
         BYT  26               ! # OF TIMES TO WRITE (176 PIXELS)
         BYT  12               ! 12,72=35012= ... PIXEL (40,168)
         BYT  72               !
! *** BOTTOM SIDE OF 'TARGET' MIDDLE BOX
         BYT  377              ! BYTE TO WRITE
         BYT  32               ! # OF TIMES TO WRITE (208 PIXELS)
         BYT  6                ! 6,74=36006= ... PIXEL (24,176)
         BYT  74               !
! *** BOTTOM SIDE OF 'TARGET' OUTER BOX
         BYT  377              ! BYTE TO WRITE
         BYT  40               ! # OF TIMES TO WRITE (256 PIXELS)
         BYT  300              ! 300,77=37700= ... PIXEL (0,191)
         BYT  77               !
! *** TOP PIXEL OF HOR CROSS-HAIR TICKS
         BYT  200              ! BYTE TO WRITE
         BYT  37               ! # OF TIMES TO WRITE (31 DEC)
         BYT  200              ! 200,47=23600= ... PIXEL (0,94)
         BYT  47               !
! *** 2ND PIXEL OF HOR CROSS-HAIR TICKS
         BYT  200              !
         BYT  37               !
         BYT  300              !
         BYT  47               !
! *** 3RD PIXEL OF HOR CROSS-HAIR TICKS
         BYT  200              !
         BYT  37               !
         BYT  100              !
         BYT  50               !
! *** 4TH PIXEL OF HOR CROSS-HAIR TICKS
         BYT  200              !
         BYT  37               !
         BYT  200              !
         BYT  50               !
! *** NW HP LOGO
HPLOGOBK BYT  377              ! 3 BYTES OF DATA
         BYT  377              !
         BYT  377              !
         BYT  100              ! 100,0 = 64DEC = OFFSET
         BYT  0                !
         BYT  47               ! # OF LINES (39DEC)
         BYT  22               ! 22,32=15022= ... PIXEL (72,40)
         BYT  32               !
!
         BYT  200              ! 3 BYTES OF DATA
         BYT  0                !
         BYT  1                !
         BYT  200              ! 200,0 = 128DEC = OFFSET (EVERY OTHER LINE)
         BYT  0                !
         BYT  23               ! # OF LINES (19DEC)
         BYT  122              ! 122,32=15122= ... PIXEL (72,41)
         BYT  32               !
!
         BYT  377              ! 3 BYTES OF DATA
         BYT  377              !
         BYT  377              !
         BYT  100              ! 100,0 = 64DEC = OFFSET
         BYT  0                !
         BYT  47               ! # OF LINES (39DEC)
         BYT  50               ! 50,54=26050= ... PIXEL (160,112)
         BYT  54               !
!
         BYT  200              ! 3 BYTES OF DATA
         BYT  0                !
         BYT  1                !
         BYT  200              ! 200,0 = 128DEC = OFFSET PER LINE
         BYT  0                !
         BYT  23               ! # OF LINES (19DEC)
         BYT  150              ! 150,54=26150= ... PIXEL (160,113)
         BYT  54               !
! WHERE TO DRAW CENTER OF NW  HP LOGO
LOGO_LOC BYT  15               ! # OF LINES TO DO
         BYT  122              ! 122,35=16522= ... PIXEL (72,53)
         BYT  35               !
! WHERE TO DRAW CENTER OF SE HP LOGO
         BYT  15               ! # OF LINES TO DO
         BYT  150              ! 150,57=27550= ... PIXEL (160,125)
         BYT  57               !
!
! ************************************************************
! CENTER OF HP LOGO
! NOTE: The ERMSGS table uses part of this data for it's
! "warning" entries (which are terminated by MSbit-set bytes).
! ************************************************************
HPLOGO   BYT  377,201,377      ! XXXXXXXXX......XXXXXXXXX
         BYT  374,0,77         ! XXXXXX............XXXXXX
         BYT  342,0,7          ! XXX...X..............XXX
         BYT  302,0,3          ! XX....X...............XX
         BYT  202,0,1          ! X.....X................X
ERMSGS   BYT  203,303,301      ! X.....XXXX....XXXX.....X
         BYT  202,42,41        ! X.....X...X...X...X....X
         BYT  202,42,41        ! X.....X...X...X...X....X
         BYT  202,43,301       ! X.....X...X...XXXX.....X
         BYT  200,2,1          ! X.............X........X
         BYT  300,2,3          ! XX............X.......XX
         BYT  340,2,7          ! XXX...........X......XXX
         BYT  374,0,77         ! XXXXXX............XXXXXX
         BYT  200              !
         ASP  "BAD SERVICE ROM" ! 14
! ************************************************************
! FULL tape test. Does:
!   STATUS (TSTAT.)
!   SPEED (TSPED.)
!   HOLE DETECTION (THOLE.)
!   WRITE (TWRIT.)
!   READ (TREAD.)
!   RECORD (TREC.)
!   WRITE PROTECT (TPROT.)
! prompting for each test, with a response of Y (YES), N (NO),
! or R (REPEAT) acceptable. REPEAT causes the prompt for the
! previous test to appear, allowing you to move 'backward' in
! the list of tests. N skips the current test and moves on to
! the next one, Y runs the current test, then shows the prompt
! for the next test.
! ************************************************************
         BYT  241              !
TAPE.    BYT  142,251          ! LDM R42,=
         DEF  MSGTEST          ! ADDRESS OF "TEST BEGINS"
         BYT  5,13             ! LEN "TAPE ", LEN "TEST BEGINS"
         DEF  MSGTAP1          ! ADDRESS OF "TAPE "
!
         JSB  =OUTLIN+S        ! SPIT OUT "TAPE TEST BEGINS"
         LDM  R76,=TAPTSTTA    ! ADDRESS OF TEST TABLE
TAPRPT   POBD R34,+R76         ! GET LEN OF MSG
         POMD R36,+R76         ! GET ADDRESS OF MSG
         POMD R30,+R76         ! GET ADDRESS OF TEST ROUTINE
         LDM  R32,=MSGYNRTN    ! ADDRESS OF PROMPT MSG (" TEST? Y, N, OR R")
         LDB  R35,=21          ! LEN OF PROMPT
         JSB  =GETKEY          ! WAIT FOR (AND GET) THE KEY
         ANM  R27,=337         ! MAKE UPPERCASE
         CMB  R27,=122         ! R?
         JNZ  NOTR             ! JIF NO
         POMD R43,-R76         ! GET THE LAST TEST
         CMM  R76,=TAPTSTTA    ! START OF LIST?
         JZR  TAPRPT           ! JIF YES
         POMD R43,-R76         ! MOVE BACK ANOTHER ONE
         JMP  TAPRPT           ! GO DO IT
!
NOTR     CMB  R27,=116         ! N?
         JZR  SKIPTEST         ! JIF YES
         CMB  R27,=131         ! Y?
         JNZ  NOTRNY           ! JIF NO
         JSB  X30,ZRO          ! CALL THE TAPE TEST
SKIPTEST CMM  R76,=HOLEARGS    ! END OF LAST TAPE TEST?
         JNZ  TAPRPT           ! JIF NO, DO NEXT
!
! *** NOT Y,N,R KEY *OR* END OF TESTS, TERMINATE TEST LOOP
!
NOTRNY   BYT  142,251          ! LDM R42,=
         DEF  MSGTEND          ! ADDRESS MSG2 " TEST ENDS"
         BYT  4,12             ! LEN MSG1, LEN MSG2
         DEF  MSGTAP1          ! ADDRESS MSG1 "TAPE"
!
SPITMSG# STM  R#,R32           ! COPY MSG ARGS
         CLB  R20              !
         STBD R20,=TAPSTS      ! POWER DOWN TAPE
         JSB  =PROMPT          ! SPIT MESSAGE
CLRVALID CLB  R20              !
         STBD R20,=VALIDD      ! INVALIDATE TAPE DIRECTORY
         RTN                   ! DONE
! ************************************************************
! TAPE STATUS - checks status bits in the Tape Controller, and
! checks the speed of the tape during "fast speed, forward"
! and "slow speed, backward."
! POSSIBLE RESULTS:
!   OK
!   TAPE CONTROL BAD!
!   TAPE ERROR!
!   TAPE ERROR! SPEED
!   TAPE NOT REMOVED!
!   TAPE STALLED!
! ************************************************************
         BYT  241              !
TSTAT.   JSB  =TSTTREGS        ! TEST TAPE CONTROLLER REGISTERS
!
! *** SPIT "REMOVE TAPE, PRESS KEY" MESSAGE
         BYT  142,251          ! LDM R42,=
         DEF  MSGTPK           ! " TAPE, PRESS KEY"
         BYT  6,20             ! LEN 1, LEN 2
         DEF  MSGREMOV         ! "REMOVE"
!
         JSB  =TAPSTS22        ! MSG, KEY, GET TAPSTS
         JOD  MNOTREM          ! JIF CARTRIDGE PRESENT, SPIT "NOT REMOVED!"
!
         JSB  =DOWNCRT         ! POWER DOWN CRT AND PRINTER
         LDM  R56,=TSTSBITS    ! LIST OF TAPSTS VALUES
TSLOP    POMD R30,+R56         ! GET NEXT PAIR
         STBD R30,=TAPSTS      ! WRITE FIRST ONE OUT
         LDB  R30,=257         ! WHICH BITS TO EXAMINE (READY,GAP,WR_EN,ILIM,STALL,CART)
         JSB  =EXAMBITS        ! SEE IF RIGHT
         CMM  R56,=HISPARGS    ! END OF TEST LIST?
         JNZ  TSLOP            ! JIF NO, LOOP
!
         JSB  =TSTLOP40        ! TEST HI-SPEED TIMING
         JSB  =TSTLOP40        ! TEST LO-SPEED TIMING
! *** SPIT "OK" MESSAGE
SPITOK   BYT  144,251          ! LDM R42,=
         DEF  MSGOK            ! "OK"
         BYT  0,2              ! LEN 1, LEN 2
         JMP  SPITMSG#         !
! ************************************************************
! TAPE PROTECT - checks whether a tape cartridge has been
! removed, whether a cartridge is inserted, rewinds the tape,
! checks whether the inserted cartridge is write-protected.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE NOT REMOVED!
!   TAPE OUT!
!   TAPE STALLED!
!   TAPE UNPROTECTED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TPROT.   BYT  142,251          ! LDM R42,=
         DEF  MSGPROTP         ! ", PROTECT, LOAD TAPE"
         BYT  6,24             ! LEN 1, LEN 2
         DEF  MSGREMOV         ! "REMOVE"
!
         JSB  =OLINKEY         ! SPIT MSG, GET KEY
         JSB  =TS2321          ! GET TAPSTS INTO R23 AND R21
         JEV  NOCARTJ          ! JIF NO CART INSERTED
         TSB  R23              ! WAS IT REMOVED?
         JOD  MNOTREM          ! JIF NO, "NOT REMOVED" ERROR
         JSB  =REWINUP         ! REWIND AND POWER UP
         LDB  R23,R72          ! GET TAPE STATUS DURING REWIND
         ANM  R23,=10          ! TAPE WRITE-ENABLED?
         JZR  SPITOK           ! JIF NO, OKAY
!
         BYT  144,251          ! LDM R44,=
         DEF  MSGUNPR-         ! "UNPROTECTED"
         BYT  5,13             ! LEN 1, LEN 2
         JMP  GTOMSG!          ! GO SPIT THE ERROR
!
! *** SPIT "TAPE NOT REMOVED!" MESSAGE
MNOTREM  BYT  144,251          ! LDM R44,=
         DEF  MSGNOT           ! "NOT REMOVED"
         BYT  5,13             ! LEN 1, LEN 2
GTOMSG!  GTO MSGBANG           !
!
! ************************************************************
! HOLE DETECTION - Assumes a rewound cartridge at the start
! of the test. Checks whether a cartridge is inserted,
! runs the tape at slow speed backward for 4 inches checking
! for a hole, runs the tape at slow speed forward for 24 inches
! checking for a hole, runs the tape at fast speed backward
! for 30 inches checking for a hole, rewinds the tape.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE ERROR! HOLE
!   TAPE OUT!
!   TAPE STALLED!
! ************************************************************
         BYT  241              !
THOLE.   BYT  142,251          ! LDM R42,=
         DEF  MSGTPK           ! " TAPE, PRESS KEY"
         BYT  14,20            ! LEN 1, LEN 2
         DEF  MSGLOAD          ! "LOAD REWOUND"
!
         JSB  =TAPSTS22        ! SPIT PROMPT, GET KEY, READ TAPSTS
         JEV  NOCARTJ          ! JIF NO CART
         LDM  R56,=HOLEARGS    ! POINT TO HOLE TEST ARGS
         JSB  =TSTHOLES        ! TEST THEM
         JSB  =REWINUP         ! REWIND TAPE
JOK      JMP  SPITOK           ! SPIT OKAY
!
TSTHOLES JSB  =TAPPUP          ! POWER UP TAPE
!
         BYT  144,251          ! LDM R44,=
         DEF  MSGHOLE          ! "HOLE "
         BYT  14,5             ! LEN 1, LEN 2
!
HOLTSTLP CLB  R71              !
         POMD R30,+R56         ! GET TAPSTS AND TEST BITS
         STBD R30,=TAPSTS      ! WRITE TAPSTS CTL
L70516LP JSB  =GETAHOLE        ! GET A HOLE
         DCB  R31              ! DEC COUNTER
         JNZ  L70516LP         ! JIF NOT TIMED-OUT, KEEP LOOKING FOR HOLE
         TSB  R71              !
         JZR  HOLYERR          ! JIF ERROR
         JSB  =TUP0MOTR        ! STOP TAPE MOTOR
         ICB  R75              ! +ERROR LETTER
         TSB  R30              ! 6, 16, OR 26
         JLZ  HOLTSTLP         ! JIF NOT LAST TEST (LZ ON 1ST TWO, LNZ 3RD ONE)
         CMM  R26,=2,0         ! WITHIN LIMITS? <2 ?
         JNC  HOLYERR          ! JIF NO, ERROR
         CMM  R26,=15,0        ! WITHIN LIMITS? <15?
         JNC  RTN70646         ! JIF YES, SUCCESS
HOLYERR+ ICB  R75              ! ++ERROR LETTER
HOLYERR  POMD R20,-R6          ! THROW AWAY 1 RTN
         GTO MSG44             ! SPIT MESSAGE
!
NOCARTJ  GTO CARTOUT           !
!
! ***
!
SRCHGAPS BYT  144,251          ! LDM R44,=
         DEF  MSGGAPSP         ! "GAP "
         BYT  14,4             ! LEN 1, LEN 2
!
         STBD R30,=TAPSTS      ! WRITE TAPE CONTROL
L70604LP POMD R22,+R56         !
         POMD R24,+R56         !
         JSB  =CHKGAP          ! CHK IF IN GAP
         JZR  HOLYERR          ! JIF YES
         ICB  R75              ! ++ERROR LETTER
         DCB  R20              ! DEC LOOP COUNTER
         JZR  LPDONE           ! JIF DONE
         JEV  L70604LP         ! JIF 2 LEFT
         STM  R24,R26          ! 1200 (640 DEC) ->R26
         JMP  L70604LP         ! LOOP
!
LPDONE   CMM  R26,=34,0        ! <34 ?
         JNC  HOLYERR          ! JIF YES, ERROR, COUNT TOO LOW
         CMM  R26,=44,0        ! >44 ?
         JCY  HOLYERR+         ! JIF YES, ERROR, COUNT TOO HI
RTN70646 RTN                   ! DONE
!
JJOK     JMP  JOK              !
! ************************************************************
! ABBREVIATED TAPE TEST - does the following:
!   Assumes a rewound cartridge.
!   Checks the Tape Controller status bits
!   Checks HI-SPEED FORWARD tape speed for 1/2 second
!   Checks LO-SPEED BACKWARD tape speed for 1/2 second
!   Runs a modified TAPE HOLE test:
!     checks whether a cartridge is inserted, runs the tape at
!     slow speed backward for 30 inches checking for a hole,
!     runs the tape at slow speed forward for 36 inches
!     checking for a hole, runs the tape at fast speed backward
!     for 36 inches checking for a hole, rewinds the tape.
!   Runs the TAPE RECORD test.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE CONTROL BAD!
!   TAPE ERROR!
!   TAPE ERROR! GAP
!   TAPE ERROR! HOLE
!   TAPE ERROR! READ
!   TAPE ERROR! SPEED
!   TAPE ERROR! WRITE
!   TAPE OUT!
!   TAPE PROTECTED!
!   TAPE STALLED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
ATAPE.   BYT  142,251          ! LDM R42,=
         DEF  MSGUNPRO         ! " UNPROTECTED TAPE"
         BYT  14,21            ! LEN 1, LEN 2
         DEF  MSGLOAD          ! "LOAD REWOUND"
!
         JSB  =OLINKEY         ! DISPLAY MSG, WAIT FOR KEY
         JSB  =TSTTREGS        ! TEST TAPE REGISTERS
         LDM  R56,=HISPARGS    !
         JSB  =TSTLOP40        ! TEST HI-SPEED
         JSB  =TSTLOP40        ! TEST LO-SPEED
         JSB  =TSTHOLES        ! TEST HOLES
         JMP  RWIN+            !
! ************************************************************
! TAPE RECORD does:
!   Check for tape inserted.
!   Rewinds the tape.
!   Checks if the tape is write-protected.
!   Writes a 25-inch gap on track A in FWD direction.
!   Writes a 25-inch gap on track B in BAK direction.
!   Rewinds the tape.
!   Moves tape FWD at LO-SPEED 12.5 inches.
!   Writes a 1-inch gap on track A in FWD direction.
!   Writes a 256-byte record on track A in FWD direction.
!   Writes a 1-inch gap on track A in FWD direction.
!   Writes a 256-byte record on track A in FWD direction.
!   Writes a 1-inch gap on track A in FWD direction.
!   Writes a 1-inch gap on track B in BAK direction.
!   Writes a 256-byte record on track B in BAK direction.
!   Writes a 1-inch gap on track B in BAK direction.
!   Writes a 256-byte record on track B in BAK direction.
!   Writes a 1-inch gap on track B in BAK direction.
!   Reads the first 2 gaps and 2 records on track A in FWD dir.
!   Reads the first 2 gaps and 2 records on track B in BAK dir.
!   Rewrites on track B in FWD direction.
!   Rewrites on track A in BAK direction.
!   Reads the rewritten 3 gaps and 2 records on track B in FWD dir.
!   Reads the rewritten 3 gaps and 2 records on track A in BAK dir.
!   Rewinds the tape.
!   Runs the tape at HI-SPEED in FWD dir until 3 gaps and 2 records on track A are read.
!   Rewinds the tape.
!   Runs the tape at HI-SPEED in FWD dir until 3 gaps and 2 records on track B are read.
!   Rewinds the tape.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE ERROR! GAP
!   TAPE ERROR! READ
!   TAPE ERROR! WRITE
!   TAPE OUT!
!   TAPE PROTECTED!
!   TAPE STALLED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TREC.    BYT  142,251          ! LDM R42,=
         DEF  MSGPKEY          ! "PRESS KEY"
         BYT  26,11            ! LEN 1, LEN 2
         DEF  MSGLDUNP         ! "LOAD UNPROTECTED TAPE,"
!
         JSB  =TAPSTS22        ! SPIT MSG, GET KEY, GET TAPSTS
         JEV  NOCARTJ          ! JIF NO CART, ERROR
!
RWIN+    LDM  R54,=40,3,116,107 ! TACH COUNT=160; WR-SYNC,LO-SPEED,FWD,ON,PUP,TRACK0; WR-SYNC,LO-SPEED,BAK,ON,PUP,TRACK1
RWIN     JSB  =REWINUP         ! POWER UP TAPE, REWIND TAPE
         JSB  =NO_WPROT        ! ENSURE NOT WRITE PROTECTED
         STM  R54,R26          ! COPY ARGS TO R26 AND R30
         JSB  =WRTGAP          ! WRITE GAP
         ADM  R54,=250,375,0,10 !
         JRN  RWIN             ! DO AGAIN
!
         LDM  R56,=TRECARGS    !
L70763   JSB  =TSTLOP40        ! TEST TREC TIMING 1ST OR 3RD SET OF TRECARGS
         SBB  R75,=14          !
         JSB  =TSTLOP40        ! 2ND OR 4TH SET OF TRECARGS
         CMM  R56,=GAPARGS     ! DONE WITH 4TH SET?
         JNZ  L70763           ! JIF NO, LOOP
!
         JSB  =REWINUP         ! REWIND THE TAPE
         LDB  R30,=36          ! READ, HI-SPEED, FWD, ON, PUP, TRACK0
GAPTRAKL LDM  R56,=GAPARGS     ! ADDRESS OF ARGS FOR READING GAPS
         LDB  R20,=4           ! # OF SETS OF ARGS
         JSB  =SRCHGAPS        ! FIND THE GAPS (OR ERROR)
         JSB  =REWIND          ! REWIND THE TAPE
         LDB  R75,=107         ! 'G'
         ICB  R30              ! CHANGE TO TRACK1
         JOD  GAPTRAKL         ! JIF TRACK1, CHECK IT, TOO
JJJOK    JMP  JJOK             ! ELSE DONE, OKAY
! ************************************************************
! TAPE SPEED test enables electrical measurements on the tape
! motor-drive ciruitry, doing:
! IF CARTRIDGE IS INSERTED:
!   Run tape at HI-SPEED in FWD direction until end-of-tape,
!     then rewind, then repeat until KEY is pressed.
!   Rewinds the tape.
!   Run tape at LO-SPEED in FWD direction until end-of-tape,
!     then rewind, then repeat until KEY is pressed.
!   Rewinds the tape.
! IF CARTRIDGE IS NOT INSERTED:
!   Run tape motor at HI-SPEED in FWD direction until KEY.
!   Run tape motor at HI-SPEED in BAK direction until KEY.
!   Run tape motor at LO-SPEED in FWD direction until KEY.
!   Run tape motor at LO-SPEED in BAK direction until KEY.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE OUT!
!   TAPE STALLED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TSPED.   LDM  R56,=SPEEDMSG    ! ADDRESS OF ARGS, THEN GET THEM
TSPED1   POMD R34,+R56         ! LEN 1 (4), LEN 2 (13)
TSPED2   POMD R36,+R56         ! MSG 1 "FAST"
         POMD R32,+R56         ! MSG 2 "? PRESS KEY"
         POBD R30,+R56         ! R30=36
         JSB  =GETKEY          ! RETURNS KEYCODE IN R27
         CLM  R46              ! R46 = 00000000, R47=0 (FWD)
         CMB  R27,=61          ! '1'?
         JNZ  DAT01            ! JIF NO
         NCB  R46              ! R46 = 11111111
DAT01    JSB  =TAPPUP          ! POWER UP TAPE UNIT
DAT01-   STBD R46,=TAPDAT      ! STORE 0s OR 1s TO TAPDAT
         STBD R30,=TAPSTS      ! 36->TAP CTRL, HI-SPEED, MTR-FWD, MTR-ON, PUP
NOKEYGO  CMB  R30,=100         ! WRITING SYNC OR GAP?
! *** IT SEEMS THAT, GIVEN THE FOLLOWING TEST FOR WRITE-PROTECT,
! *** THAT THE ABOVE *SHOULD* BE TESTED AGAINST 040, TO INCLUDE
! *** WRITING DATA. BUT MAYBE THAT NEVER GETS SET, SO IT'S OK?
         JNC  L71120           ! JIF NO
         LDBD R72,=TAPSTS      ! GET TAPSTS
         JSB  =NO_WPROT        ! MAKE SURE IT'S NOT WRITE PROTECTED
L71120   JSB  =NOHOLGAP        !
         JZR  TSPDKEY?         ! JIF ... END OF TAPE?
         JSB  =REWIND          ! WIND TAPE
         JMP  DAT01-           ! DO AGAIN
!
TSPDKEY? LDMD R64,=KYIDLE      ! SAVE KYIDLE
         JSB  =TAKHOOKS        ! TAKE KYIDLE
         LDBD R27,=SVCWRD      ! GET INTS NEEDING SERVICE
         JSB  =CLRKBD          ! CLEAR ANY PENDING KEY SERVICE
         STBD R64,=GINTDS      ! DISABLE INTERRUPTS
         STMD R64,=KYIDLE      ! RESTORE KYIDLE
         TSB  R27              ! WAS A KEY PENDING?
         JEV  NOKEYGO          ! JIF NO, KEEP GOING
!
         JSB  =TS2321          ! GET TAPSTS INTO R23 AND R21
         JOD  TSPDCART         ! JIF CART
         TSB  R47              ! GOING FWD?
         JNZ  TSPDCAR-         ! JIF NO, GOING BAK
         ICB  R47              !
         LDB  R21,R30          ! GET TAPSTS ARG
         ANM  R21,=367         ! CHANGE TO MTR-BAK
         STBD R21,=TAPSTS      ! REVERSE DIRECTION
         JMP  NOKEYGO          ! KEEP GOING
!
TSPDCART JSB  =REWIND          !
TSPDCAR- TSB  R30              ! TRACK1?
         JEV  TSPED2           ! JIF NO, REPEAT FOR TRACK 1
JJJJOK   JMP  JJJOK            ! SPIT "OK"
! ************************************************************
! TAPE WRITE - generates a test tape for use in the TAPE READ
! test. If, when prompted WRITE TRACK A/B? PRESS KEY the '1'
! key is pressed, then 1s will be written, but if any other
! key is pressed, then 0s will be written. The test:
!   Checks for cartridge inserted.
! IF CARTRIDGE IS IN:
!   Checks for write-protect.
!   Rewinds the tape.
!   Writes continuous 0s or 1s on track A in FWD dir until
!     end-of-tape, rewinds, then repeats until KEY.
!   Rewinds the tape.
!   Writes continuous 0s or 1s on track B in FWD dir until
!     end-of-tape, rewinds, then repeats until KEY.
!   Rewinds the tape.
! IF CARTRIDGE IS NOT IN:
!   Writes continuous 0s or 1s on track A FWD until KEY.
!   Writes continuous 0s or 1s on track A BAK until KEY.
!   Writes continuous 0s or 1s on track B FWD until KEY.
!   Writes continuous 0s or 1s on track B FWD until KEY.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE OUT!
!   TAPE PROTECTED!
!   TAPE STALLED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TWRIT.   LDM  R56,=TWRITMSG    ! ADDRESS OF ARGS
         JSB  =TS2321          ! CART IN?
         JEV  TSPED1           ! JIF NO, GO TEST (PROMPT OR FAIL?)
         JSB  =REWINUP         ! REWIND
         JSB  =NO_WPROT        ! FAIL IF WRITE PROTECTED
J71043   JMP  TSPED1           ! GO TEST
! ************************************************************
! TAPE READ checks the read ciruitry and also enables measurements
! on the circuitry. The test:
!   Checks for cartridge inserted.
! IF CARTRIDGE IS IN:
!   Rewinds the tape.
!   Reads track A FWD until the end of tape, rewinds, repeats until KEY.
!   Reads track B FWD until the end of tape, rewinds, repeats until KEY.
!   Rewinds the tape.
! IF CARTRIDGE IS NOT IN:
!   Reads track A FWD continuously until KEY.
!   Reads track A BAK continuously until KEY.
!   Reads track B FWD continuously until KEY.
!   Reads track B BAK continuously until KEY.
! POSSIBLE RESULTS:
!   OK
!   END OF TAPE!
!   TAPE OUT!
!   TAPE STALLED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TREAD.   LDM  R56,=TREADMSG    ! ADDRESS OF ARGS
         JSB  =TS2321          ! CART IN?
         JEV  J71043           ! JIF NO, GO TEST (PROMPT OR FAIL?)
         JSB  =REWINUP         ! REWIND TAPE
         JMP  J71043           ! GO TEST
!
NOHOLGAP LDM  R22,=122,1       ! TACH,HOLE,STALL; CART
         JSB  =GETHOLE-        ! GET OUT OF HOLE IF IN ONE, PAST GAP
         RTN                   !
!
TREGSSUB JSB  =TSTTREGS        ! TEST THE CPU REGS W/EXTRA RTN ON STACK
         RTN                   !
!
TAPSTS22 STM  R42,R32          ! MOVE MSG PARAMS
         JSB  =GETKEY          ! SPIT PROMPT AND GET A KEY
!
TS2321   BIN                   !
         LDBD R23,=TAPSTS      ! READ TAPE STATUS to get PREVIOUS state
         LDBD R21,=TAPSTS      ! READ IT AGAIN to get CURRENT state
         RTN                   ! DONE
!
OLINKEY  JSB  =OUTLIN+S        ! SPIT MESSAGE
         LDB  R52,=2           ! LONG KBD WAIT
         JSB  =GETTAKEY        ! GET THE KEY
         STBD R53,=KEYSTS      !
         RTN                   !
!
GETKEY   CLB  R27              !
         STBD R27,=TAPSTS      ! TAPE UNIT OFF AND POWERED DOWN
         JSB  =PROMPT          ! PROMPT/HOOKS INIT
         JSB  =FLUSHKBD        ! CLEAR ANY ALREADY PENDING KEY
WAITKEY  LDBD R27,=SVCWRD      ! GET INTERRUPT FLAGS
         JEV  WAITKEY          ! JIF NOT KEYBOARD
         LDBD R27,=KEYHIT      ! GET KEYCODE OF PRESSED KEY
         JSB  =CLRKBD          ! CLEAR THE SVCWRD BIT
         RTN                   !
!
TAPPUP   JSB  =DOWNCRT         ! POWER DOWN CRT & PRINTER
         LDB  R75,=101         ! 'A'
         LDB  R21,=157         !
UPTAP    LDB  R20,=2           ! POWER UP TAPE
         STBD R20,=TAPSTS      !
UPWAIT   DCM  R#               ! SLIGHT WAIT FOR
         JNZ  UPWAIT           !   FOR TAPE TO POWER-UP
         RTN                   ! DONE
!
WRTGAP   LDB  R0,=30           ! POINT INDIRECTLY AT R30
WGLOOP   LDM  R24,R26          ! COPY TACH COUNT
         JSB  =WRTGAPC1        ! WRITE THE GAP
         ICB  R0               ! NEXT INDIRECT REGISTER
         JOD  WGLOOP           ! JIF LAST ONE (R31)
         JSB  =TUP0MOTR        ! STOP MOTOTRS
         RTN                   !
!
WRTGAPC1 JSB  =WRITGAP-        ! WRITE THE GAP WITH AN EXTRA RTN ON STK
         RTN                   !
!
TSTLOP40 POMD R40,+R56         ! GET ARGS FOR TAPE TEST
TSTLOP41 POMD R30,+R56         ! MORE ARGS
         POMD R66,+R56         ! STILL MORE
         LDM  R34,=14,6        ! MESSAGE LENGTHS?
         LDB  R0,=30           ! POINT INDIRECT REG TO R30
         JSB  X46,ZRO          ! CALL THE ROUTINE
         JSB  =TUP0MOTR        ! STOP MOTOR
         DCB  R41              ! DEC ARGS LOOP COUNT
         JNZ  TSTLOP41         ! JIF MORE, DO ANOTHER LOOP
         RTN                   !
!
! ************************************************************
! CALLED (VIA TABLE) FROM 'TREC.'
! ************************************************************
TAPTSTZ  LDM  R32,=MSGREAD     ! "READ "
         LDB  R35,=5           ! LEN 2
         JSB  =L71740          ! INIT
         DRP  R20              !
         JSB  =TGAP            ! R20->TAPSTS, TEST IF GAP
         JZR  TERRJMP          ! JIF ?
READLOOP JSB  =WAITOK          ! WAIT UNTIL TAPE READY
         CMBD R*,=TAPDAT       ! READ DATA AND SEE IF WHAT WE EXPECT
         JNZ  RDERR            ! JIF NOT, ERROR
         DCM  R26              ! DEC COUNTER
         JNZ  READLOOP         ! LOOP TIL DONE
!
         LDB  R0,R53           ! RESET R0
         JSB  =WRITGAP         ! WRITE A GAP
         ADB  R75,=3           ! CHANGE ERROR LETTER
         ICB  R0               ! BUMP INDIRECT REGISTER
         JOD  TAPTSTZ          ! IF A 2nd PASS IS WARRENTED
         DCB  R0               ! ELSE MOVE REGISTER PTR BACK
         TSB  R44              ! DO WRITGAP?
         JZR  RTN71550         ! JIF NO
         JSB  =WRITGAP         ! ELSE DO IT
RTN71550 RTN                   ! DONE
!
RDERR    ICB  R75              ! +ERROR LETTER
TERRJMP  GTO TAPERR-2          !
!
! ************************************************************
! CALLED (VIA TABLE) FROM 'TREC.'
! ************************************************************
TAPTSTY  LDM  R24,R42          ! COPY MAX TIME-OUT FOR SPEED
         TSB  R40              !
         JPS  DOGAP            ! JIF MSBIT NOT SET
         TSB  R41              ! 2 SETS OF ARGS?
         JEV  DOGAP            ! JIF YES
         ADB  R24,R70          ! ELSE ADD SOMETHING IN
DOGAP    JSB  =WRITGAP-        !
DOGAP-   LDM  R32,=MSGWRITE    ! "WRITE "
         LDB  R21,=40          !
         ANM  R21,R*           ! WRITING DATA?
         JNZ  DODATA           ! JIF YES
         JSB  =WRITGAP         ! ELSE WRITE A GAP
         JMP  DOCOM            ! DO COMMON FINISH
!
DODATA   LDB  R21,=26          ! DELAY AFTER POWER-UP
         JSB  =UPTAP           ! POWER UP TAPE
         LDB  R21,=17          ! MTR-FWD, MTR-ON, PUP, TRACK1
         ANM  R21,R*           ! KEEP ONLY THE ABOVE BITS FROM R* REG
         JSB  =TGAP            ! SKIP ANY GAP
         JZR  TERRJMP          ! JIF FAILED, ERROR
!
DOCOM    JSB  =L71740          ! INIT
         LDB  R21,=1           !
         STBD R21,=TAPDAT      ! 1 -> TAPE
         STBD R20,=TAPSTS      ! 116 WR_SYNC, LO_SPEED, MTR_FWD, MTR_ON, PUP
!
L71655   JSB  =WAITOK          ! EITHER ERROR OR COME BACK WHEN CONTROLLER READY
         STBD R*,=TAPDAT       ! BYTE -> TAPE
         DCM  R26              ! DEC COUNT
         JNZ  L71655           ! LOOP TIL R26 COUNT OF BYTES WRITTEN
!
         LDB  R20,=200         !
L71673   JSB  =WAITOK          ! WAIT TIL TAPE CONTROLLER READY
         STBD R20,=TAPDAT      ! BYTE (200 OR 0) -> TAPE
         LLB  R20              ! SHIFT BYTE
         JCY  L71673           ! JIF MSBIT WAS SET
!
         LDB  R20,=4           ! WAIT COUNT
L71707   JSB  =WAITOK          ! WAIT TIL TAPE CONTROLLER READY
         DCB  R20              ! DO IT 4 TIMES
         JNZ  L71707           !
!
         LDB  R0,R53           ! GET REG# OF OTHER CONTROL BITS
         JSB  =WRITGAP         ! WRITE A GAP
         ADB  R75,=3           ! ...ADD 3 TO POSSIBLE ERROR LETTER???
         ICB  R0               ! MOVE TO NEXT "OTHER CONTROL BITS"
         JOD  DOGAP-           ! IF ODD, WRITE MORE DATA
         DCB  R0               ! ELSE RESTORE R0
         JSB  =WRITGAP         ! WRITE ANOTHER GAP
         RTN                   ! DONE
!
L71740   LDM  R26,R44          ! COPY TIME-OUT VALUE
         STB  R*,R20           ! R20 = R30 (OR R31)
         STB  R0,R53           ! R53 = 30 (OR 31)
         ADB  R0,=36           ! R0  = 66 (OR 67)
         TSB  R40              ! MSBIT OF TIME-OUT SET?
         JPS  RTN71762         ! JIF NO
         LDB  R0,=26           ! RESET R0 TO 26
RTN71762 RTN                   !
!
! ************************************************************
! Rewinds tape to BEFORE 2-holes.
! Writes SYNC at LO-SPEED FWD on TRACK1/B until AFTER 2 holes.
! Moves FWD until AFTER 1 holes.
!
! ************************************************************
REWINUP  JSB  =TAPPUP          ! POWER UP TAPE UNIT
REWIND   LDB  R20,=26          ! HI-SPEED,BAK,MTR-ON,PUP,TRACK0
REWIND-  JSB  =GOTOENDT        ! WIND TAPE
         JZR  REWIND-          ! JIF NOT THERE YET
         TSB  R20              ! TRACK 1?
         JOD  BEGTRK1          ! JIF YES
         LDB  R20,=117         ! WR-SYNC,LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK1
         STBD R20,=TAPSTS      ! TELL CONTROLLER
         LDBD R72,=TAPSTS      ! GET CONTROLLER STATUS
         LDB  R20,=17          ! LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK1
         JMP  REWIND-          !
!
BEGTRK1  JSB  =GO2ENDT         ! GO TO END OF TAPE
         JNZ  BEGTRK1          ! JIF NO
         LDM  R22,=123,1       ! CARE ABOUT: TACH,HOLE,STALL,CART; DEMAND: CART
         LDM  R24,=32,0        ! TACH COUNTER
         JSB  =CHKGAP          ! FWD UP TO 1" TO GET PAST POSSIBLE GAP
         JSB  =TUP0MOTR        ! TURN OFF TAPE MOTOR
         RTN                   !
!
! ************************************************************
! WIND OR REWIND TO END OF TAPE
! ENTRY:
!   R20 = TAPE CONTROL BITS TO WRITE TO TAPSTS (HI/LO-SPEED, FWD/BACK, ON, PUP)
! EXIT:
!   PAST HOLE AT END OF TAPE (FRONT OR BACK, DEPENDING ON DIRECTION)
!   NZ set if 2 holes found, else Z set if only 1 hole found
! ************************************************************
GOTOENDT STBD R20,=TAPSTS      ! WRITE CONTROL BYTE (START TAPE MOVING)
GO2ENDT  JSB  =FINDHOLE        ! WAIT FOR HOLE TO APPEAR
         JSB  =SKIPHOLE        ! SKIP IT
GETAHOLE LDM  R22,=123,1       ! CARE ABOUT: TACH,HOLE,STALL,CART; DEMAND: CART
GETHOLE- LDM  R24,=6,0         ! TACH CNT=6 (GET PAST HOLE IF IN ONE)
         LDB  R70,=1           ! NO "END OF TAPE" ERROR IF SEE HOLE
         JSB  =CHKSTS          ! LOOK FOR 2ND HOLE
         JZR  RTN72101         ! JIF DIDN'T FIND IT AND RETURN Z
         JSB  =SKIPHOLE        ! ELSE SKIP IT AND RETURN NZ
RTN72101 RTN                   !
! ************************************************************
! WAIT FOR A HOLE TO APPEAR (TAPE MUST BE ALREADY MOVING)
! ERROR IF NO CART OR STALL OCCURS
! ************************************************************
FINDHOLE LDM  R22,=23,1        ! CARE ABOUT: HOLE,STALL,CART; DEMAND: CART
NOHOLYET JSB  =ISHOLE?         ! TEST STALL FAIL, RETURN HOLE RESULTS
         JEV  NOHOLYET         ! JIF NOT HOLE, LOOP
         RTN                   !
! ************************************************************
! 6 TACHS IS MORE THAN THE WIDTH OF A HOLE
! IF WE DON'T SEE 'NOT-HOLE' IN 6 TACHS, WE'RE AT "END OF TAPE" ERROR
! NOTE: if this routine returns to the caller (no error), NZ will be set.
! ************************************************************
SKIPHOLE LDM  R24,=6,0         ! TACH COUNT
         LDM  R22,=123,1       ! CARE ABOUT: TACH,HOLE,STALL,CART; DEMAND: CART
         CLM  R26              ! PRECLEAR "SEEN HOLE" FLAG
         CLB  R71              ! PRECLEAR "SEEN NOT-HOLE" FLAG
! *** MOVE 6 TACHS, WATCHING FOR 'HOLE' AND 'NOT-HOLE'
WTACH    JSB  =ISHOLE?         ! IN HOLE?
         JEV  NOHOLE           ! JIF NO
         ICM  R26              ! SET "SEEN HOLE" FLAG
         JMP  YESHOLE          !
!
NOHOLE   LDB  R71,=1           ! SET "SEEN NOT-HOLE" FLAG
YESHOLE  LRB  R21              ! GET TACH BIT
         LRB  R21              !
         JEV  WTACH            ! JIF NO TACH, WAIT FOR TACH
         DCM  R24              ! DECREMENT TACH COUNT
         JNZ  WTACH            ! JIF NOT DONE, LOOP
! *** END OF TACH COUNT
         TSB  R71              ! SEEN NOT-HOLE?
         JNZ  RTN72237         ! JIF YES, OKAY
         LDB  R75,=132         ! 'Z' ("END OF TAPE!" ???)
         JMP  POP3RTNA         ! POP 3 RTN'S
!
! ************************************************************
! R0 (FOR R*) APPEARS TO BE EITHER 30-31 OR 53-54 AT ENTRY
!
WRITGAP  LDM  R24,R42          ! COPY TACH COUNT
WRITGAP- LDB  R22,=200         ! WRITE-GAP BIT
         ORB  R22,R*           ! OR-IN OTHER CTL BITS
         STBD R22,=TAPSTS      ! WRITE TO TAPE CONTROLLER
         LDM  R22,=123,1       ! CARE ABOUT: TACH,HOLE,STALL,CART; DEMAND CART
CHKSTS+  CLB  R70              ! DO END-OF-TAPE-ERROR IF SEE HOLE
!
! ........falling through................
!
! ENTRY:
!   R22    = TAPSTS BITS THAT WE CARE ABOUT (FREQUENTLY 'TACH','HOLE','STALL','CART')
!   R23    = REQUIRED TAPSTS BITS (FREQUENTLY 'CART')
!   R24-25 = TACH COUNTER "TIME OUT"
!   R70    = 0 REPORT "END OF TAPE" IF SEE HOLE
!            1 RETURN NZ IF SEE HOLE
! EXIT:
!   IF CARE ABOUT HOLES AND SEE HOLE AND R70==0, THROW AWAY 3 RTNs AND ERROR END-OF-TAPE
!   ELSE IF CARE ABOUT HOLES AND SEE HOLE AND R70!=0, RTN NZ
!   ELSE IF CARE ABOUT GAP AND SEE NOT-GAP, RTN NZ
!   ELSE IF CARE ABOUT TACH AND SEE TACH, DEC R24, LOOP UNTIL R24==0, THEN RTN Z
!   ELSE IF CARE ABOUT READY AND DON'T SEE READY, LOOP
!   ELSE RTN NZ
!
CHKSTS   JSB  =ISHOLE?         ! TEST IN HOLE
         JEV  NHOLE220         ! JIF NOT IN HOLE
         TSB  R70              !
         JZR  POP3RTNA         ! JIF END OF TAPE ERROR
         RTN                   !
!
NHOLE220 LRB  R21              ! GAP?
         JOD  RTN72237         ! JIF NOT GAP, DONE
         LRB  R21              ! TACH?
         JEV  NOTACH33         ! JIF NO TACH
         DCM  R24              ! DEC TACH COUNT
         JZR  RTN72237         ! JIF 0, DONE
NOTACH33 LRB  R21              ! READY?
         JEV  CHKSTS           ! JIF NO
RTN72237 RTN                   ! DONE
!
CHKGAP   JSB  =CHKSTS+         ! THIS JUST GETS AN EXTRA RTN
         RTN                   !   ON STACK IN CASE OF ERROR
!
! ************************************************************
! CALLED FROM TAPTSTY AND TAPTSTZ
! ************************************************************
TGAP     STBD R#,=TAPSTS       ! WRITE TAPSTS
         LDM  R22,=163,1       ! CARE ABOUT: TACH,GAP,HOLE,STALL,CART; DEMAND: CART
!
! *** NOTE!!! THE FOLLOWING TWO LINES APPEAR TO BE A BUG
! *** IN THE 85A SERVICE ROM. THE "ARP R0" SHOULD BE THE
! *** 2ND BYTE OF A LDM R24,=100,0. 'CHKSTS' IS EXPECTING
! *** A "TACH TIME-OUT" COUNTER IN R24, WHICH ISN'T GETTING
! *** LOADED, AND R23 IS GETTING OVERWRITTEN (LOADED ABOVE)
! *** WITH '100'. THE '1' LOADED ABOVE WOULD BE "DEMAND CART"
! *** WHEREAS THE '100' BELOW WOULD BE "DEMAND TACH SEEN".
! *** I SUSPECT THIS SIMPLY WASN'T CAUGHT BECAUSE IT JUST
! *** HAPPENS TO NOT CAUSE AN ERROR TO BE REPORTED.
!
         LDM  R23,=100         ! ??? BUG ??? SHOULD HAVE BEEN LDM R24,=100,0 ???
         ARP  R0               ! ????
!
         JMP  CHKSTS+          !
!
! ************************************************************
! WAIT UP TO 140 TACHS (ABOUT 4 1/3 INCHES) FOR TAPE CONTROLLER TO BE 'READY'
! ERROR IF NO CART, HOLE, OR STALL.
! RETURN AS SOON AS 'READY' TRUE.
! ************************************************************
WAITOK   LDB  R24,=140         ! COUNTER
WAITOK-  LDBD R21,=TAPSTS      ! GET TAPE STATUS
         JEV  POP3RTNC         ! JIF NO CART, ERROR
         ANM  R21,=322         ! KEEP READY,TACH,HOLE,STALL
         JRN  POP3RTNB         ! JIF STALL, ERROR
         JNG  RTN72237         ! IF READY, DONE
! *** TCTLLR IS *NOT* READY!
         LRB  R21              ! GET HOLE
         JRN  POP3RTNA         ! JIF HOLE, ERROR
         JLZ  WAITOK-          ! JIF NOT TACH
         DCB  R24              ! DEC TACH TIME-OUT
         JNZ  WAITOK-          ! LOOP IF NOT TIMED-OUT
         POMD R24,-R6          ! THROW AWAY A RETURN
         GTO TAPERR+2          ! ERROR
! ************************************************************
! CHECK IF CART PRESENT AND ERROR IF NOT AND ONE IS REQUIRED.
! CHECK FOR "STALL" AND ERROR IF SET
!
! ENTRY:
!   R22 = TAPSTS BITS THAT WE CARE ABOUT (FREQUENTLY 'TACH','HOLE','STALL','CART')
!   R23 = REQUIRED TAPSTS BITS (FREQUENTLY 'CART')
! EXIT:
!   IF NO-CART-ERROR, THROW AWAY 1 RTN AND ERROR OUT
!   IF STALL-ERROR, THROW AWAY 4 RTNs AND ERROR OUT
!   ELSE
!      DRP R21
!      R21 BIT0 = 'HOLE' STATUS
! ************************************************************
ISHOLE?  LDBD R21,=TAPSTS      ! GET TAPE STATUS
         JOD  CARTHERE         ! JIF CARTRIDGE PRESENT, HOLE RESULTS COUNT
         TSB  R22              ! CHECK IF CART REQUIRED
         JOD  POP4RTNC         ! JIF YES, ERROR BECAUSE NO CART WHEN ONE IS REQUIRED
! *** IF CART NOT PRESENT AND NOT REQUIRED, THEN SET "NO HOLE"
         ANM  R21,=357         ! SET HOLE TO 0
CARTHERE ANM  R#,R22           ! KEEP ONLY THE "CARE ABOUT" BITS
         XRB  R#,R23           ! FLIP REQUIRED BITS
         LRB  R#               ! GET STALL BIT
         JOD  POP4RTNB         ! JIF STALL, ERROR (STALL is never demanded :-)
         LRB  R#               !
         LRB  R#               !
         LRB  R#               ! GET HOLE STATE IN LSBIT
RTN72346 RTN                   !
!
! ************************************************************
! SPIT "TAPE OUT" ERROR
! ************************************************************
POP4RTNC POMD R20,-R6          ! THROW AWAY A RTN
POP3RTNC POMD R42,-R6          ! THROW AWAY 3 RTNs
!
CARTOUT  BYT  144,251          ! LDM R44,=
         DEF  MSGOUT           ! "OUT"
         BYT  5,3              ! LEN 1, LEN 2
!
         JMP  MSGBANG          ! SPIT "TAPE OUT" ERROR
!
! ************************************************************
! SPIT "END OF TAPE!" ERROR
! ************************************************************
POP3RTNA POMD R20,-R6          ! THROW AWAY 1 RTN
POP2RTNA POMD R44,-R6          ! THROW AWAY 2 RTNs
!
         BYT  251              ! LDM R44,=
         DEF  MSGEOT           ! "END OF TAPE! "
         BYT  0,15             ! LEN 1, LEN 2
!
         JMP  MSG44            ! SPIT ERROR MESSAGE
!
! ************************************************************
! SPIT "TAPE STALLED!"
! ************************************************************
POP4RTNB POMD R20,-R6          ! THROW AWAY 1 RTN
POP3RTNB POMD R20,-R6          ! THROW AWAY 1 RTN
POP2RTNB POMD R44,-R6          ! THROW AWAY 2 RTNs
!
         BYT  251              ! LDM R44,=
         DEF  MSGSTALL         ! "STALLED! "
         BYT  5,11             ! LEN 1, LEN 2
!
         JMP  MSG44            !
!
! ************************************************************
! ENSURE "WRITE-ENABLED" OR ERROR
! ************************************************************
NO_WPROT LDB  R21,R72          ! GET COPY OF TAPSTS
         ANM  R21,=10          ! WRITE ENABLED?
         JNZ  RTN72346         ! JIF YES, OKAY
!
! ************************************************************
! SPIT "TAPE PROTECTED!" ERROR
! ************************************************************
         POMD R20,-R6          ! THROW AWAY 1 RTN
!
         BYT  144,251          ! LDM R44,=
         DEF  MSGPROTE         ! "PROTECTED "
         BYT  5,11             ! LEN 1, LEN 2
!
MSGBANG  LDB  R75,=41          ! TRAILING '!'
MSG44    STM  R44,R32          ! COPY MSG PARAMS
         JMP  TAPERR           !
!
! TAPE SPEED TEST
! R* (FREQUENTLY POINTS TO R30) = TAPSTS WRITE (CONTROL)
! R31    = # TIMES TO REPEAT ENTIRE TEST
! IF TRACK1, ERROR LETTER GETS SET TO 'D'
! R40    = LOOP COUNTER WAITING BETWEEN TACHS
! R41    = # TIMES TO REPEAT
! R42    = MAX R40 TIME-OUT VALUE
! R43    = RE-LOAD VALUE FOR R40 IF REPEATING TEST
! R44-45 = TIME-OUT LOOP COUNTER WAITING FOR R66 TACHS
! R46-47 = ADDRESS OF THIS ROUTINE (GOT HERE VIA JSB X46,=ZRO)
! R66-67 = MAX # OF TACHS TO COUNT DURING ALL REPEATS OF TEST
!
TAPTSTX  JSB  =TAPPUP          ! POWER UP TAPE
         LDM  R32,=MSGSPEED    !
         STBD R*,=TAPSTS       !
         JEV  TRACK0           ! JIF TRACK 0
         LDB  R75,=104         ! 'D'
TRACK0   LDM  R24,R44          ! COPY TIME-OUT VALUE
!
! *** WAIT FOR TACH TRUE, TIME-OUT WITH R24-25
! *** READ TAPSTS AGAIN IMMEDIATELY AFTER TACH IS TRUE, TACH SHOULD BE FALSE, ELSE ERROR
! *** REPEAT ABOVE FOR R66-67 TACHs
! *** THE FOLLOWING LOOP COUNTS R66 TACHS, MAYBE WAITING FOR TAPE TO GET TO SPEED AND STABILIZE
TACHLOP  DCM  R24              ! DEC TIME-OUT
         JZR  TAPERR-2         ! JIF TIMED-OUT
         LDBD R21,=TAPSTS      ! GET TAPSTS
         ANM  R21,=123         ! KEEP TACH,HOLE,STALL,CART
         LRB  R21              !
         JOD  L72566           ! JIF STALL
         JNC  L72516           ! JIF NO CART
         JRN  POP2RTNA         ! JIF HOLE
L72516   JLZ  TACHLOP          ! JIF NO TACH
         LDBD R21,=TAPSTS      ! GET STATUS
         ANM  R21,=100         ! KEEP TACH
         JNZ  CTLBAD+          ! JIF TACH
         DCM  R66              ! DEC COUNT
         JNZ  TACHLOP          ! LOOP TIL DONE
! *** R40 = TIME-OUT COUNTER FOR TIME (LOOPS) BETWEEN TACHS
! *** THIS NEXT LOOP WAITS FOR TACH TO HAPPEN AGAIN, COUNTING DOWN TIME-OUT FOR EACH LOOP
L72534   DCB  R40              ! DEC TIME-OUT VALUE
         JNC  TAPERR+1         ! IF GOES NEGATIVE, TAPE IS TOO SLOW
         LDBD R21,=TAPSTS      ! GET STATUS
         ANM  R21,=102         ! KEEP TACH, STALL
         JRN  L72566           ! JIF STALL, ERROR
         JLZ  L72534           ! JIF NO TACH, LOOP, KEEP WAITING
! *** NOW SEE IF TAPE IS TOO FAST (TIME-OUT COUNT IS TOO SMALL)
         CMB  R40,R42          ! TIME-OUT COUNT TOO BIG?
         JCY  TAPERR+2         ! JIF YES, TAPE TOO FAST
         LDB  R40,R43          ! RELOAD TIMING REG
         DCB  R31              ! DEC TRY COUNT
         JNZ  L72534           ! JIF TRY AGAIN
         RTN                   ! DONE
!
L72566   LDB  R75,=131         ! 'Y'
         TSB  R30              ! TRACK 0?
         JEV  STALLJMP         ! JIF YES
         ICB  R75              ! ++ERROR LETTER
STALLJMP JMP  POP2RTNB         !
!
TAPERR+2 ICB  R75              ! ++ERROR LETTER
TAPERR+1 ICB  R75              ! ++ERROR LETTER
TAPERR-2 POMD R20,-R6          ! THROW AWAY A RETURN
TAPERR-1 POMD R20,-R6          ! THROW AWAY A RETURN
TAPERR   LDM  R36,=MSGTAP1     ! "TAPE "
         CLB  R23              !
         STBD R23,=TAPSTS      ! POWER DOWN TAPE UNIT
         JSB  =OUTBEEP         !
         GTO CLRVALID          !
!
CTLBAD+  ICB  R75              ! +ERROR LETTER
         JMP  CTRLBAD!         !
!
CTRLBAD  LDB  R75,=101         ! 'A'
CTRLBAD! LDM  R32,=MSGCTRLR    ! " CONTROL BAD! "
         LDM  R34,=4,16        ! LEN 1, LEN 2
         JMP  TAPERR-2         ! SPIT ERROR MESSAGE
!
TUP0MOTR LDB  R21,=2           ! TAPE PUP, MOTORS STOPPED
         STBD R21,=TAPSTS      !
         LDBD R21,=TAPSTS      ! GET ITS STATUS
         ANM  R21,=202         ! LOOK AT READY, STALL BITS
         JRN  STALLJMP         ! JIF STALL
         JNG  CTRLBAD          ! JIF READY
         CLB  R70              ! INIT LOOP COUNTER
GETTREDY CLM  R22              ! TIME-OUT COUNTER
L72677   ICM  R22              ! INCREMENT COUNTER
         JCY  TUPTO            ! JIF TIMED-OUT
         LDBD R21,=TAPSTS      ! GET TAPSTS
         ANM  R21,=300         ! KEEP TACH AND READY BITS
         JNG  TAPREADY         ! JIF READY
         JLZ  L72677           ! JIF DIDN'T SEE TACH TICK
         ICB  R70              ! INC LOOP COUNTER
         JMP  L72677           ! KEEP GOING
!
TAPREADY LDBD R#,=TAPSTS       ! GET TAPE STATUS REG
         JNG  TRRTN            ! JIF READY
TUPTO    POMD R20,-R6          ! THROW AWAY RETURN ADDR
         LDB  R75,=102         ! 'B' FAILURE (TAPE CONTROLLER NOT READY)
TAPERROR LDM  R34,=14,0        ! LEN OF MESSAGES
         JMP  TAPERR           ! REPORT ERROR
!
TSTTREGS BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDB  R75,=101         ! 'A' FAILURE (WR/RD TAPDAT FAILED)
         LDM  R22,=0,125       !
         STBD R22,=TAPSTS      ! POWER DOWN TAPE
TAPDAT2  STBD R23,=TAPDAT      ! WRITE TO TAPE DATA
         CMBD R23,=TAPDAT      ! GET THE SAME THING BACK?
         JNZ  TAPERROR         ! JIF NO
         NCB  R23              ! INVERT 125 TO 252 (ALTERNATING 1s AND 0s)
         JEV  TAPDAT2          ! JIF ONLY SECOND TIME, DO THE OTHER MASK
         JSB  =GETTREDY        ! TEST READY FLAG
         LDM  R30,=256,240     !
!
! ENTRY:
!   R30 = BIT MASK OF BITS TO KEEP FOR TEST
!   R31 = BITS THAT SHOULD BE SET
!
EXAMBITS LDBD R23,=TAPSTS      ! GET TAPE STATUS
         ANM  R23,R30          ! KEEP ONLY DESIRED BITS
         XRB  R31,R23          ! CHECK WHICH ARE SET
         JZR  TRRTN            ! JIF ONLY CORRECT BITS SET (OKAY)
!
         POMD R34,-R6          ! THROW AWAY 1 RETURN ADDR
         ICB  R75              ! ++ERROR LETTER
TAPERR_D ICB  R75              ! ++ERROR LETTER
         LRB  R31              ! CARTRIDGE PRESENT?
         JNC  NOTAPE           ! JIF NO
         JSB  =TAPERROR        ! ELSE ERROR FOR TAPE PRESENT RESULTS
NOTAPE   TSB  R31              ! STALL?
         JNZ  TAPERR_D         ! JIF YES (STALL)
TRRTN    RTN                   !
!
         JSB  =GETTREDY        !
         RTN                   !
! ************************************************************
! ************************************************************
TAPTSTTA BYT  6                ! LEN OF MSG
         DEF  MSGSTATS         ! MSG ADDR "STATUS"
         DEF  TSTAT.           ! ROUTINE ADDR
!
         BYT  5                ! LEN
         DEF  MSGSPEED         ! "SPEED"
         DEF  TSPED.           !
!
         BYT  4                ! LEN
         DEF  MSGHOLE          ! "HOLE"
         DEF  THOLE.           !
!
         BYT  5                ! LEN
         DEF  MSGWRITE         ! "WRITE"
         DEF  TWRIT.           !
!
         BYT  4                ! LEN
         DEF  MSGREAD          ! "READ"
         DEF  TREAD.           !
!
         BYT  6                ! LEN
         DEF  MSGRECRD         ! "RECORD"
         DEF  TREC.            !
!
         BYT  7                ! LEN
         DEF  MSGPROTE         ! "PROTECT"
         DEF  TPROT.           !
! ************************************************************
! *** TAPSTS ARGS FOR THOLE. TEST
HOLEARGS BYT  6,26             ! LO-SPEED,BAK,ON,PUP,TRACK0; COUNTER
         BYT  16,200           ! LO-SPEED,FWD,ON,PUP,TRACK0; COUNTER
         BYT  26,240           ! HI-SPEED,BAK,ON,PUP,TRACK0; COUNTER
TSTSBITS BYT  3,240            ! TRACK 1, POWER-UP, MTR_OFF, MTR_BAK, LO_SPEED; SHOULD RESULT IN READY,NOT-GAP
         BYT  52,240           ! TRACK 0, PUP, MTR_OFF, MTR_FWD, LO_SPEED, WR_DATA; SHOULD RESULT IN READY,NOT-GAP
         BYT  103,250          ! TRACK 1, PUP, MTR_OFF, MTR_BAK, LO_SPEED, WR_SYNC; SHOULD RESULT IN READY,NOT-GAP,WR_EN
         BYT  312,250          ! TRACK 0, PUP, MTR_OFF, MTR_FWD, LO_SPEED, WR_SYNC, WR_GAP; SHOULD RESULT IN READY,NOT-GAP,WR_EN
! *** USED BY TSTAT. AND ATAPE. TESTS
HISPARGS BYT  13               ! 40 TIME-OUT VALUE FOR TAPE SPEED TEST
         BYT  1                ! 41 # SETS OF R30 & R66 ARGS BELOW
         BYT  6                ! 42 MAX TIME-OUT VALUE FOR TAPE SPEED
         BYT  13               ! 43 RELOAD FOR 40 IF REPEATING TEST
         BYT  0,150            ! R44-45 TIME-OUT VALUE
         DEF  TAPTSTX          ! 46-47 TEST ROUTINE ADDR
! R30=36: HI-SPEED, MTR-FWD, MTR-ON, PWR-UP, TRACK0
         BYT  36,51            ! R30 (STORED TO TAPSTS @72461) R31=REPEAT COUNT
         BYT  200,1            ! 66-67 TACH COUNT = 384 (DEC) = 12" TAPE (32 TACHS/INCH)
! *** USED BY TSTAT. AND ATAPE. TESTS
         BYT  76               ! 40 TIME-OUT VALUE FOR TAPE SPEED TEST
         BYT  1                ! 41 # SETS OF R30 & R66 ARGS BELOW
         BYT  6                ! 42 MAX TIME-OUT VALUE FOR TAPE SPEED
         BYT  76               ! 43 RELOAD FOR 40 IF REPEATING TEST
         BYT  0,150            ! R44-45 TIME-OUT VALUE
         DEF  TAPTSTX          ! 46-47 TEST ROUTINE ADDR
! R30=7: LO-SPEED, MTR-BAK, MTR-ON, PWR-UP, TRACK1
         BYT  7,51             ! R30 (STORED TO TAPSTS @72461) R31=REPEAT COUNT
         BYT  20,0             ! 66-67 TACH COUNT = 16 (DEC) = 1/2" TAPE (32 TACHS/INCH)
! *** TAPSTS ARGS FOR ATAPE. TEST
         BYT  6,240            ! LO-SPEED,MTR_BAK,MTR_ON,PUP,TRACK0;
         BYT  16,300           ! LO-SPEED,MTR_FWD,MTR_ON,PUP,TRACK0;
         BYT  26,300           ! HI-SPEED,MTR_BAK,MTR_ON,PUP,TRACK0;
!
! *** USED (ONLY) BY TREC
TRECARGS BYT  200              ! 40 TIME-OUT VALUE FOR TAPE SPEED TEST
         BYT  2                ! 41 # SETS OF R30 & R66 ARGS BELOW
         BYT  20,0             ! 42 MAX TIME-OUT TACH VALUE FOR TAPE SPEED
         BYT  377,0            ! R44-45 GAP FLAG (@71541)
         DEF  TAPTSTY          ! 46-47 TEST ROUTINE ADDR
         BYT  116,116          ! R30 (STORED TO TAPSTS @72461) R31=REPEAT COUNT
         BYT  0,377            ! 66-67 TACH COUNT = 65280 (DEC) = 2040" TAPE (32 TACHS/INCH)
         BYT  107,107          ! R30 (STORED TO TAPSTS @72461) R31=REPEAT COUNT
         BYT  0,377            ! 66-67 TACH COUNT = 65280 (DEC) = 2040" TAPE (32 TACHS/INCH)
! *** 2nd SET OF TRECARGS (SEE TREC.)
         BYT  200              !
         BYT  2                !
         BYT  20,0             ! 42 MAX TIME-OUT TACH VALUE FOR TAPE SPEED
         BYT  377,0            ! R44-45 GAP FLAG (@71541)
         DEF  TAPTSTZ          !
         BYT  16,16            !
         BYT  0,377            !
         BYT  7,7              !
         BYT  0,377            !
! *** 3rd SET OF TRECARGS (SEE TREC.)
         BYT  0                !
         BYT  2                !
         BYT  20,0             ! 42 MAX TIME-OUT TACH VALUE FOR TAPE SPEED
         BYT  377,0            ! R44-45 GAP FLAG (@71541)
         DEF  DOGAP-           !
         BYT  57,57            !
         BYT  125,252          !
         BYT  46,46            !
         BYT  125,252          !
! *** 4TH SET OF TRECARGS (SEE TREC.)
         BYT  0                !
         BYT  2                !
         BYT  20,0             ! 42 MAX TIME-OUT TACH VALUE FOR TAPE SPEED
         BYT  377,0            ! R44-45 GAP FLAG (@71541)
         DEF  TAPTSTZ          !
         BYT  17,17            !
         BYT  125,252          !
         BYT  6,6              !
         BYT  125,252          !
! *** FINAL 4 SETS OF TRECARGS (FETCHED @ 70604)
GAPARGS  BYT  163,41           ! CARE ABOUT: TACH,GAP,HOLE,STALL,CART; DEMAND: NOT-GAP,CART
         BYT  200,2            ! 1200 OCTAL, 640 DEC
!
         BYT  163,1            ! CARE ABOUT: TACH,GAP,HOLE,STALL,CART; DEMAND: NOT-GAP,CART
         BYT  100,0            ! 100 OCTAL, 64 DEC
!
         BYT  163,41           ! CARE ABOUT: TACH,GAP,HOLE,STALL,CART; DEMAND: NOT-GAP,CART
         BYT  100,0            ! 100 OCTAL, 64 DEC
!
         BYT  163,1            ! CARE ABOUT: TACH,GAP,HOLE,STALL,CART; DEMAND: NOT-GAP,CART
         BYT  100,0            ! 100 OCTAL, 64 DEC
!
SPEEDMSG BYT  4,13             ! LEN 1, LEN 2
         DEF  MSGFAST          ! "FAST"
         DEF  MSG?             ! "? PRESS KEY"
         BYT  36               ! HI-SPEED,MTR-FWD,MTR-ON,PUP,TRACK0
         DEF  MSGSLOW          ! "SLOW"
         DEF  MSG?             ! "? PRESS KEY"
         BYT  17               ! LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK1
!
TWRITMSG BYT  6,22             ! LEN 1, LEN 2
         DEF  MSGWRITE         ! "WRITE "
         DEF  MSGTRAKA         ! "TRACK A? PRESS KEY"
         BYT  116              ! WR-SYNC,LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK0
         DEF  MSGWRITE         ! "WRITE "
         DEF  MSGTRAKB         ! "TRACK B? PRESS KEY"
         BYT  117              ! WR-SYNC,LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK1
!
TREADMSG BYT  5,22             ! LEN 1, LEN 2
         DEF  MSGREAD          ! "READ "
         DEF  MSGTRAKA         ! "TRACK A? PRESS KEY"
         BYT  16               ! LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK0
         DEF  MSGREAD          ! "READ "
         DEF  MSGTRAKB         ! "TRACK B? PRESS KEY"
         BYT  17               ! LO-SPEED,MTR-FWD,MTR-ON,PUP,TRACK1
!
! ************************************************************
! KEYBOARD TEST - checks the keyboard controller and also
! verifies key entry for every key on the keyboard. At certain
! times, CTRL and SHIFT keys must be held while the next key is
! pressed. In steps 4 through 7 and 9, each key is entered
! when it is pressed down, while in steps 1 through 3 and 8,
! each key is entered when it is released.
!
! 1) KEY TEST: CTRL, CAPS, SHIFT UP; PRESS A
! 2) PRESS A K1
! 3) PRESS A K1
! 4) CAPS DOWN; PRESS A K1
! 5) CAPS UP; PRESS A K1
! 6) CTRL DOWN; PRESS A K1 (hold down CTRL)
! 7) CTRL UP; PRESS A K1
! 8) SHIFT DOWN; PRESS KEYS (hold down SHIFT)
! 9) SHIFT UP; PRESS KEYS
!
! POSSIBLE RESULTS:
!   KEY CONTROL BAD!
!   NO KEY!
!   KEY STUCK!
!   KEY BAD!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TSTKBD   JSB  =CHKINTS         ! TEST KBD & GLOBAL INTERRUPTS
!
         BYT  142,251          ! LDM R42,=
         DEF  MSGCOLON         ! ":"
         BYT  10,1             ! LEN 1, LEN 2
         DEF  MSGKEY           ! "KEY TEST"
!
         JSB  =OUTLIN+S        ! SPIT MSG
!
! *** R56 WAS SET UP DURING 'CHKINTS' TO POINT TO 'KT1' @ 74663
!
NXTKTST  POMD R40,+R56         ! GET ADDRESS OF TEST ROUTINE & PROMPT MSG ARGS
         STM  R40,R30          ! COPY MSG ARGS
         STM  R36,R54          ! SAVE PROMPT ADDRESS IN CASE WRONG KEY PRESSED
         JSB  =OUTLIN+         ! SPIT TEST PROMPT
         POMD R44,+R56         ! GET KEY COUNT, ?, AND PTR TO KEY LIST
         STB  R44,R40          ! COPY HOW MANY KEYS ON LIST TO PRESS
TKBDLOP1 LDB  R50,=10          ! ERROR KEY RETRY-COUNT
TKBDLOP2 LDB  R51,=2           ! STUCK KEY COUNTER
         CLB  R52              !
         JSB  =CLRKBD          ! CLEAR ANY PENDING KEYBOARD INTERRUPTS
         JSB  X30,ZRO          ! CALL THE FUNCTION TO GET A KEY (TGETKEY, TGETKEYS, JSBTK)
         STB  R27,R75          ! COPY THE KEYCODE (IN CASE ERROR?)
         LDB  R27,R41          ! GET COPY OF RIGHT-SHIFTED KEYSTS
         XRB  R27,R45          ! SHIFT IN PROPER STATE?
         ANM  R27,=4           ! NOW THROW AWAY ALL BUT "SHIFT DOWN"
         JNZ  DECBAD           ! JIF SHIFT IN WRONG STATE, ERROR
         CMBD R75,R46          ! R75=KEYCODE PRESSED, R46=PTR TO DESIRED KEY
         JNZ  DECBAD           ! JIF DON'T MATCH
NXTKEY   ICM  R46              ! ELSE MOVE TO NEXT KEY
         DCB  R44              ! DEC # OF KEYS IN THIS LIST
         JNZ  TKBDLOP1         ! JIF MORE TO DO
!
         JSB  =FIXFF           ! FIXS CAPS LOCK IF NECESSARY
!
         BYT  144,251          ! LDM R44,=
         DEF  MSGOK            ! "OK"
         BYT  0,2              ! LEN 1, LEN2
!
         JSB  =OUTLIN+S        ! SPIT MSG
         CMM  R56,=KSEQUNSH    ! FINISHED LAST TEST?
         JNZ  NXTKTST          ! JIF NO, DO NEXT
!
         LDM  R34,=15,0        ! LEN 1, LEN 2
         LDM  R36,=MSGKEY      ! "KEY TEST ENDS"
         JSB  =PROMPT          ! SPIT MESSAGE
!
FLUSHKBD BIN                   !
         STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         LDB  R20,=1           !
         STBD R20,=KEYSTS      ! ENABLE KEYBOARD
CLRKBD   LDB  R20,=376         ! CLEAR ANY PENDING KBD INT
         JSB  =CLRINTS         !
         STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         RTN                   ! DONE
! ************************************************************
! PUT CAPS LOCK FLIP-FLOP BACK IF NECESSARY
! ************************************************************
FIXFF    CMM  R56,=KT3         ! NEED TO TOGGLE CAPS FLIP-FLOP?
         JNZ  RTN73551         ! JIF NO
TOGGLEFF LDB  R23,=200         ! TOGGLE CAPS LOCK FLIP-FLOP
         STBD R23,=KEYSTS      !
RTN73551 RTN                   !
!
TOGNEXT  JSB  =TOGGLEFF        ! TOGGLE THE CAPS FLIP-FLOP
         JMP  NXTKEY           !
!
! ************************************************************
DECBAD   DCB  R50              ! DEC KEY RETRY COUNT
         JZR  KTBAD            ! JIF GIVE UP, ERROR
         CLM  R20              !
         LDB  R20,R40          ! COPY # REMAINING KEYS IN TEST
         LDM  R34,=5,5         ! LEN 1, LEN 2
         TSB  R45              ! SHIFT SUPPOSED TO BE DOWN?
         JNZ  R45NZ            ! JIF TRUE
         LDB  R35,=3           ! ELSE LEN 2 FOR " UP"
         LDM  R32,=MSGSHFTU    ! "SHIFT UP; ...
         ADM  R20,R46          ! SKIP REMAINING KEYS IN TEST
         STM  R46,R42          ! COPY PTR
         JMP  L73630           !
!
R45NZ    LDM  R32,=MSGSHDWN    ! "SHIFT DOWN; ...
         TCM  R20              ! NEGATE R20 COUNT
         ADM  R20,R46          ! MOVE TO END OF LIST
         STM  R20,R42          ! COPY PTR
L73630   TSB  R27              ! SHIFT IN RIGHT STATE?
         JZR  SHFTOK           ! JIF YES
         TSB  R45              ! SUPPOSED TO BE DOWN?
         JRN  SHFTDOWN         ! JIF YES
         LDM  R32,=MSGSHFTU    ! "SHIFT UP; ...
         LDB  R35,=3           ! CHANGE LEN2 FOR " UP"
SHFTDOWN ADB  R35,R34          ! COMBINE MESSAGE LENS (IT'S ALL MSG 2)
         CLB  R34              ! CLEAR LEN 1
         JMP  KBDOMB           ! GO SPIT IT
!
SHFTOK   CMBD R75,R20          ! RIGHT KEY?
         JNZ  WRNGKEY          ! JIF NO
         LDM  R36,R54          ! GET SAVED COPY OF PROMPT ADDRESS
         ADB  R32,=5           ! SKIP "SHIFT" PART OF MSG
         CMM  R56,=KT2         ! ON TEST 1? ("NEXT TEST" PTR POINTS TO TEST 2)
         JZR  TOGNEXT          ! JIF YES, TRY THE KEY AGAIN
         CMM  R56,=KT5         ! ON TEST 2-4?
         JNC  WRNGKEY          ! JIF YES, SPIT WRONG KEY MSG
         CMM  R56,=KT9         ! ON TEST 9?
         JCY  KBDOMB           ! JIF YES, SPIT MSG
         DCB  R34              ! ELSE SHORTEN LEN OF MSG 1
KBDOMB   JSB  =OUTBEEP-        ! OUTPUT MSG & BEEP
WRNGKEY  LDM  R32,R42          ! GET KEY PTR
         LDM  R34,=0,1         ! LEN 1, LEN 2
         CLM  R20              !
         LDBD R20,R32          ! GET KEY AS 2-BYTE VALUE
         JPS  L73742           ! JIF ASCII CHAR
         LLB  R20              ! ELSE DOUBLE IT
         ADM  R20,=K1-K4       ! POINT TO NEEDED K1,K2,K3,K4 NAME
         STM  R20,R32          ! MAKE IT MSG 2
         ICB  R35              ! CHANGE LEN 2 FROM 1 TO 2
L73742   JSB  =OUTBEEP-        ! OUTPUT MSG & BEEP
         GTO TKBDLOP2          ! GIVE 'EM ANOTHER CHANCE
!
! ************************************************************
KTBAD    LDM  R32,=MSGBAD!     ! " BAD! "
         LDM  R34,=3,6         ! LEN 1, LEN 2
KTENDS   LDM  R36,=MSGKEY      ! "KEY TEST ENDS"
         JSB  =FIXFF           ! FIX CAPS LOCK FF
         JSB  =OUTBEEP         ! OUTPUT MSG AND BEEP
         GTO FLUSHKBD          ! FINISH UP
!
! ************************************************************
CHKINTS  BIN                   !
         LDB  R75,=101         ! 'A'
!
         DRP  R55              !
         BYT  251              ! LDM R55,=
         BYT  1                ! 1 (CMD BYTE FOR TURNING ON/OFF INTS)
         DEF  KT1              ! ADDRESS OF FIRST KBD TEST STRUCTURE
!
         STBD R55,=GINTDS      ! DISABLE GLOBAL INTERRUPTS
         STBD R55,=KEYSTS      ! ENABLE KEYBOARD SCANNING
         LDBD R55,=KEYSTS      ! READ KYBD STATE
         JNG  K+ERR2           ! JIF GLOBAL INTERRUPTS ENABLED
         JEV  K+ERR2           ! JIF KEYBOARD INTERRUPTS DISABLED
         LDB  R55,=2           !
         STBD R55,=KEYSTS      ! DISABLE KEYBOARD INTERRUPTS
         LDBD R55,=KEYSTS      ! GET KBD STATUS
         JNG  K+ERR2           ! JIF GLOBAL INTERRUPTS ENABLED
         JOD  K+ERR2           ! JIF KEYBOARD ENABLED
         STBD R55,=GINTEN      ! ENABLE GLOBAL INTERRUPTS
         LDBD R55,=KEYSTS      ! GET KBD STATUS
         JPS  K+ERR2           ! JIF GLOBAL INTERRUPTS DISABLED
         JOD  K+ERR2           ! JIF KBD ENABLED
         RTN                   ! DONE
! ************************************************************
K+ERR1   ICB  R75              ! ++ERROR LETTER
K+ERR    ICB  R75              ! ++ERROR LETTER
K+ERR2   POMD R32,-R6          ! THROW AWAY 1 RTN
         LDM  R32,=MSGCTRLR    ! " CONTROL BAD! "
         LDM  R34,=3,16        ! LEN 1, LEN 2
JMPJMP73 JMP  KTENDS           !
!
TGETKEY  JSB  =GETTAKEY        ! WAIT FOR A KEY
         JSB  =GETKCODE        ! GET THE CODE
         STBD R53,=KEYSTS      !
         LDBD R53,=SVCWRD      ! GET PENDING INTERRUPTS
         JOD  K+ERR1           ! JIF KEYBOARD PENDING, ERROR
         RTN                   ! DONE
!
TGETKEYS JSB  =GETTAKEY        ! WAIT FOR A KEY
         STBD R53,=KEYSTS      ! 1 -> KEYSTS, ENABLE KEY SCANNING
         LDM  R66,=0,40        ! TIME-OUT COUNT
TGKLOP   DCM  R66              ! DEC COUNT
         JZR  TGKTO            ! JIF TIMED-OUT
         LDBD R27,=SVCWRD      ! GET INTERRUPT FLAGS
         JEV  TGKLOP           ! JIF NO KBD INTERRUPT, LOOP
TGKTO    LDBD R27,=KEYHIT      ! GET THE KEYCODE (HOPEFULLY)
         RTN                   ! DONE
!
JSBTK    JSB  =TK              ! CALL TK WITH AN EXTRA RTN ON STACK
         RTN                   ! DONE
!
TK       CLB  R60              ! CLEAR STUCK COUNTER
KINTLOP1 JSB  =LODTO           ! LOAD LONG/SHORT TIME-OUT
KINTLOP2 DCM  R65              ! DEC IT
         JNZ  L74215           ! JIF NO TIME-OUT
         LDBD R21,=KEYSTS      ! GET KEYBOARD STATUS
         LRB  R21              ! GET "KEY DOWN" FLAG INTO BIT0
         JOD  TKDOWN           ! JIF KEY IS DOWN (SHOULDN'T BE)
         TSB  R60              ! ANY STUCK?
         JZR  NOINC51          ! JIF NO
         ICB  R51              ! SO DAMNIT! BAILS OUT
NOINC51  JSB  =DAMNIT!         ! SPIT "NO KEY!
         JMP  TK               ! TRY AGAIN IF DAMNIT! COMES BACK
!
TKDOWN   JSB  =STUCKKEY        ! SPIT "KEY STUCK! " MSG
         ICB  R60              ! INC STUCK COUNTER
         JMP  KINTLOP1         ! TRY AGAIN
!
L74215   LDBD R41,=SVCWRD      ! GET INTERRUPT FLAGS
         JEV  KINTLOP2         ! JIF NO KBD PENDING
         LDBD R41,=KEYSTS      ! GET KEY STATUS
         LRB  R41              ! GET "KEY DOWN" INTO BIT0
         JMP  TGKTO            ! GET KEYCODE AND RETURN
!
DAMNIT!  LDB  R75,=15          ! CR
!
         BYT  162,251          ! LDM R62,=
         DEF  MSGNOKEY         ! "NO KEY!  "
         BYT  0,11             ! LEN 1, LEN 2
         DEF  MSGPKEY          ! "PRESS KEY"
!
L74244   STM  R62,R32          ! COPY MSG ARGS
         TSB  R52              !
         JNG  MSGBEP           ! JIF DISP STR & BEEP
         JRN  TOSSER1          ! JIF NO MSG, NO BEEP
         DCB  R51              !
         JNZ  JUST1STR         ! JIF JUST 1 STRING
         POMD R62,-R6          ! THROW AWAY 3 RTNs
         JMP  JMPJMP73         !
!
JUST1STR STB  R35,R34          ! COPY LEN2 TO LEN1
         CLB  R35              ! SET LEN2=0 (NO STR 2)
         JSB  =OUTBEEP-        ! DISPLAY STRING AND BEEP
         RTN                   ! DONE
!
STUCKKEY JSB  =GETKCODE        ! START KBD SCANNER
         LDB  R75,R27          ! GET TRAILING CHAR
!
         BYT  162,251          ! LDM R62,=
         DEF  MSGKSTUK         ! "KEY STUCK! "
         BYT  0,13             ! LEN 1, LEN 2
         DEF  MSGRELES         ! (DOESN'T MATTER, AS LEN1 IS 0)
!
         JMP  L74244           !
!
MSGBEP   JSB  =OUTBEEP         ! DISPLAY MSG, BEEP
TOSSER1  POMD R20,-R6          ! THROW AWAY 1 RTN
         CLB  R52              ! RESULTS=FAILED
GETKCODE LDBD R27,=KEYCOD      ! READ KEYCOD
         LDB  R53,=1           !
         STBD R53,=KEYCOD      ! RESTART KBD SCANNER
         TSB  R52              ! TEST RESULTS FOR RETURN
         RTN                   !
!
GKSHORT  LDB  R52,=1           ! SHORT WAIT, ENABLE KEY INTERRUPTS
GETTAKEY STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         JSB  =GETKCODE        ! FLUSH ANY PENDING KEY
         LLB  R53              ! 1 -> 2
         STBD R53,=KEYSTS      ! ENABLE KYD INTERRUPT
         LRB  R53              ! 2 -> 1
         JSB  =LODTO           ! GET SHORT/LONG COUNT
KEYWAIT5 DCM  R65              ! WAS 0,0,1 OR 0,0,5
         JNZ  CIFKEY           ! JIF NOT TIMED-OUT
         JSB  =DAMNIT!         ! TIMED-OUT, SPIT "NO KEY! PRESS KEY"
         JMP  GETTAKEY         ! KEEP WAITING
!
CIFKEY   LDBD R41,=KEYSTS      ! GET KEYSTS
         LRB  R41              ! GET "KEY DOWN" BIT
         JEV  KEYWAIT5         ! JIF NO KEY DOWN
         JSB  =DBOUNCE         ! WAIT FOR BOUNCING KEY
         LDBD R41,=KEYSTS      ! GET KEYSTS
         LRB  R41              ! GET "KEY DOWN" BIT
         JEV  GETTAKEY         ! JIF NO, START OVER
         LDB  R51,=2           !
WAITKYUP JSB  =LODTO           ! GET SHORT/LONG COUNT
WAITKYU- DCM  R65              ! DEC COUNT
         JNZ  STILWAIT         ! JIF NOT TOO LONG YET
         JSB  =STUCKKEY        ! REPORT STUCK KEY
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         JMP  WAITKYUP         ! LOOP
!
STILWAIT LDBD R21,=KEYSTS      ! GET KEYSTS
         LRB  R21              !
         JOD  WAITKYU-         ! JIF KEY DOWN
         TSB  R52              ! JUST RETURN THE KEY?
         JNZ  GETKCODE         ! JIF YES, RETURN THE KEYCODE
         LDB  R75,=101         ! 'A'
         STBD R53,=KEYSTS      ! ENABLE KEY SCANNING OR DISABLE KBD INTERRUPTS
         LLB  R53              !
         STBD R53,=KEYSTS      ! DISABLE KBD INTERRUPTS OR NOTHING
         STBD R53,=GINTEN      ! ENABLE GLOBAL INTERRUPTS
         LRB  R53              ! RESTORE R53
         LDBD R20,=SVCWRD      ! GET INTERRUPTS PENDING
         JEV  RTN74560         ! JIF NO KBD INTERRUPT
         POMD R20,-R6          ! THROW AWAY 1 RTN
         GTO K+ERR             !
!
! ************************************************************
! KYIDLE HOOK ENTRY POINTER
! ************************************************************
KYIDLENT LDBD R27,=KEYCOD      ! GET KEYCODE
         CMB  R27,=213         ! RESET KEY?
         JNZ  RTN74560         ! JIF NO
         CLB  R20              !
         STBD R20,=TAPSTS      ! TURN OFF TAPE SYSTEM
         LDBD R20,=ROMFL       ! GET MOST RECENT ROM INIT REASON
         JPS  RTN74560         ! JIF...
         GTO POWRON            ! GO BACK TO SYSTEM POWER-ON (START)
! ************************************************************
! TABLE OF ASCII NAMES FOR DISPLAYING "K1,K2,K3,K4" DURING KBD ERRORS
! ************************************************************
K1-K4    ASC  "K1"             !
         ASC  "K2"             !
         ASC  "K3"             !
         ASC  "K4"             !
!
LODTO    CLM  R65              ! CLEAR 3-BYTE COUNT
         ICB  R67              ! SET TO 0,0,1
         TSB  R52              ! WANT BIG COUNT?
         JOD  RTN74560         ! JIF NO
         LDB  R67,=5           ! SET TO 0,0,5
RTN74560 RTN                   ! DONE
!
! ************************************************************
! ARE THESE TWO BYTES USED????
! ************************************************************
         BYT  120              ! P?
         BYT  101              ! A?
!
CHKINTSJ JSB  =CHKINTS         !
         RTN                   !
!
! ************************************************************
! ARE THESE SIX BYTES USED????
! ************************************************************
         BYT  122              ! R
         BYT  125              ! U
         BYT  120              ! P
         BYT  123              ! S
         BYT  103              ! C
         BYT  116              ! N
! ************************************************************
! *** DELAY TO DEBOUNCE A KEY
! ************************************************************
DBOUNCE  LDM  R20,=71,2        ! DELAY COUNT
L74601   DCM  R#               ! DEC COUNT
         JNZ  L74601           ! LOOP TIL DONE
         RTN                   ! DONE
!
! ************************************************************
! ARE THESE SEVEN BYTES USED????
! ************************************************************
         BYT  114              ! L
         BYT  111              ! I
         BYT  120              ! P
         BYT  114              ! L
         BYT  114              ! L
         BYT  101              ! A
         BYT  101              ! A
! ************************************************************
! KEY FOR TEST 1
KEYLST1  BYT  141              ! A
!
! ************************************************************
! WHERE ARE THESE BYTES USED?????
! ************************************************************
         BYT  242              ! DOWN CURSOR
         BYT  254              ! AUTO (SHIFT)
         BYT  102              ! B
         BYT  123              ! S
         BYT  105              ! E
         BYT  116              ! N
         BYT  102              ! B
         BYT  101              ! A
         BYT  103              ! C
         BYT  74               ! <
         BYT  103              ! C
         BYT  76               !
         BYT  122              !
         BYT  136              !
         BYT  122              !
         BYT  166              !
         BYT  55               !
         BYT  114              !
         BYT  103              !
         BYT  136              !
         BYT  103              !
         BYT  166              !
         BYT  122              !
         BYT  120              !
         BYT  55               !
         BYT  103              !
!
! KEYS FOR TEST 4
KSEQ4    BYT  101              ! A
         BYT  200              ! K1
!
! KEYS FOR TEST 3
KSEQ3    BYT  141              ! a
         BYT  200              ! K1
!
! KEYS FOR TEST 2
KSEQ2    BYT  101              ! A
         BYT  200              ! K1
!
! KEYS FOR TEST 1
KSEQ1    BYT  1                ! CTL-A
         BYT  200              ! K1
!
! ************************************************************
! ARE THESE FOUR BYTES USED????
! ************************************************************
         BYT  123              ! S
         BYT  124              ! T
         BYT  114              ! L
         BYT  117              ! O
!
! ************************************************************
! THE FOLLOWING ARE THE "TEST STRUCTURES" FOR EACH OF THE
! NINE KEYBOARD TESTS (SEE COMMENTS IN FRONT OF 'TSTKBD').
! ************************************************************
KT1      DEF  TGETKEY          !
         DEF  MSGPRESS         ! "PRESS A"
         BYT  26,7             !
         DEF  MSGCTRL          ! "CTRL, CAPS, SHIFT UP; "
         BYT  1,200            !
         DEF  KEYLST1          !
!
KT2      DEF  TGETKEYS         !
         DEF  MSGPRESS         ! NOT DISPLAY (LEN 2=0)
         BYT  12,0             !
         DEF  MSGPRESS         ! "PRESS A K1"
         BYT  2,200            !
         DEF  KSEQ3            !
!
KT3      DEF  TGETKEYS         !
         DEF  MSGPRESS         ! NOT DISPLAYED (LEN 2=0)
         BYT  12,0             !
         DEF  MSGPRESS         ! "PRESS A K1"
         BYT  2,0              !
         DEF  KSEQ4            !
!
KT4      DEF  JSBTK            !
         DEF  MSGSPDWN         ! " DOWN; PRESS A K1"
         BYT  4,21             !
         DEF  MSGCAPS          ! "CAPS"
         BYT  2,200            !
         DEF  KSEQ3            !
!
KT5      DEF  JSBTK            !
         DEF  MSGSPUP          ! " UP; PRESS A K1"
         BYT  4,17             !
         DEF  MSGCAPS          ! "CAPS"
         BYT  2,0              !
         DEF  KSEQ4            !
!
KT6      DEF  JSBTK            !
         DEF  MSGSPDWN         ! " DOWN; PRESS A K1"
         BYT  4,21             !
         DEF  MSGCTRL          ! "CTRL"
         BYT  2,200            !
         DEF  KSEQ1            !
!
KT7      DEF  JSBTK            !
         DEF  MSGSPUP          ! " UP; PRESS A K1"
         BYT  4,17             !
         DEF  MSGCTRL          ! "CTRL"
         BYT  2,0              !
         DEF  KSEQ2            !
!
KT8      DEF  TGETKEY          !
         DEF  MSGPKEY          ! "PRESS KEYS"
         BYT  14,12            !
         DEF  MSGSHDWN         ! "SHIFT DOWN; "
         BYT  130,204          !
         DEF  KSEQSH           !
!
KT9      DEF  JSBTK            !
         DEF  MSGPKEY          ! "PRESS KEYS"
         BYT  12,12            !
         DEF  MSGSHFTU         ! "SHIFT UP; "
         BYT  130,0            !
         DEF  KSEQUNSH         !
!
! ************************************************************
! UNSHIFTED KEYS FOR TEST 9
! ************************************************************
KSEQUNSH BYT  226              ! KEY LABEL
         BYT  200              ! K1
         BYT  201              ! K2
         BYT  202              ! K3
         BYT  203              ! K4
         BYT  241              ! UP CURSOR
         BYT  242              ! DOWN CURSOR
         BYT  234              ! LEFT CURSOR
         BYT  235              ! RIGHT CURSOR
         BYT  243              ! INS/REP
         BYT  244              ! DEL CHR
         BYT  237              ! ROLL DOWN
         BYT  252              ! LOAD
         BYT  251              ! STORE
         BYT  212              ! PAPER ADV
         BYT  61               ! 1
         BYT  62               ! 2
         BYT  63               ! 3
         BYT  64               ! 4
         BYT  65               ! 5
         BYT  66               ! 6
         BYT  67               ! 7
         BYT  70               ! 8
         BYT  71               ! 9
         BYT  60               ! 0
         BYT  55               ! -
         BYT  75               ! =
         BYT  134              ! \
         BYT  231              ! BACKSPACE
         BYT  240              ! -LINE
         BYT  50               ! (
         BYT  51               ! )
         BYT  136              ! ^
         BYT  121              ! Q
         BYT  127              ! W
         BYT  105              ! E
         BYT  122              ! R
         BYT  124              ! T
         BYT  131              ! Y
         BYT  125              ! U
         BYT  111              ! I
         BYT  117              ! O
         BYT  120              ! P
         BYT  50               ! (
         BYT  51               ! )
         BYT  217              ! CONT
         BYT  67               ! 7
         BYT  70               ! 8
         BYT  71               ! 9
         BYT  57               ! /
         BYT  101              ! A
         BYT  123              ! S
         BYT  104              ! D
         BYT  106              ! F
         BYT  107              ! G
         BYT  110              ! H
         BYT  112              ! J
         BYT  113              ! K
         BYT  114              ! L
         BYT  73               ! ;
         BYT  47               ! '
         BYT  232              ! END LINE
         BYT  64               ! 4
         BYT  65               ! 5
         BYT  66               ! 6
         BYT  52               ! *
         BYT  132              ! Z
         BYT  130              ! X
         BYT  103              ! C
         BYT  126              ! V
         BYT  102              ! B
         BYT  116              ! N
         BYT  115              ! M
         BYT  54               ! ,
         BYT  56               ! .
         BYT  57               ! /
         BYT  224              ! LIST
         BYT  61               ! 1
         BYT  62               ! 2
         BYT  63               ! 3
         BYT  55               ! -
         BYT  40               ! SPACE
         BYT  216              ! PAUSE
         BYT  215              ! RUN
         BYT  60               ! 0
         BYT  56               ! .
         BYT  54               ! ,
         BYT  53               ! +
! ************************************************************
! SHIFTED KEYCODES FOR TEST 8
! ************************************************************
KSEQSH   BYT  140              ! SHIFT-KEY LABEL
         BYT  204              ! K5
         BYT  205              ! K6
         BYT  206              ! K7
         BYT  207              ! K8
         BYT  245              ! HOME CURSOR
         BYT  254              ! AUTO
         BYT  223              ! GRAPH
         BYT  211              ! COPY
         BYT  243              ! INS/RPL
         BYT  250              ! DELETE
         BYT  236              ! ROLL UP
         BYT  210              ! REW
         BYT  221              ! TEST
         BYT  212              ! PAPER ADV
         BYT  41               ! !
         BYT  100              ! @
         BYT  43               ! #
         BYT  44               ! $
         BYT  45               ! %
         BYT  136              ! ^
         BYT  46               ! &
         BYT  52               ! *
         BYT  50               ! (
         BYT  51               ! )
         BYT  137              ! _
         BYT  53               ! +
         BYT  174              ! |
         BYT  233              ! FAST BACKSPACE
         BYT  222              ! CLEAR SCREEN
         BYT  213              ! RESET
         BYT  214              ! INIT
         BYT  246              ! RESULT
         BYT  161              ! q
         BYT  167              ! w
         BYT  145              ! e
         BYT  162              ! r
         BYT  164              ! t
         BYT  171              ! y
         BYT  165              ! u
         BYT  151              ! i
         BYT  157              ! o
         BYT  160              ! p
         BYT  133              ! [
         BYT  135              ! ]
         BYT  255              ! SCRATCH
         BYT  46               ! &
         BYT  52               ! *
         BYT  50               ! (
         BYT  173              ! SHIFT /
         BYT  141              ! a
         BYT  163              ! s
         BYT  144              ! d
         BYT  146              ! f
         BYT  147              ! g
         BYT  150              ! h
         BYT  152              ! j
         BYT  153              ! k
         BYT  154              ! l
         BYT  72               ! :
         BYT  42               ! "
         BYT  232              ! END LINE
         BYT  44               ! $
         BYT  45               ! %
         BYT  136              ! ^
         BYT  176              ! SHIFT *
         BYT  172              ! Z
         BYT  170              ! X
         BYT  143              ! C
         BYT  166              ! V
         BYT  142              ! B
         BYT  156              ! N
         BYT  155              ! M
         BYT  74               ! <
         BYT  76               ! >
         BYT  77               ! ?
         BYT  225              ! PLIST
         BYT  41               ! !
         BYT  100              ! @
         BYT  43               ! #
         BYT  175              ! SHIFT -
         BYT  40               ! SPACE
         BYT  220              ! STEP
         BYT  215              ! RUN
         BYT  51               ! )
         BYT  76               ! >
         BYT  74               ! <
         BYT  177              ! SHIFT +
! ************************************************************
! TIMER TEST - activates the test mode in the keyboard
! controller, which contains the four timers. In test mode,
! the keyboard controller uses the keyboard scan to run the
! timers. Because of this, pressing a key during the Timer Test
! aborts the test. The Timer Test is begun about 3 seconds
! after it is selected. Therefore, be sure that all keys are
! released by that time.
! POSSIBLE RESULTS:
!   TIMER OK
!   TIMER BAD!
!   TIMER TEST ABORTED!
! ************************************************************
         BYT  241              ! ATTRIBUTES
TIMER.   BIN                   !
         JSB  =INITCLKS        ! INIT CLOCKS
         STBD R#,=GINTEN       ! ENABLE GLOBAL INTERRUPTS
         JSB  =CLRTINTS        ! CLEAR ANY PENDING INTERRUPTS
         JSB  =CHK0=0          ! ERROR IF ANY CLK COUNT IS NOT 0
         LDM  R24,=2,2         ! USED BY CKTMRS TO CHK SVCWRD
!
! NOTE: INITIALLY, WE'RE CHECKING TIMER 0, SO THESE BITS
! WON'T MATCH UP WITH SVCWRD, AS THEY'D BE CHECKING FOR
! AN I/O INTERRUPT. BUT AFTER GONXTCLK, THEY'LL BE ALIGNED
! FOR THE APPROPRIATE CLOCK INTERRUPT BIT TO TEST IN CKTMRS.
!
         LDM  R26,=44,72       ! TWO CLKSTS VALUES
!
! R26=44:
!    CLEAR INTERRUPT SERVICE FLIP-FLOP
!    STOP TIMER
! R27=72:
!    CLEAR INTERRUPT SERVICE FLIP-FLOP
!    CLEAR TIMER
!    START TIMER
!    ENABLE TIMER
!
CLKLOOP1 LDB  R50,=376         ! INITIAL TIME-OUT COUNT (IMMEDIATELY GETS INCREASED TO 007)
         CLM  R54              !
         LDB  R54,=6           ! R54 = CLOCK 0 COUNT VALUE
CLKLOOP2 JSB  =SETCLK          ! SET CLOCK COUNT TO 6
         LDB  R23,=14          ! START TIMER + STOP TIMER = TEST MODE
         STBD R23,=CLKSTS      ! PUT CLOCK IN TEST MODE
         ADB  R50,=11          ! INCREASE TIME-OUT COUNT FOR CLK READY
         STB  R50,R23          ! COPY TIME-OUT COUNT FOR CLK READY
         JSB  =WCLK            ! WAIT FOR CLOCK DATA READY
         TSB  R54              ! DOING CLOCK 0?
         JZR  GONXTCLK         ! JIF NO
         CMB  R27,=72          ! CLEARING/STARTING/ENABLING TIMER?
         JNZ  L75416           ! JIF NO
         JSB  =TSTCINT         ! ERROR IF CLK 0 DOESN'T INTERRUPT
         JSB  =CLRTINTS        ! CLEAR THE INTERRUPT FLAGS
         JMP  L75421           !
!
L75416   JSB  =CKTMRS          ! ERROR IF CLOCK INTERRUPTED (KEY PRESSED?)
L75421   TSB  R57              ! CLOCK COUNTED ALL THE WAY?
         JNG  TMROVF           ! JIF YES, MSdigit==9
         BCD                   ! BCD MODE
!
! NOTE: MODE IS LEFT IN BCD, BUT WILL GET SET BACK TO BIN
! WHEN SETCLK IS CALLED ABOVE, WHICH CALLS WAITRDY- WHICH SETS BIN
!
         LDB  R53,=90C         ! FOR SHIFTING INTO COUNT
         LLM  R53              ! SHIFT CLOCK COUNT, BRING IN 9 TO LSdigit
         JMP  CLKLOOP2         ! KEEP TESTING
!
TMROVF   STM  R54,R44          ! COPY TIMER VAL TO R44
         CLM  R54              ! SET TIMER VAL TO 0
         JMP  CLKLOOP2         ! GO AGAIN
!
GONXTCLK LLM  R24              ! SHIFT TEST BITS OVER FOR NEXT CLK
         STBD R26,=CLKSTS      ! CLEAR/STOP CURRENT CLK
         ADM  R26,=100,100     ! MOVE TO NEXT CLK
         JNC  CLKLOOP1         ! JIF DIDN'T MOVE PAST CLK 3, KEEP TESTING
!
         JSB  =CKNOTMRI        ! ERROR IF ANY TIMERS INTERRUPTED (KEY PRESSED)
         JSB  =CHK0=44         ! ERROR IF CLK0 COUNT DOESN'T MATCH R44
         LDM  R54,=67C,1,0,0   ! SET CLOCK 0 TO 167
         JSB  =SETCLK+         !
         LDB  R27,=12          ! START TIMER, ENABLE TIMER
         JSB  =ALLCLKS         !
         CLM  R44              ! COUNT VAL==0
L75504   JSB  =CHK0=44         ! DISABLE TIMERS AND CHECK COUNT = R44
         BCD                   !
         ICM  R44              ! INCREMENT COUNT
         BIN                   !
         CMM  R44,R54          ! REACHED LIMIT?
         JNZ  L75504           ! JIF NO, KEEP CHECKING
!
         JSB  =CHK0=0          ! ERROR IF CLK0 COUNT != 0
         JSB  =TSTCINT         ! ENABLE INTERRUPTS AND ERROR IF NO CLK0 INTERRUPT
         LDB  R25,=34          ! TIMERS 1,2,3 IN SVCWRD
         JSB  =CKTMR123        ! ERROR IF CLKS 1,2,3 DIDN'T INTERRUPT
         LDB  R23,=300         !
         JSB  =WCLK            ! WAIT FOR 300 !READY/READY CYCLES
         LDB  R27,=44          ! CLEAR INT FLAG, STOP TIMER
         JSB  =ALLCLKS         !    FOR ALL CLKS
         JSB  =CKNOTMRI        ! ERROR IF ANY TIMER INTERRUPTS
         LDB  R27,=32          ! CLEAR, START, ENABLE TIMER 0
         CLM  R54              !
         LDB  R54,=10C         ! COUNT = 10
CNTCLKS  JSB  =SETCLK          ! SET AND START CLK
         BCD                   !
         SBB  R54,=2           ! COUNT -= 2
         BIN                   !
         ADB  R27,=100         ! MOVE TO NEXT CLOCK
         JNC  CNTCLKS          ! JIF NOT FINISHED WITH ALL 4
!
         LDB  R27,=302         ! CLK 3, ENABLE
L75600   JSB  =CTLCHK44        ! CHECK COUNT
         ADB  R44,=2           ! BUMP COUNT
         SBB  R27,=100         ! DEC TIMER#
         JCY  L75600           ! JIF NOT FINISHED WITH ALL TIMERS
!
         LDB  R27,=305         ! CLK 3, STOP TIMERS, DISABLE TIMERS
         LDB  R25,=20          ! TIMER 3 SVCWRD SERVICE PENDING
L75620   JSB  =STOPTIMR        ! STOP TIMER, CHECK
         LRB  R25              ! SHIFT TO NEXT LOWER TIMER SVCWRD FLAG
         JEV  L75620           ! DO ALL 4 TIMERS
!
         JSB  =CLRTINTS        ! CLEAR PENDING INTERRUPTS
         LDB  R27,=10          ! START TIMER
         JSB  =ALLCLKS         ! TELL ALL CLKS
         JSB  =WCLK120         ! WAIT 120 CLK NOT/IS READY CYCLES
         JSB  =CKNOTMRI        ! ERROR IF ANY INTERRUPTS
         LDM  R32,=MSGSPOK     ! " OK"
         JSB  =OFFTIMRS        ! TURN OFF TIMERS, LOAD "TIMER" MSG
         JSB  =PROMPT          ! SPIT "TIMER OK"
         RTN                   ! DONE
!
! ************************************************************
! TURN OFF ALL TIMERS, CLEAR ALL TIMER INTERRUPTS, PRE-LOAD
! "TIMER" MESSAGE.
! ************************************************************
OFFTIMRS LDM  R36,=MSGTIMER    ! "TIMER"
         LDM  R34,=5,6         ! LEN 1, LEN 2
         STBD R34,=GINTDS      ! DISABLE INTERRUPTS
         LDB  R20,=34          ! TIME-OUT VALUE
WCREADY  DCB  R20              ! DEC COUNT
         JZR  CTIMOUT          ! JIF TIMED-OUT
         LDBD R22,=CLKSTS      ! WAIT FOR CLOCK READY
         JPS  WCREADY          ! JIF NOT READY
CTIMOUT  LDB  R27,=65          ! CLEAR INTERRUPT, CLEAR TIMER, STOP TIMER, DISABLE TIMER
         JSB  =CLKCTLLP        ! DO THAT TO ALL TIMERS
         JSB  =CLRTINTS        ! CLEAR ALL SVCWRD PENDING TIMER INTERRUPTS
         RTN                   !
!
! ************************************************************
! WAIT FOR *VERY* START OF "TIMER DATA READY"
! ************************************************************
WAITTRDY LDB  R75,=101         ! 'A' ERROR LETTER
WATTRDY- BIN                   !
         STBD R#,=GINTDS       ! DISABLE INTERRUPTS
         LDM  R20,=50,10       ! TIME-OUT COUNT
TMRBSY   DCB  R20              ! DEC TIME-OUT
         JZR  L76052           ! JIF TIMED-OUT
         LDBD R22,=CLKSTS      ! GET CLOCK STATUS
         JNG  TMRBSY           ! JIF TIMER DATA READY
TMRBSY2  DCB  R21              ! DEC TIME-OUT #2
         JZR  L76050           ! JIF TIMED-OUT
         LDBD R22,=CLKSTS      ! GET CLOCK STATUS
         JPS  TMRBSY2          ! JIF TIMER DATA NOT READY
         RTN                   ! DONE
!
! ************************************************************
! VARIOUS ENTRIES:
!   CHK0=0   CHECKS THAT TIMER 0 DATA==0
!   CHK0=44  CHECKS THAT TIMER 0 DATA==R44
!   CTLCHK44 CTRLS TIMER AND CHECKS THAT DATA==R44 (FOR 1 OR ALL TIMERS, DEPENDING UPON R27)
! ************************************************************
CHK0=0   CLM  R44              ! DESIRED TIMER VALUE
CHK0=44  CLB  R27              ! ADDRESS TIMER 0
CTLCHK44 JSB  =WATTRDY-        ! WAIT FOR TIMER READY
CTLCHK-  STBD R27,=CLKSTS      ! CTRL TIMER
         CMMD R44,=CLKDAT      ! TIMER DATA RIGHT?
         JNZ  TMVALBAD         ! JIF NO, ERROR
         TSB  R27              ! ENABLING/DISABLING TIMER?
         JRN  RTN76014         ! JIF YES, ONLY DO THE ONE
         ADB  R27,=100         ! ELSE MOVE TO NEXT TIMER
         JNC  CTLCHK-          ! DO ALL 4 OF THEM
RTN76014 RTN                   !
!
TMVALBAD CLE                   ! ARE WE IN BCD OR BIN????
         LDB  R26,=1           ! 26=1
         ELM  R26              ! 2*26
         ELM  R26              ! 4*26
         ELM  R26              ! 8*26
         JEZ  L76030           !
         JNO  L76032           !
L76030   SBB  R#,=4            ! 26 = ?
L76032   ADB  R75,R26          ! ERROR CHAR BASED ON TIMER #???
CLKER+5  ICB  R75              !
CLKER+4  ICB  R75              !
CLKER+3  ICB  R75              !
CLKER+2  ADB  R75,=2           !
         JMP  L76055           !
!
L76050   ICB  R75              ! ++ ERROR LETTER
L76052   POMD R20,-R6          ! THROW AWAY 1 RTN
L76055   POMD R20,-R6          ! THROW AWAY 1 RTN
         JSB  =OFFTIMRS        !
         STBD R#,=GINTEN       ! ENABLE INTERRUPTS
         LDBD R21,=KEYSTS      ! GET KEYBOARD STATUS
         ANM  R21,=12          ! SHIFT OR KEY DOWN?
         JNZ  KDNFAIL          ! JIF YES, FAIL
         JSB  =DBOUNCE         ! DEBOUNCE THE KEY NOT DOWN
         LDBD R32,=SVCWRD      ! GET PENDING INTERRUPTS
         JOD  KDNFAIL          ! JIF KBD PENDING
         LDM  R32,=MSGBAD!     ! " BAD! "
         JMP  KBDMSG           !
!
KDNFAIL  LDM  R32,=MSGSPABO    ! " TEST ABORTED! "
         LDB  R35,=17          ! LEN 2
KBDMSG   JSB  =OUTBEEP         ! OUTPUT THE MESSAGE AND BEEP
         RTN                   ! DONE
!
INITCLKS LDB  R52,=1           ! JUST RETURN ANY DOWN KEY, NO ERROR MESSAGE
         JSB  =WAITKYUP        ! WAIT FOR ANY KEY TO BE RELEASED
         JSB  =FLUSHKBD        ! FLUSH ANY PENDING KEYS
         LDM  R26,=0,1         ! WANT DISABLED TIMERS, CMD=DISABLE TIMERS
CLKENDIS JSB  =WAITTRDY        ! WAIT FOR TIMER DATA READY
         JSB  =CLKCTLLP        ! WRITE CLKSTS FOR ALL 4 TIMERS (1=DISABLE, 0=ENABLE)
         LDBD R23,=CLKSTS      ! GET CLK STATUS
         ANM  R23,=17          ! KEEP ONLY THE 4 TIMER ENABLED BITS
         CMB  R23,R26          ! WHAT WE WANT?
         JNZ  CLKER+2          ! JIF NO, ERROR
         ADM  R26,=17,25       ! WANT ENABLED TIMERS, CMD=CLEAR TIMER, STOP TIMER, ENABLE TIMER
         JOD  CLKENDIS         ! JIF DISABLING
!
TSTTLEN  JSB  =WAITTRDY        ! WAIT FOR START OF TIMER-READY
         LDB  R20,=30          ! INIT COUNTER
CLKREADY DCB  R20              ! DEC COUNTER
         JZR  CLKER+3          ! JIF TIMED-OUT
         LDBD R21,=CLKSTS      ! GET CLOCK STATUS
         JNG  CLKREADY         ! JIF STILL READY
         CMB  R20,=4           ! TOO SHORT?
         JCY  CLKER+4          ! JIF YES, ERROR
         RTN                   ! DONE
!
TSTCINT  LDMD R65,=DATE        ! GET DATE
         STBD R65,=GINTEN      ! ENABLE GLOBAL INTERRUPTS
DATECHG? CMMD R65,=DATE        ! DATE THE SAME?
         JZR  CLKER+5          ! JIF YES, SHOULD HAVE BEEN UPDATED
RTN76235 RTN                   ! DONE
!
STOPTIMR JSB  =WATTRDY-        ! WAIT FOR CLK READY
         LDMD R65,=DATE        ! GET DATE
         STBD R27,=GINTEN      ! ENABLE GLOBAL INTERRUPTS
         STBD R27,=CLKSTS      ! 305 = TIMER 3, STOP, DISABLE
         JLZ  DATECHG?         ! JIF JUST WROTE CLK 0 CTRL
         SBB  R27,=100         ! NEXT LOWER # TIMER
         JMP  CKTMR123         !
!
CKNOTMRI CLB  R25              ! NO TIMER INTERRUPTS SHOULD BE PENDING
CKTMR123 LDB  R24,=34          ! CARE ABOUT TIMERS 3,2,1
CKTMRS   STBD R#,=GINTEN       ! ENABLE GLOBAL INTERRUPTS
         LDBD R23,=SVCWRD      ! GET PENDING INTS
         ANM  R23,R24          ! ONLY KEEP ONES OF INTEREST
         XRB  R23,R25          ! TIMER INTS IN DESIRED STATES?
         JZR  CLRTINTS         ! JIF YES
SETCLKER ICB  R75              ! BUMP ERROR CHAR
         LRB  R23              ! SHIFT OVER
         JNC  SETCLKER         ! UNTIL WE GET THE BAD TIMER
         GTO CLKER+3           ! SPIT THE ERROR MESSAGE
!
CLRTINTS LDB  R20,=343         ! CLEAR TIMER INTERRUPT BITS
CLRINTS  STBD R21,=GINTDS      ! DISABLE INTERRUPTS
         LDBD R21,=SVCWRD      ! GET INTERRUPT FLAGS
         ANM  R21,R20          ! CLEAR ALL BUT DESIRED BIT
         STBD R21,=SVCWRD      ! RE-STORE
         JNZ  CINTSRTN         ! JIF THAT INT PENDING
         ANM  R17,=357         ! CLEAR 'SVC REQ' BIT IN XCOM
CINTSRTN RTN                   !
!
! ************************************************************
! ENTRY:
!   R27 = CTL BYTE TO SEND TO ALL 4 CLOCKS
! ************************************************************
ALLCLKS  JSB  =WATTRDY-        ! WAIT FOR TIMER TIMER READY
CLKCTLLP STBD R27,=CLKSTS      ! SEND CTL BYTE TO CLOCK
         ADB  R27,=100         ! INC CLOCK#
         JNC  CLKCTLLP         ! JIF UNTIL CLOCKS 0-3 ALL DONE
         RTN                   ! DONE
!
! ************************************************************
! AT ENTRY, R54-57 = CLOCK COUNT VALUE
! ************************************************************
SETCLK+  CLB  R27              ! CLOCK 0,
SETCLK   JSB  =WATTRDY-        ! WAIT FOR CLK READY
SETCLK-  STBD R27,=CLKSTS      ! WRITE CMD (ADDRESS TIMER)
         STMD R54,=CLKDAT      ! WRITE CLOCK DATA TO THAT CLK
         TSB  R27              ! ENABLING,DISABLING,STARTING,STOPPING?
         JRN  RTN76404         ! JIF YES, ONLY ONE TIMER
         ADB  R27,=100         ! ELSE MOVE ON TO NEXT TIMER
         JNC  SETCLK-          ! DO THEM ALL
RTN76404 RTN                   ! DONE
!
WCLK120  LDB  R23,=120         ! 120 CYCLES
WCLK     JSB  =WATTRDY-        ! WAIT FOR CLK DATA NOT READY, CLK DATA READY
         DCB  R23              ! DEC COUNT
         JNZ  WCLK             ! LOOP TIL DONE
         RTN                   !
!
TSTTLENJ JSB  =TSTTLEN         ! GET AN EXTRA JSB ON THE STACK FOR ERRORS
         RTN                   !
! ************************************************************
! BEEPER TEST - causes the beeper to sound three short tones
! of the same frequency, followed by a scale of eight longer
! tones of increasing frequency.
! ************************************************************
         BYT  241              !
BEEPR.   JSB  =STDBEEP         !
         JSB  =STDBEEP         ! 3 STANDARD BEEPS
         JSB  =STDBEEP         !
!
         LDM  R#,=40,0         ! (DRP 20) BEEPER ON/OFF VALUES FOR KEYSTS
!
! *** 'CPUVAL' IS THE ADDRESS OF A LIST OF 8 COUNT VALUES
! *** OF DECREASING AMOUNTS (THUS, INCREASING THE FREQUENCY
! *** OF THE TONE DUE TO THE SHORTER CYCLE TIME). THE VALUES
! *** ARE: 311,262,235,224,203,163,146,136
!
         LDM  R36,=CPUVAL      ! POINT TO FREQ COUNTERS
NXTTONE  LDM  R24,=200,0       ! TONE DURATION COUNTER
         POBD R26,+R36         ! GET NEXT TONE FREQ COUNT
OFFTONE  STBD R21,=KEYSTS      ! TURN *OFF* BEEPER
         LDB  R30,R26          ! COPY TONE FREQ COUNT
FREQLOOP DCB  R#               ! DEC TONE FREQ COUNT
         JCY  FREQLOOP         ! JIF >= 0, LOOP
         STBD R20,=KEYSTS      ! TURN *ON* BEEPER
         LDB  R30,=6           ! DELAY COUNTER
ONLOOP   DCB  R#               ! DEC COUNTER
         JCY  ONLOOP           ! LOOP WHILE >= 0
         DCM  R24              ! DEC DURATION COUNTER
         JCY  OFFTONE          ! JIF WHILE >= 0
!
         STBD R21,=KEYSTS      ! TURN *OFF* BEEPER
         CMM  R36,=MSGBPR      ! END OF TONE LIST?
         JNZ  NXTTONE          ! JIF NO, LOOP, DO NEXT TONE
!
         BYT  144,251          ! LDM R44,=
         DEF  MSGTEND          ! " TEST ENDS"
         BYT  6,12             ! LEN 1, LEN 2
!
! *** R36 (MSG 1) IS LEFT POINTING TO MSGBPR ("BEEPER")
! *** AFTER THE ABOVE TEST LOOPS, SO THE FOLLOWING CALL
! *** WILL PRINT "BEEPER TEST ENDS".
!
         JSB  =OUTLIN+S        ! SPIT MSG
         RTN                   ! DONE
!
! ************************************************************
! * GIVE A SHORT STANDARD BEEP
! ************************************************************
STDBEEP  BIN                   ! OUR FAVORITE MODE
         STBD R20,=GINTDS      ! DISABLE INTERRUPTS
         LDB  R20,=100         ! STANDARD 1.2 Khz BEEP
STDBEPOF STBD R#,=KEYSTS       ! TURN IT ON
         LDM  R22,=0,20        ! BEEP DURATION COUNTER
STDBEPLP DCM  R#               ! DEC DUR COUNTER
         JNZ  STDBEPLP         ! LOOP TIL DONE
         SBB  R20,=100         ! TOGGLE THE ON/OFF BIT
         JZR  STDBEPOF         ! IF ZERO, TURN IT OFF
         RTN                   ! DONE
! ************************************************************
! THE FOLLOWING 8 BYTES ARE USED FOR BOTH THE CPU TEST AND
! THE BEEPER "8 TONES OF INCREASING FREQUENCIES" TEST.
!
CPUVAL   BYT  311              !
         BYT  262              !
         BYT  235              !
         BYT  224              !
         BYT  203              !
         BYT  163              !
         BYT  146              !
         BYT  136              !
!
MSGBPR   ASC  "B"              !
         ASC  "E"              !
         ASC  "E"              !
         ASC  "P"              !
         ASC  "E"              !
MSGRECRD ASC  "R"              !
         ASC  "E"              !
         ASC  "C"              !
         ASC  "O"              !
         ASC  "R"              !
         ASC  "D"              !
MSGSVROM ASP  "S"              !
         ASP  "E"              !
         ASP  "R"              !
         ASP  "V"              !
         ASP  "I"              !
         ASP  "C"              !
         ASP  "E"              !
         ASP  " "              !
         ASP  "R"              !
         ASP  "O"              !
         ASP  "M"              !
         ASC  ":"              !
         ASC  " "              !
         ASC  "S"              !
         ASC  "E"              !
         ASC  "L"              !
         ASC  "E"              !
         ASC  "C"              !
         ASC  "T"              !
MSGYNRTN ASC  " "              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "T"              !
         ASC  "?"              !
         ASC  " "              !
         ASC  "Y"              !
         ASC  ","              !
         ASC  " "              !
         ASC  "N"              !
         ASC  ","              !
         ASC  " "              !
         ASC  "O"              !
         ASC  "R"              !
         ASC  " "              !
MSGRTNSY ASC  "R"              !
         ASC  "E"              !
         ASC  "T"              !
         ASC  "U"              !
         ASC  "R"              !
         ASC  "N"              !
         ASC  " "              !
         ASC  "T"              !
         ASC  "O"              !
         ASC  " "              !
         ASC  "B"              !
         ASC  "A"              !
         ASC  "S"              !
         ASC  "I"              !
         ASC  "C"              !
         ASC  " "              !
MSGSYSTM ASC  "S"              !
         ASC  "Y"              !
         ASC  "S"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  " "              !
MSGNOKEY ASC  "N"              !
         ASC  "O"              !
         ASC  " "              !
         ASC  "K"              !
         ASC  "E"              !
         ASC  "Y"              !
         ASC  "!"              !
         ASC  " "              !
         ASC  " "              !
MSGCPUBA ASC  "C"              !
         ASC  "P"              !
         ASC  "U"              !
         ASC  " "              !
         ASC  "B"              !
         ASC  "A"              !
         ASC  "D"              !
         ASC  "!"              !
         ASC  " "              !
         ASC  "N"              !
         ASC  " "              !
         ASC  "O"              !
         ASC  "R"              !
         ASC  " "              !
MSGCPUOK ASC  "C"              !
         ASC  "P"              !
         ASC  "U"              !
MSGSPOK  ASC  " "              !
MSGOK    ASC  "O"              !
         ASC  "K"              !
         ASC  " "              !
         ASC  " "              !
MSGHLOOP ASC  " "              !
         ASC  "H"              !
MSGREAD  ASC  "R"              !
         ASC  "E"              !
         ASC  "A"              !
         ASC  "D"              !
         ASC  " "              !
MSGXROMA ASC  "X"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "M"              !
MSGSPABO ASC  " "              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "A"              !
         ASC  "B"              !
         ASC  "O"              !
         ASC  "R"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "D"              !
         ASC  "!"              !
         ASC  " "              !
MSGLOAD  ASC  "L"              !
         ASC  "O"              !
         ASC  "A"              !
         ASC  "D"              !
         ASC  " "              !
         ASC  "R"              !
         ASC  "E"              !
         ASC  "W"              !
         ASC  "O"              !
         ASC  "U"              !
         ASC  "N"              !
         ASC  "D"              !
MSGLDUNP ASC  "L"              !
         ASC  "O"              !
         ASC  "A"              !
         ASC  "D"              !
MSGUNPRO ASC  " "              !
MSGUNPR- ASC  "U"              !
         ASC  "N"              !
MSGPROTE ASC  "P"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "C"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "D"              !
MSGTPK   ASC  " "              !
         ASC  "T"              !
         ASC  "A"              !
         ASC  "P"              !
         ASC  "E"              !
         ASC  ","              !
         ASC  " "              !
MSGPKEY  ASC  "P"              !
         ASC  "R"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "S"              !
         ASC  " "              !
         ASC  "K"              !
         ASC  "E"              !
         ASC  "Y"              !
MSGSPEED ASC  "S"              !
         ASC  "P"              !
         ASC  "E"              !
         ASC  "E"              !
         ASC  "D"              !
MSGORXRA ASC  " "              !
         ASC  "O"              !
         ASC  "R"              !
         ASC  " "              !
         ASC  "X"              !
MSGRAMBA ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "B"              !
         ASC  "A"              !
         ASC  "D"              !
         ASC  "!"              !
MSGA-V   ASC  " "              !
         ASC  "A"              !
         ASC  "-"              !
         ASC  "V"              !
MSGXROM  ASC  "X"              !
MSGROM   ASC  "R"              !
         ASC  "O"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "0"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "1"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "2"              !
MSGROM3  ASC  "R"              !
         ASC  "O"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "3"              !
MSGNOXRA ASC  "N"              !
         ASC  "O"              !
         ASC  " "              !
MSGXRAM  ASC  "X"              !
MSGRAM   ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  " "              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "0"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "1"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "2"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "3"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "4"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "5"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "6"              !
         ASC  "X"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "7"              !
MSGCRT   ASC  "C"              !
         ASC  "R"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "0"              !
         ASC  "C"              !
         ASC  "R"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "1"              !
         ASC  "C"              !
         ASC  "R"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "2"              !
         ASC  "C"              !
         ASC  "R"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "M"              !
         ASC  " "              !
         ASC  "3"              !
MSGOUT   ASC  "O"              !
         ASC  "U"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "O"              !
         ASC  "F"              !
         ASC  " "              !
         ASC  "P"              !
         ASC  "A"              !
         ASC  "P"              !
         ASC  "E"              !
         ASC  "R"              !
MSGNOT   ASC  "N"              !
         ASC  "O"              !
         ASC  "T"              !
         ASC  " "              !
MSGREMOV ASC  "R"              !
         ASC  "E"              !
         ASC  "M"              !
         ASC  "O"              !
         ASC  "V"              !
         ASC  "E"              !
         ASC  "D"              !
MSGEOT   ASC  "E"              !
         ASC  "N"              !
         ASC  "D"              !
         ASC  " "              !
         ASC  "O"              !
         ASC  "F"              !
         ASC  " "              !
         ASC  "T"              !
         ASC  "A"              !
         ASC  "P"              !
         ASC  "E"              !
         ASC  "!"              !
         ASC  " "              !
MSGPROTP ASC  ","              !
         ASC  " "              !
         ASC  "P"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "C"              !
         ASC  "T"              !
         ASC  ","              !
         ASC  " "              !
         ASC  "L"              !
         ASC  "O"              !
         ASC  "A"              !
         ASC  "D"              !
         ASC  " "              !
MSGTAP1  ASC  "TAPE "          !
         ASC  "E"              !
         ASC  "R"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "R"              !
         ASC  "!"              !
         ASC  " "              !
MSGHOLE  ASC  "H"              !
         ASC  "O"              !
         ASC  "L"              !
         ASC  "E"              !
         ASC  " "              !
MSGPRTR  ASC  "P"              !
         ASC  "R"              !
         ASC  "I"              !
         ASC  "N"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "R"              !
MSGCTRLR ASC  " "              !
         ASC  "C"              !
         ASC  "O"              !
         ASC  "N"              !
         ASC  "T"              !
         ASC  "R"              !
         ASC  "O"              !
         ASC  "L"              !
MSGBAD!  ASC  " "              !
         ASC  "B"              !
         ASC  "A"              !
         ASC  "D"              !
         ASC  "!"              !
         ASC  " "              !
MSGHEAT  ASC  "H"              !
         ASC  "E"              !
         ASC  "A"              !
         ASC  "T"              !
MSGSPTST ASC  " "              !
MSGTEST  ASC  "T"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "B"              !
         ASC  "E"              !
         ASC  "G"              !
         ASC  "I"              !
         ASC  "N"              !
MSGSTATS ASC  "S"              !
         ASC  "T"              !
         ASC  "A"              !
         ASC  "T"              !
         ASC  "U"              !
MSGSLOW  ASC  "S"              !
         ASC  "L"              !
         ASC  "O"              !
MSGWRITE ASC  "W"              !
         ASC  "R"              !
         ASC  "I"              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  " "              !
MSGTRAKA ASC  "T"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "C"              !
         ASC  "K"              !
         ASC  " "              !
         ASC  "A"              !
MSG?     ASC  "?"              !
         ASC  " "              !
         ASC  "P"              !
         ASC  "R"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "S"              !
         ASC  " "              !
         ASC  "K"              !
         ASC  "E"              !
         ASC  "Y"              !
MSGTRAKB ASC  "T"              !
         ASC  "R"              !
         ASC  "A"              !
         ASC  "C"              !
         ASC  "K"              !
         ASC  " "              !
         ASC  "B"              !
         ASC  "?"              !
         ASC  " "              !
         ASC  "P"              !
         ASC  "R"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "S"              !
         ASC  " "              !
MSGKEY   ASC  "K"              !
         ASC  "E"              !
         ASC  "Y"              !
MSGTEND  ASC  " "              !
         ASC  "T"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "T"              !
         ASC  " "              !
         ASC  "E"              !
         ASC  "N"              !
         ASC  "D"              !
MSGSTALL ASC  "S"              !
         ASC  "T"              !
         ASC  "A"              !
         ASC  "L"              !
         ASC  "L"              !
         ASC  "E"              !
         ASC  "D"              !
         ASC  "!"              !
         ASC  " "              !
MSGCOLON ASC  ":"              !
MSGCTRL  ASC  "C"              !
         ASC  "T"              !
         ASC  "R"              !
         ASC  "L"              !
         ASC  ","              !
         ASC  " "              !
MSGCAPS  ASC  "C"              !
         ASC  "A"              !
         ASC  "P"              !
         ASC  "S"              !
         ASC  ","              !
         ASC  " "              !
MSGSHFTU ASC  "S"              !
         ASC  "H"              !
         ASC  "I"              !
         ASC  "F"              !
         ASC  "T"              !
MSGSPUP  ASC  " "              !
         ASC  "U"              !
         ASC  "P"              !
         ASC  ";"              !
         ASC  " "              !
         ASC  "P"              !
         ASC  "R"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "S"              !
         ASC  " "              !
         ASC  "A"              !
         ASC  " "              !
         ASC  "K"              !
         ASC  "1"              !
MSGGAPSP ASC  "G"              !
         ASC  "A"              !
         ASC  "P"              !
         ASC  " "              !
MSGSHDWN ASC  "S"              !
         ASC  "H"              !
         ASC  "I"              !
         ASC  "F"              !
         ASC  "T"              !
MSGSPDWN ASC  " "              !
         ASC  "D"              !
         ASC  "O"              !
         ASC  "W"              !
         ASC  "N"              !
         ASC  ";"              !
         ASC  " "              !
MSGPRESS ASC  "P"              !
         ASC  "R"              !
         ASC  "E"              !
         ASC  "S"              !
         ASC  "S"              !
         ASC  " "              !
         ASC  "A"              !
         ASC  " "              !
         ASC  "K"              !
         ASC  "1"              !
MSGFAST  ASC  "F"              !
         ASC  "A"              !
         ASC  "S"              !
MSGTIMER ASC  "T"              !
         ASC  "I"              !
         ASC  "M"              !
         ASC  "E"              !
MSGRELES ASC  "R"              !
         ASC  "E"              !
         ASC  "L"              !
         ASC  "E"              !
         ASC  "A"              !
         ASC  "S"              !
         ASC  "E"              !
         ASC  " "              !
MSGKSTUK ASC  "K"              !
         ASC  "E"              !
         ASC  "Y"              !
         ASC  " "              !
         ASC  "S"              !
         ASC  "T"              !
         ASC  "U"              !
         ASC  "C"              !
         ASC  "K"              !
         ASC  "!"              !
         ASC  " "              !
! ************************************************************
         BSZ  120              ! UNUSED BYTES
! ************************************************************
! ROM CHECKSUMS
! ************************************************************
         BYT  130,124,46,105   ! CHECKSUMS
         FIN
